/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes
 * Contributor(s): 
 *
 * --------------------------------------------------------------------------
 * $Id: JmxEntityFactory.java,v 1.4 2003/07/03 10:05:31 durieuxp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import org.objectweb.jonas_ejb.container.EntityCounters;
import org.objectweb.jonas_ejb.container.JEntityFactory;

/**
 * MBean Class for Entity EJB Management 
 * MBean type: Standard
 * MBean model: Delegate (JEntityFactory)
 */
public class JmxEntityFactory extends JmxBeanFactory implements JmxEntityFactoryMBean {

    /**
     * Constructor
     * @param factoryToManage JEntityFactory  to manage
     */
    public JmxEntityFactory(JEntityFactory factoryToManage) {
        super(factoryToManage);
    }

    /**
     * get passivation time out
     * @return passivation timeout in milliseconds
     */
    public Long getPassivationTimeOut(){
        return new Long(ejbToManage.getContainer().getSwapTime());
    }

    /**
     * set passivation time out
     * @param timeOut passivation timeout in seconds
     */
    public void setPassivationTimeOut(Long timeOut) {
        ejbToManage.getContainer().setSwapTime(timeOut.intValue());
    }

    /**
     * @return true if bean is shared
     */
    public Boolean getShared() {
        return new Boolean (((JEntityFactory) ejbToManage).isShared());
    }

    /**
     * @return min-pool-size value
     */
    public Integer getMinPoolSize() {
        return new Integer(((JEntityFactory) ejbToManage).getMinPoolSize());
    }

    /**
     * @return max-cache-size value
     */
    public Integer getMaxCacheSize() {
        return new Integer(((JEntityFactory) ejbToManage).getMaxCacheSize());
    }

    /**
     * @return pool-size value
     */
    public Integer getPoolSize() {
        return new Integer(((JEntityFactory) ejbToManage).getPoolSize());
    }

    /**
     * @return Cache Size value
     */
    public Integer getCacheSize() {
        return new Integer(((JEntityFactory) ejbToManage).getCacheSize());
    }

    /**
     * Instance Counters (inTx, outTx, idle, passive, removed)
     * @return table of int values for Entity counters
     */
    public Integer[] getEntityCounters() {
        EntityCounters ec = ((JEntityFactory) ejbToManage).getEntityCounters();
        Integer[] result = new Integer[5];
        result[0] = new Integer(ec.inTx);
        result[1] = new Integer(ec.outTx);
        result[2] = new Integer(ec.idle);
        result[3] = new Integer(ec.passive);
        result[4] = new Integer(ec.removed);
        return result;
    }

    /**
     * Synchronize bean state for all its instances outside transactions
     */
    public void synchronize() {
        ((JEntityFactory) ejbToManage).sync();
    }

    /**
     * Reduce number of instances in memory 
     */
    public void reduceCache() {
        ((JEntityFactory) ejbToManage).reduceCache();
    }
}
