/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes
 * Contributor(s): 
 *
 * --------------------------------------------------------------------------
 * $Id: JmxSblFactory.java,v 1.2 2003/05/28 14:08:07 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import org.objectweb.jonas_ejb.container.JStatelessFactory;

/**
 * MBean Class for Stateless Session EJB Management 
 * MBean type: Standard
 * MBean model: Delegate (JSessionFactory)
 */
public class JmxSblFactory extends JmxSessionFactory implements JmxSblFactoryMBean {
        
    /**
     * Constructor
     * @param JSessionFactory factory to manage
     */
    public JmxSblFactory(JStatelessFactory factoryToManage) {
        super(factoryToManage);
    }

    /**
     * MBean method for bean instances pool management.
     * @return the minimum size of the bean instances pool.
     */
    public Integer getMinPoolSize() {
        return new Integer(((JStatelessFactory)ejbToManage).getMinPoolSize());
    }

    /**
     * MBean method for bean instances pool management.
     * @return the maximum size of the bean instances pool
     */
    public Integer getMaxPoolSize() {
        return new Integer(((JStatelessFactory)ejbToManage).getMaxCacheSize());
    }
 
    /**
     * MBean method for bean instances pool management.
     * @return the current number of created bean instances
     */
    public Integer getNbInstances() {
        return new Integer(((JStatelessFactory)ejbToManage).getCacheSize());
    }

     /**
      * MBean method for bean instances pool management.
      * Allows reducing the number of instances.
      */
    public void reducePoolToMinSize() {
        ((JStatelessFactory)ejbToManage).reduceCache();
    }   
} 
