/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Eric HARDESTY
 * --------------------------------------------------------------------------
 * $Id: ConnectionImpl.java,v 1.4 2003/09/19 14:59:17 ehardesty Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jdbc;

import java.io.PrintWriter;
import java.util.Map;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning ;
import java.sql.Statement;

import javax.resource.ResourceException;

import org.objectweb.jonas.resource.MCInfo;
import org.objectweb.jonas.resource.SQLManager;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


/**
 * The class <b>Connection</b> provides the Connection
 * implementation for encapsulating JDBC connections
 */
public class ConnectionImpl 
    implements Connection {

    public Logger trace = null; 

    ManagedConnectionImpl mc = null;
    Connection connection = null;
    PrintWriter pw = null;
    long key = 0;
    String user = "";
    
    private SQLManager conman = null;
    private MCInfo mci = null;

    protected ConnectionImpl(ManagedConnectionImpl _mc, Connection _con, 
                              long _key, PrintWriter _pw) {
        mc = _mc;
        connection = _con;
        key = _key;
        pw = _pw;
        trace = mc.trace;
        try {
            user = mc.getMetaData().getUserName();
        } catch (Exception ex) {
        }
    }
    
    public void setJonasInfo(MCInfo _mci, SQLManager _conman) {
        mci = _mci;
        conman = _conman; 
    }
    
    // IMPLEMENTATION OF METHODS FROM THE java.sql.Connection INTERFACE //

    public synchronized void clearWarnings() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,"checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        connection.clearWarnings();
    }
    
    public synchronized void close() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            closeConnectionImpl();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,"error");
            e.printStackTrace();
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
    }
    
    public synchronized void commit() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
            connection.commit();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,"error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
    }
    
    public synchronized Statement createStatement()
               throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,"checkContext error", e,"ConnectionImpl"
                ,"createStatement");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.createStatement();
    }
    
    public synchronized Statement createStatement(int resultSetType,
                                     int resultSetConcurrency)
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.createStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection
            .createStatement(resultSetType, resultSetConcurrency);
    }
    
    public synchronized boolean getAutoCommit() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
            return connection.getAutoCommit();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,"ConnectionImpl.getAC error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
    }

    public synchronized String getCatalog() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.getCatalog: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.getCatalog();
    }
// JDK 1.4 SQL
    public synchronized int getHoldability()
               throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.getHoldability: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.getHoldability();
*/    }

    public synchronized DatabaseMetaData getMetaData()
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.getMetaData: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.getMetaData();
    }
    
    public synchronized int getTransactionIsolation()
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.getTransactionIsolation: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.getTransactionIsolation();
    }
    
    public synchronized Map getTypeMap() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.getTypeMap: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.getTypeMap();
    }
    
    public synchronized SQLWarning getWarnings() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.getWarnings: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.getWarnings();
    }
    
    public synchronized boolean isClosed() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.isClosed: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.isClosed();
    }
    
    public synchronized boolean isReadOnly() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.isReadOnly: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.isReadOnly();
    }
    
    public synchronized String nativeSQL(String sql) throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.nativeSQL: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.nativeSQL(sql);
    }
    
    public synchronized CallableStatement prepareCall(String sql) throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareCall: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.prepareCall(sql);
    }
    
    public synchronized CallableStatement prepareCall(String sql,
        int resultSetType, int resultSetConcurrency) throws SQLException {
        
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareCall: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.prepareCall(sql, resultSetType, resultSetConcurrency);
    }
    
    public synchronized PreparedStatement prepareStatement(String sql)
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        if (conman != null) {
            return conman.getPStatement(mci, connection, user, sql);
        } else {
            return connection.prepareStatement(sql);
        }
    }
    
    public synchronized PreparedStatement prepareStatement(String sql,
                                                           int rsettype,
                                                           int rsetconcurrency)
               throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        if (conman != null) {
            return conman.getPStatement(mci, connection, user, sql, rsettype, rsetconcurrency);
        } else {
            return connection.prepareStatement(sql, rsettype, rsetconcurrency);
        }
    }
    
    public synchronized void rollback() throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
            connection.rollback();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,e.getMessage());
            e.printStackTrace();
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
    }

// JDK 1.4 SQL
// Define thoses methods to be able to compile JOnAS in JDK 1.4
    public synchronized PreparedStatement prepareStatement(String sql,
                                                  int rsettype,
                                                  int rsetconcurrency,
                                                  int holdability)
               throws SQLException {
        throw new UnsupportedOperationException();
    }
    public synchronized PreparedStatement prepareStatement(String sql,
                                                           int[] t)
        throws SQLException {
        throw new UnsupportedOperationException();
    }
    public synchronized PreparedStatement prepareStatement(String sql,
                                                           String[] t)
        throws SQLException {
        throw new UnsupportedOperationException();
    }
    public synchronized PreparedStatement prepareStatement(String sql,
                                                           int autoGenKeys)
        throws SQLException {
        throw new UnsupportedOperationException();
    }

/*
    public synchronized PreparedStatement prepareStatement(String sql,
                                                  int rsettype,
                                                  int rsetconcurrency,
                                                  int holdability)
               throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.prepareStatement(sql, rsettype, 
                                            rsetconcurrency, holdability);
    }

//  JDK 1.4 SQL

    public synchronized PreparedStatement prepareStatement(String sql,
                                                  int[] t)
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.prepareStatement(sql, t);
    }

//  JDK 1.4 SQL
    public synchronized PreparedStatement prepareStatement(String sql,
                                                  String[] t)
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.prepareStatement(sql, t);
    }

//  JDK 1.4 SQL
    public synchronized PreparedStatement prepareStatement(String sql,
                                                  int autoGenKeys)
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.prepareStatement(sql, autoGenKeys);
    }

*/
//  JDK 1.4 SQL
    public synchronized CallableStatement prepareCall(String sql,
                                                  int rsettype,
                                                  int rsetconcurrency,
                                                  int holdability)
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.prepareCall(sql, rsettype, 
                                       rsetconcurrency, holdability);
*/    }

//  JDK 1.4 SQL
    public synchronized Statement createStatement(int rsettype,
                                                    int rsetconcurrency,
                                                    int holdability)
             throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.prepareStatement: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        return connection.createStatement(rsettype, rsetconcurrency, 
                                           holdability);
*/    }


//  JDK 1.4 SQL
    public synchronized void releaseSavepoint(java.sql.Savepoint sp)
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
            connection.releaseSavepoint(sp);
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,e.getMessage());
            e.printStackTrace();
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
*/    }

//  JDK 1.4 SQL
    public synchronized void rollback(java.sql.Savepoint sp) throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
            connection.rollback(sp);
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,e.getMessage());
            e.printStackTrace();
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
*/    }

//  JDK 1.4 SQL
    public synchronized java.sql.Savepoint setSavepoint()
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
            return connection.setSavepoint();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,e.getMessage());
            e.printStackTrace();
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
*/    }

//  JDK 1.4 SQL
    public synchronized java.sql.Savepoint setSavepoint(String name)
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
            return connection.setSavepoint(name);
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,e.getMessage());
            e.printStackTrace();
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
*/    }

    public synchronized void setAutoCommit(boolean isauto) throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
            connection.setAutoCommit(isauto);
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,"ConnectionImpl.setAC: error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
    }

    public synchronized void setCatalog(String catalog)
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.setCatalog: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        connection.setCatalog(catalog);
    }

// JDK 1.4 SQL
    public synchronized void setHoldability(int holdability)
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        throw new UnsupportedOperationException();
/*        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.setCatalog: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        connection.setHoldability(holdability);
*/    }

    public synchronized void setReadOnly(boolean readOnly)
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.setReadOnly: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        connection.setReadOnly(readOnly);
    }
    
    public synchronized void setTransactionIsolation(int level)
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.setTransactionIsolation: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        connection.setTransactionIsolation(level);
    }
    
    public synchronized  void setTypeMap(Map map)
        throws SQLException {
        trace.log(BasicLevel.DEBUG,"");
        try {
            checkContext();
        } catch (Exception e) {
            trace.log(BasicLevel.ERROR,
                "ConnectionImpl.setTypeMap: checkContext error");
            throw new SQLException("JOnAS JDBC: " + e.getMessage());
        }
        connection.setTypeMap(map);
    }

    private void closeConnectionImpl() throws ResourceException { 
        if (key != 0) {
            mc.close(this);
            key = 0;
        }
    }
    private void checkContext() throws Exception {
        if (key == 0) {
            throw new Exception("Connection is closed");
        }
        if (key != mc.getSignature()) {
            if (mc.getSignature() == 0) {
                mc.setSignature(key);
            } else {
                throw new Exception("Connection w/sig("+key+") is not currect active Connection ("+mc.getSignature()+")");
            }
        }
    }
    
    public void setSignature(long sig) {
        key = sig;
    }
}
