/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ManagementReprFactory.java,v 1.2 2003/09/18 15:44:24 danesa Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx;

//import java.util.Properties;
//import java.io.InputStream;

// JOnAS Log
import org.objectweb.jonas.common.Log;
// JOnAS
import org.objectweb.jonas.common.JProp;

// Monolog
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Provides an appropriate management representative: an instance of the class indicated in the Jadmin.properties file
 * @author Adriana Danes
 */
public class ManagementReprFactory {

    static String classNameForMx4j = "org.objectweb.jonas.jmx.Mx4jManagementReprImpl";
    static String classNameForSunri = "org.objectweb.jonas.jmx.ManagementReprImpl"; // default 
    static String implClassNameForMx4j = "org.objectweb.jonas.jmx.mx4j.Mx4jJmxServiceImpl";
    static String implClassNameForSunri = "org.objectweb.jonas.jmx.sunri.JmxServiceImpl";

    //static String propName = "Jadmin.ConnectorFactory.class";
    //static String propFileName = "Jadmin.properties";

    static private Logger logger = null; 

    public static ManagementRepr getManagementRepr() {
        logger = Log.getLogger("org.objectweb.jonas.jmx");
	
        ManagementRepr representative = null;
        String managemntRepClassName = null;
        ClassLoader classLoader = ManagementReprFactory.class.getClassLoader();

        /*
        Properties props = new Properties();
        InputStream input = null;
        try {
            //input = Thread.currentThread().getContextClassLoader().getResourceAsStream(propFileName);
            input = classLoader.getResourceAsStream(propFileName);
            props.load(input);
        } catch (Exception e) {
            logger.log(BasicLevel.DEBUG, "ManagementReprFactory , can't load Jadmin properties : " + e.toString());
        }
        */
        // get the jonas configuration parameter giving the JMX implementation class
        try {
            JProp props = JProp.getInstance();
            String myjmx =  props.getValue(AbsJmxServiceImpl.CLASS);
            if (myjmx.equals(implClassNameForSunri) == true) {
                managemntRepClassName = classNameForSunri;
            } else if (myjmx.equals(implClassNameForMx4j) == true) {
                managemntRepClassName = classNameForMx4j;
            } else {
                // no other possibility, take default
                managemntRepClassName = classNameForSunri;
            }
        } catch (Exception e) {
            logger.log(BasicLevel.DEBUG, "ManagementReprFactory exception : " + e.toString());
        }
        try {
            Class managemntRepClass = classLoader.loadClass(managemntRepClassName);
            logger.log(BasicLevel.DEBUG, "ManagementReprFactory , managemntRep class loaded");
            representative = (ManagementRepr)managemntRepClass.newInstance();
            logger.log(BasicLevel.DEBUG, "ManagementReprFactory , managemntRep created");
        } catch (Exception e) {
            logger.log(BasicLevel.DEBUG, "ManagementReprFactory exception : " + e.toString());
        }
        return representative;
    }
}
