/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id: MailService.java,v 1.4 2003/08/05 15:03:21 benoitf Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.mail;


//import java.util
import java.util.Properties;

//import jonas
import org.objectweb.jonas.service.Service;
import org.objectweb.jonas.mail.factory.JMail;


/**
 * This interface provides a description of the JavaMail service.
 * @author Ludovic Bert
 * @author Florent Benoit
 */
public interface MailService extends Service {

    /**
     * Create a mail factory with the specified properties and register it 
     * into the registry. The created object is an instance of one of the 2 classes :
     * JMailSession or 
     * @param name the mail factory name
     * @param props the properties used to configure the mail factory.
     * @throws MailServiceException if the creation or the registration of 
     * the factory failed.
     */
    void createMailFactory(String name, Properties props) throws MailServiceException;

    /**
     * This method is used when a Mail Factory configuration is modified via Jadmin.
     * In this case, the updated JMail object (JMailSession or JMailMimePartDS object)
     * must be rebound in JNDI
     * @param factory the factory
     * @throws MailServiceException if the recreation of 
     * the factory failed.
     */
    void recreateMailFactory(JMail factory) throws MailServiceException;

    /**
     * This method is used when a particular Mail Factory configuration operation is done via Jadmin : 
     * when the JNDI name of this resource is modified.
     * In this case, the initial JMail object (JMailSession or JMailMimePartDS object) must be unbound
     * and the updated JMail object must be reloaded.
     * Also, the Mail Service private data structures must be updated.
     * @param oldName old name of the factory
     * @param factory the new factory
     * @throws MailServiceException if the rename of the
     * the factory failed.
     */
    void renameMailFactory(String oldName, JMail factory) throws MailServiceException;

    /**
     * Unregister all the binding factories on the server.
     * @throws MailServiceException if the unregistration of the factories 
     * failed.
     */
    void unbindMailFactories() throws MailServiceException;

}
