/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConnectionManagerImpl.java,v 1.8 2003/09/18 17:07:34 ehardesty Exp $
 * --------------------------------------------------------------------------
 */
  

package org.objectweb.jonas.resource;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.AbstractSet;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javax.naming.Context;
import javax.naming.NamingException;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ResourceAllocationException;

import javax.security.auth.Subject;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.transaction.RollbackException;
import javax.transaction.Synchronization;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.xa.XAResource;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import org.objectweb.jonas.resource.pool.api.Pool;
import org.objectweb.jonas.resource.pool.api.PoolMatchFactory;
import org.objectweb.jonas.resource.pool.api.PoolProvider;
import org.objectweb.jonas.resource.pool.lib.HArrayPool;

import org.objectweb.transaction.api.ConnectionManager;

import org.objectweb.transaction.api.ResourceManagerEventListener;
import org.objectweb.transaction.api.ResourceManagerEvent;

import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;
/**
 *  Description of the ConnectionManagerImpl
 *
 *@author     chassand
 *@created    15 novembre 2001
 */
public class ConnectionManagerImpl 
    implements ConnectionEventListener, ConnectionManager, PoolMatchFactory,
                SQLManager {

    /**
     *  The Logger instance where messages are written.
     */
    protected Logger trace = null;
    protected Logger poolTrace = null;

    /**
     *  This variable is used by the server to register connections when no
     *  transactionnal context exists
     */
    protected ResourceManagerEventListener rmel = null;

    protected String resourceBundleName = null;
    
    /**
     *  The transaction manager in server
     */
    protected TransactionManager tm;

    /**
     *  This hashtable allows to find the list of connection handle associated to a
     *  ManagedConnection
     */
    protected Hashtable mc2mci = null;

    /**
     *  The max pool size of ManagedConnection. The default value is -1.
     */
    protected int mcMaxPoolSize = -1;

    /**
     *  The min pool size of ManagedConnection. The default value is 0
     */
    protected int mcMinPoolSize = 0;

    /**
     *  The ManagedConnectionFactory instance which represents the resource
     *  adapter.
     */
    protected ManagedConnectionFactory mcf;

    /**
     *  The pool of ManagedConnections associated to a ManagedConnectionFactory
     */
    protected Pool poolMCs = null;

    /**
     *  The list of used ManagedConnections key = transaction reference value =
     *  MCInfo
     */
    protected Hashtable usedMCs = null;

    /**
     *  The default max pool size of pstmts per ManagedConnection.
     */
    final static int MAX_PSTMT_SIZE = 10;

    /** These are the PrepareStatement pooling members
     * 
     */
    protected int maxPstmtPoolSize = MAX_PSTMT_SIZE;
    protected Vector vPstmts = null; 

    /** JDBC connection info
     * 
     */
    protected int jdbcConnLevel = 0;
    protected String jdbcConnTestStmt = "";

    /**
     *  The list of managedConnection used without transaction
     */
    protected Vector mcs = new Vector();

    /**
     *  The list of Synchronisation instance managed in this ConnectionManager
     */
    protected Vector synchros = new Vector();

    /**
     *  This is a cache to the last instance of ResourceSpec used by the
     *  ConnectionManager. Indeed this instance is always the same.
     */
    protected ResourceSpec rs = null;

    /**
     *  The holds the transaction support level for the associated RAR file
     */
    protected String transSupport = null;

    /**
     * This constant is used in the by the init method
     */
    public final static String RESOURCE_BUNDLE_NAME = "resourceBundleName";
    /**
     * This constant is used in the by the init method
     */
    public final static String LOGGER = "org.objectweb.util.monolog.logger";
    /**
     * This constant is used in the by the init method
     */
    public final static String POOL_LOGGER = "org.objectweb.util.monolog.logger_pool";
    /**
     * This constant is used in the by the init method
     */
    public final static String LOGGER_FACTORY = "org.objectweb.util.monolog.loggerFactory";
    /**
     * This constant is used in the by the init method
     */
    public final static String TRANSACTION_MANAGER = "transactionManager";
    /**
     * This constant is used in the by the init method
     */
    public final static String RESOURCE_MANAGER_EVENT_LISTENER = "resourceManagerEventListener";
    /**
     * This constant is used in the by the init method
     */
    public final static String RESOURCE_ADAPTER = "resourceAdapter";
    /**
     * This constant is used in the by the init method
     */
    public final static String PRINT_WRITER = "printWriter";

    /**
     * These constants define the different transaction support values
     */
    public final static String NO_TRANS_SUPPORT = "NoTransaction";
    public final static String LOCAL_TRANS_SUPPORT = "LocalTransaction";
    public final static String XA_TRANS_SUPPORT = "XATransaction";

    public ConnectionManagerImpl(String _transSupport) {
        if (_transSupport.length() == 0) {
            transSupport = NO_TRANS_SUPPORT;
        }
        else {
            transSupport = _transSupport;
        }
    }

    /**
     *  Setters method to initialize the ConnectionManager The logger instance
     *  where events are logged
     *
     *@param  l  The new Logger value
     */
    public void setLogger(Logger l) {
        trace = l;
    }


    /**
     *  Setters method to initialize the ConnectionManager A logger factory to
     *  obtain a logger
     *
     *@param  lf  The new LoggerFactory value
     */
    public void setLoggerFactory(LoggerFactory lf) {
        if (resourceBundleName == null) {
            trace = lf.getLogger("org.objectweb.resource.server");
        } else {
            trace = lf.getLogger("org.objectweb.resource.server");
        //,resourceBundleName);
        }
    }


    /**
     *  Setters method to initialize the ConnectionManager The printwriter where
     *  event are logged
     *
     *@param  pw  The new PrintWriter value
     */
    public void setPrintWriter(PrintWriter pw) {
        trace = new LoggerImpl(pw);
    }


    /**
     *  Setters method to initialize the ConnectionManager The logger instance
     *  where events are logged
     *
     *@param  _rmel  The new ResourceManagerEventListener value
     */
    public void setResourceManagerEventListener(
                    ResourceManagerEventListener _rmel) {
        rmel = _rmel;
    }


    /**
     *  Setters method to initialize the ConnectionManager The Transaction manager
     *  linked to this resource managed
     *
     *@param  _tm  The new TransactionManager value
     */
    public void setTransactionManager(TransactionManager _tm) {
        tm = _tm;
    }


    /**
     *  Setters method to initialize the ConnectionManager The
     *  managedConnectionFactory instance of the resource which must be managed by
     *  this connectionManager
     *
     *@param  _mcf           The new ResourceAdapter value
     *@exception  Exception  Description of Exception
     */
    public void setResourceAdapter(ManagedConnectionFactory _mcf)
                     throws Exception {
        setResourceAdapter(_mcf, 0, 0, -1, MAX_PSTMT_SIZE, 0L, 0, "");
    }

    /**
     *  Setters method to initialize the ConnectionManager The
     *  managedConnectionFactory instance of the resource which must be maneged by
     *  this connectionManager
     *
     *@param  _mcf           The new ResourceAdapter value
     *@param  initPoolSize   The initial pool size
     *@param  minPoolSize    The min pool size
     *@param  maxPoolSize    The max pool size
     *@param  _maxPstmtPoolSize  The max prepared statement pool size
     *@exception  Exception  Description of Exception
     */
    public void setResourceAdapter(ManagedConnectionFactory _mcf, 
                                   int initPoolSize, int minPoolSize, int maxPoolSize,
                                   int _maxPstmtPoolSize, long maxAge, int jConnLevel,
                                   String jConnStmt)
                     throws Exception {

        // set the max/min pool values
        if (maxPoolSize != 0)
                mcMaxPoolSize = maxPoolSize;
        if (minPoolSize > 0)
                mcMinPoolSize = minPoolSize;
                
        // set the jdbc connection info
        jdbcConnLevel = jConnLevel;
        jdbcConnTestStmt = jConnStmt;
        
        mcf = _mcf;
        poolMCs = (Pool) new HArrayPool();
        poolMCs.setMatchFactory(this);
        poolMCs.setMaxSize(mcMaxPoolSize);
        poolMCs.setMinSize(mcMinPoolSize);
        poolMCs.setInitSize(initPoolSize);
        poolMCs.setMaxAge(maxAge);
        vPstmts = new Vector();
        maxPstmtPoolSize = _maxPstmtPoolSize;
        
        usedMCs = new Hashtable();
        rs = new ResourceSpec(null, null);
                
        if (trace.isLoggable(BasicLevel.DEBUG)) {
            trace.log(BasicLevel.DEBUG, "");
        }
    }


        /**
         *  This method permits to initialize the ConnectionManager with the following
         *  parameter: RESOURCE_BUNDLE_NAME: The name of the resource bundle in order
         *  to internationalize the logging LOGGER: The logger instance where events
         *  are logged LOGGER_FACTORY: A logger factory to obtain a logger
         *  PRINT_WRITER: The printwriter where event are logged TRANSACTION_MANAGER:
         *  The Transaction manager linked to this resource manager
         *  RESOURCE_MANAGER_EVENT_LISTENER: The resource manage event listener which
         *  subscribed to later connection enlistement. RESOURCE_ADAPTER: The
         *  managedConnectionFactory instance of the resource which must be maneged by
         *  this connectionManager
         *
         *@param  ctx            Description of Parameter
         *@exception  Exception  Description of Exception
         */
        public void init(Context ctx) throws Exception {
                mc2mci = new Hashtable();

                // Get the resource bundle name to internationalise the log
                // Optional
                String resourceBundleName = null;
                try {
                        resourceBundleName = (String) ctx.lookup(RESOURCE_BUNDLE_NAME);
                }
                catch (NamingException e) {
                }

                // Get the logger or the logger factory or the printwriter
                try {
                    trace = (Logger) ctx.lookup(LOGGER);
                    poolTrace = (Logger) ctx.lookup(POOL_LOGGER);
                }
                catch (NamingException e) {
                }
                if (trace == null) {
                        try {
                                setLoggerFactory((LoggerFactory) ctx.lookup(LOGGER_FACTORY));
                        }
                        catch (NamingException e2) {
                        }
                }
                if (trace == null) {
                        PrintWriter pw = null;
                        try {
                                pw = (PrintWriter) ctx.lookup(PRINT_WRITER);
                        }
                        catch (NamingException e3) {
                        }
                        setPrintWriter(pw);
                }

                if (!transSupport.equalsIgnoreCase(NO_TRANS_SUPPORT)) {
                        // Get the transaction manager
                        tm = (TransactionManager) ctx.lookup(TRANSACTION_MANAGER);
                }

                // Get the set of connection
                try {
                        rmel = (ResourceManagerEventListener)
                                        ctx.lookup(RESOURCE_MANAGER_EVENT_LISTENER);
                }
                catch (NamingException ne) {
                }

                // Get the managedConnectionFactory instance which represents the resource
                // adapter
                try {
                        setResourceAdapter(
                                        (ManagedConnectionFactory) ctx.lookup(RESOURCE_ADAPTER));
                }
                catch (NamingException ne) {
                }
        }


        /**
         *  Description of the Method
         *
         *@exception  ResourceException  Description of Exception
         */
        public void cleanResourceAdapter() throws ResourceException { 
            // Delete the PreparedStatements
            PreparedStatementWrapper pw = null;
            while (vPstmts != null && vPstmts.size() > 0) {
                pw = (PreparedStatementWrapper) vPstmts.remove(0);
                try {
                    pw.destroy();
                } catch (Exception ex) { }
            }
            while (mcs != null && mcs.size() > 0 ) {
                MCInfo mci = (MCInfo) mcs.remove(0);
                mci.usedCs.clear();
                // Remove the PreparedStatements on the MC
                while (mci.pStmts != null && mci.pStmts.size() > 0) {
                    pw = (PreparedStatementWrapper) mci.pStmts.remove(0);
                    try {
                        pw.destroy();
                    } catch (Exception ex) { }
                }
                try {
                    mc2mci.remove(mci.mc);
                    mci.mc.destroy();
                } catch (Exception ex) { }
            }
            if (usedMCs != null) {
                for (Enumeration en = usedMCs.keys(); en.hasMoreElements(); ) {
                    Transaction tx = (Transaction) en.nextElement();
                    MCInfo mci = (MCInfo) usedMCs.get(tx);
                    // Get the current transaction
                    if (tx != null) {
                            // There is a transaction
                            // => unregister the connection from the transaction
                            try {
                                tx.delistResource((XAResource) mci.getXAResource(),
                                                XAResource.TMFAIL);
                            } catch (Exception ex) { }
                    }
                    mci.usedCs.clear();
                    // Remove the PreparedStatements on the MC
                    while (mci.pStmts != null && mci.pStmts.size() > 0) {
                        pw = (PreparedStatementWrapper) mci.pStmts.remove(0);
                        try {
                            pw.destroy();
                        } catch (Exception ex) { }
                    }
                    try {
                        mc2mci.remove(mci.mc);
                        mci.mc.destroy();
                    } catch (Exception ex) { }
                }
            }
            while (synchros != null && synchros.size() > 0 ) {
                MCInfo mci = (MCInfo) synchros.remove(0);
                mci.usedCs.clear();
                // Remove the PreparedStatements on the MC
                while (mci.pStmts != null && mci.pStmts.size() > 0) {
                    pw = (PreparedStatementWrapper) mci.pStmts.remove(0);
                    try {
                        pw.destroy();
                    } catch (Exception ex) { }
                }
                try {
                    mc2mci.remove(mci.mc);
                    mci.mc.destroy();
                } catch (Exception ex) { }
            }
        }


        /**
         *
         *  The method allocateConnection gets called by the resource adapter's
         *  connection factory instance.
         *
         * see javax.resource.cci.ConnectionManager
         */
        public synchronized Object allocateConnection(
                        ManagedConnectionFactory _mcf, ConnectionRequestInfo cxRequestInfo)
                         throws ResourceException {

            MCInfo mci = null;
            Transaction currentTx = null;
            Object connection = null;
            int retries = 0;
            
            while (connection == null && retries < 20) {

                if (mcf != _mcf) {
                    throw new ResourceException(
                                        "This ConnectionManager doesn't manage this RA:" + mcf);
                }

                currentTx = null;
                try {
                    if (tm != null) {
                        currentTx = tm.getTransaction();
                    }
                }
                catch (Exception e) {
                    trace.log(BasicLevel.ERROR,
                                    "Impossible to get the current transaction", e,
                                    "ConnectionManagerImpl", "allocateConnection");
                }

                //if there is a transaction check if a MC is already associated
                mci = (currentTx == null ? null : (MCInfo) usedMCs.get(currentTx));
                if (mci != null) {
                    if (mci.mc != null) {
                        // There are connections, try to match with the
                        // ManagedConnectionFactory
                        trace.log(BasicLevel.DEBUG, "MC (" + mci.mc + ") associated to the current Tx (" + currentTx + ") found");
                        Set s = (Set) new HashSet();
                        s.add(mci.mc);
                        if (mci.mc != mcf.matchManagedConnections(s, null, cxRequestInfo)) {
                                throw new ResourceException(
                                       "ConnectionManagerImpl.allocateConnection: illegal state : no mc is matched by mcf");
                        }
                        try {
                            currentTx.enlistResource(mci.getXAResource());
                        } catch (RollbackException e) {
                            // Although tx has been marked to be rolled back, 
                            // XAResource has been correctly enlisted.
                            trace.log(BasicLevel.WARN, "XAResource enlisted, but tx is marked rollback");
                        } catch (Exception e) {
                            e.printStackTrace();
                            throw new ResourceException(
                                      "ResourceService : an error occurred trying to enlistResource " + e.getMessage());
                        }
                    } else {
                      // This connection occurred an error before
                       trace.log(BasicLevel.INFO, "remnant of an old failed connection");
                       mci = null;
                       usedMCs.remove(currentTx);
                    }
                }

                if (mci == null) {
                    // No managed connection found => get a free ManagedConnection
                    // from the right pool
    
                    //ri.rs.subject = null; // Never set => already at null
                    rs.cxRequestInfo = cxRequestInfo;
                    try {
                        ManagedConnection mc = (ManagedConnection) poolMCs.getResource(rs);
                        if (mc == null) {
                            throw new ResourceException("ConnectionManagerImpl.allocateConnection: cannot allocate a ManagedConnection");
                        }
                        mci = (MCInfo) mc2mci.get(mc);
                        if (mci == null) {
                            mci = new MCInfo(mc);
                            mc2mci.put(mc, mci);
                        }
                        trace.log(BasicLevel.DEBUG, "get a MC from the ra pool, mc=" + mci.mc);
                        LocalXAWrapper lw = null;
                        if (transSupport.equalsIgnoreCase(LOCAL_TRANS_SUPPORT)) {
                            if (mci.lw == null) {
                                mci.lw = new LocalXAWrapper(mci.mc.getLocalTransaction(), trace);
                            }
                        } else if (mci.lw != null) {
                            mci.lw = null;
                        }
                        mci.mc.addConnectionEventListener(this);
/*                        MCInfo mci2 = (MCInfo) mc2mci.get(mci.mc);
                        if (mci2 != null && mci2 != mci) {
                                throw new ResourceException("ConnectionManagerImpl.allocateConnection: illegal state : bad old mc/mci association found");
                        }
*/    
                        mci.synchro = null;
                        // If a global transaction is already started then enlist the
                        // ManagedConnection instance
                        if (currentTx != null) {
                            currentTx.enlistResource(mci.getXAResource());
                            usedMCs.put(currentTx, mci);
                            mci.ctx = currentTx;
                            trace.log(BasicLevel.DEBUG, "Enlist the XA Resource in Tx:" + mci.ctx);
                        } else {
                            // There are not Transaction at the moment but the user can
                            // start a transaction after the getConnection call.
                            mci.ctx = null;
                            // must be clean
                            mcs.add(mci);
                            mci.rme = new RMEImpl(mci);
                            rmel.connectionOpened(mci.rme);
                            trace.log(BasicLevel.DEBUG, "Register the managed connection (no tx)");
                        }
        
                    } catch (ResourceException re) {
                        throw re;
                    } catch (Exception e) {
                        e.printStackTrace();
                        throw new ResourceException("ResourceService : an error related allocation of ManagedConnection " + e.getMessage());
                    }
        
                }
    
                //Fetch a free Connection from the ManagedConnection
                connection = mci.mc.getConnection(null, cxRequestInfo);
                // Want to add the non-wrapped Connection object
                mci.usedCs.add(connection);
                if (connection instanceof java.sql.Connection) {
                    try {
                        // Need a wrapper for non JOnAS jdbc ConnectionImpl
                        if (connection instanceof org.objectweb.jonas.jdbc.ConnectionImpl) {
                            ((org.objectweb.jonas.jdbc.ConnectionImpl)connection).setJonasInfo(mci, this);
                        } 
                        else {
                          connection = JonasSQLWrapper.createSQLWrapper(connection, mci, this, trace);
                        }
                        // Check the connection before reusing it
                        if (jdbcConnLevel > 0) {
                            try {
                                if(trace.isLoggable(BasicLevel.DEBUG)) {
                                    trace.log(BasicLevel.DEBUG, "Check the JDBC connection");
                                }
                                if (((Connection) connection).isClosed()) {
                                    connectionErrorOccurred(new ConnectionEvent(mci.mc, 
                                                                 ConnectionEvent.CONNECTION_ERROR_OCCURRED));
                                    connection = null;
                                    retries++;
                                    continue;
                                }
                                if (jdbcConnLevel > 1 && jdbcConnTestStmt != null &&
                                    jdbcConnTestStmt.length() > 0) {
                                    if(trace.isLoggable(BasicLevel.DEBUG))
                                        trace.log(BasicLevel.DEBUG, "retrying connection: "+jdbcConnTestStmt);
                                    java.sql.Statement stmt = ((Connection) connection).createStatement();
                                    stmt.execute(jdbcConnTestStmt);
                                    stmt.close();
                                }
                            } catch (Exception e) {
                                trace.log(BasicLevel.ERROR, "Error on connection: removing invalid managed connection "+mci.mc+": " + e);
                                connectionErrorOccurred(new ConnectionEvent(mci.mc, 
                                                             ConnectionEvent.CONNECTION_ERROR_OCCURRED));
                                connection = null;
                                retries++;
                                continue;
                            }
                        }
                    } catch (Exception ex) {
                        throw new ResourceException(ex.getMessage());
                    }
                }
                  
            }
            trace.log(BasicLevel.DEBUG, "get a logical connection on MC:"+connection);
            
            if (connection == null) {
                if (retries > 0) {
                    throw new ResourceAllocationException(
                                "Unable to obtain a connection object.  Check the validity of the jdbc-test-statement");
                } else {
                    throw new ResourceAllocationException("Unable to obtain a connection object");
                }
            }

            return connection;
        }


        /**
         * see org.objectweb.transaction.api..ConnectionManager
         */
        public synchronized void checkManagedConnection(
                        ManagedConnectionFactory mcf,
                        ManagedConnection current,
                        Object connection,
                        ConnectionRequestInfo cri) throws ResourceException {

                MCInfo mci = (MCInfo) mc2mci.get(current);
                Transaction currentTx = null;
                try {
                        if (tm != null) {
                                currentTx = tm.getTransaction();
                        }
                }
                catch (Exception e) {
                        trace.log(BasicLevel.ERROR,
                                        "Impossible to get the current transaction",
                                        e, "ConnectionManagerImpl", "checkManagedConnection");
                        throw new ResourceException(e.getMessage());
                }

                if (currentTx != null) {
                        if (mci.ctx == null) {
                                //before no tx, now tx

                                // This case will disappear when the
                                // ResourceManagerEventListener will be used
                                // The code will must go in the RME.openConnection(..) method

                                try {
                                        currentTx.enlistResource(mci.getXAResource());
                                }
                                catch (Exception e) {
                                        trace.log(BasicLevel.ERROR, "Impossible to enlist Resource",
                                                        e, "ConnectionManager", "checkManagedConnection");
                                }
                                mci.ctx = currentTx;
                                trace.log(BasicLevel.DEBUG, "mci.synchro=" + mci.synchro);
                                if (mci.synchro == null) {
                                        try {
                                                currentTx.registerSynchronization(new MySynchro(this, mci));
                                        }
                                        catch (Exception e) {
                                                throw new ResourceException("Error on registerSynchronization: "
                                                                 + e.getMessage());
                                        }
                                        trace.log(BasicLevel.DEBUG, "registerSynchro mc=" + mci.mc);
                                }
                                usedMCs.put(currentTx, mci);
                                mcs.remove(mci);
                                trace.log(BasicLevel.DEBUG, "*******enlist Resource");

                        }
                        else if (mci.ctx != currentTx) {
                                // The logical connection is shared by two transactionnal
                                // context
                                MCInfo mci2 = (MCInfo) usedMCs.get(currentTx);
                                trace.log(BasicLevel.DEBUG,
                                                "******* Another tx ctx ~ Connection shared => switch");
                                mci.usedCs.remove(connection);
                                mci2.usedCs.add(connection);
                                mci2.mc.associateConnection(connection);
                        }
                }
                else if (mci.ctx != null) {
                        //before tx, now no tx
                        trace.log(BasicLevel.DEBUG, "before tx, now no tx");
                }
                else {
                        trace.log(BasicLevel.DEBUG, "nothing to do");
                }
        }


        /**
         *  All method of the pool match to the right type of ManagedConnection because
         *  there is one pool by ManagedConnectionFactory.
         *
         * see org.objectweb.util.pool.api.PoolMatchFactory
         */
        public boolean matchResource(Object res, Object hints) {
                return true;
        }

        /**
         *  All method of the pool match to the right type of ManagedConnection because
         *  there is one pool by ManagedConnectionFactory.
         *
         * see org.objectweb.util.pool.api.PoolMatchFactory
         */
        public Object matchResource(Set res, Object hints) throws Exception {
                ResourceSpec spec = (hints != null) ? (ResourceSpec) hints : new ResourceSpec(null, null);
                return mcf.matchManagedConnections(res, null, spec.cxRequestInfo);
        }


        /**
         *  Call the ManagedConnectionFactory in order to create a new instance. of
         *  ManagedConnection. The Object is a ManagedConnection instance. The hints is a
         *  local structure: ResourceSpec
         *
         * see org.objectweb.util.pool.api.PoolMatchFactory
         */
        public Object createResource(Object hints) throws Exception {
                ResourceSpec spec = (hints != null) ? (ResourceSpec) hints : new ResourceSpec(null, null);
                ManagedConnection mc = mcf.createManagedConnection(spec.subject,
                                spec.cxRequestInfo);
                trace.log(BasicLevel.DEBUG, "Created MC: "+mc);
                return mc;
        }


        /** IMPLEMENTATION OF INTERFACE SQLManager
         */
        public PreparedStatement getPStatement(MCInfo mcinfo, Object conn, String user, String sql)
            throws SQLException {
                return getPStatement(mcinfo, conn, user, sql, ResultSet.TYPE_FORWARD_ONLY,
                                      ResultSet.CONCUR_READ_ONLY);
        }

        public PreparedStatement getPStatement(MCInfo mcinfo,
                                               Object conn, 
                                               String user, 
                                               String sql, 
                                               int resultSetType, 
                                               int resultSetConcurrency)
            throws SQLException {
        
            trace.log(BasicLevel.DEBUG, "Sql: "+sql+" User: "+user);
            // Use for the equals test
            PreparedStatementWrapper psw = new PreparedStatementWrapper(null, user, sql,
                                                                     resultSetType,
                                                                     resultSetConcurrency,
                                                                     trace);

            trace.log(BasicLevel.DEBUG, "MC pStmts: "+mcinfo.pStmts);
            //Check if statement matches ManagedConnection list & valid
            for (int i=0;i<mcinfo.pStmts.size();i++) {
                if (((PreparedStatementWrapper) mcinfo.pStmts.get(i)).equals(psw)) {
                    PreparedStatementWrapper ps =  (PreparedStatementWrapper) mcinfo.pStmts.remove(i);
                    mcinfo.pStmts.add(ps);  // add last 
                    mcinfo.stmtHash.add(ps);
//                    ps.clearPstmtValues();
                    return ps;
                }
            }
            
            //Check if statement is in ConnectionManager list of prepared Statements & valid
            trace.log(BasicLevel.DEBUG, "PStmts pool: "+vPstmts);
            for (int i=0;i<vPstmts.size();i++) {
                if (((PreparedStatementWrapper) vPstmts.get(i)).equals(psw)) {
                    PreparedStatementWrapper pw = (PreparedStatementWrapper) vPstmts.remove(i);
                    pw.clearPstmtValues();
                    mcinfo.pStmts.add(pw);
                    return pw;
                }
            }
            
            //If not in either, call con.prepareStatement, wrap the returned one
            // and add it to MC pool
            PreparedStatement ps = ((java.sql.Connection) conn).
                                          prepareStatement(sql, resultSetType, resultSetConcurrency);
            if (maxPstmtPoolSize < 0) {                                         
                // No prepared statement pool
                trace.log(BasicLevel.DEBUG, "Pooling is disabled");
                return ps;
            } else if (maxPstmtPoolSize == 0 || mcinfo.pStmts.size() < maxPstmtPoolSize) {
                psw.setPreparedStatement(ps);
                mcinfo.pStmts.add(psw);
                mcinfo.stmtHash.add(ps);
                trace.log(BasicLevel.DEBUG, "Adding PStmt: "+psw);
                return psw;
            } else if (mcinfo.isFreeStmt()){
                // Remove an entry from the current pool
                PreparedStatementWrapper pw = (PreparedStatementWrapper) mcinfo.pStmts.remove(0);
                pw.destroy();
                psw.setPreparedStatement(ps);
                mcinfo.pStmts.add(psw);
                mcinfo.stmtHash.add(ps);
                trace.log(BasicLevel.DEBUG, "Replacing "+pw+" with "+psw);
                return psw;
            } else {
                // No room for the Wrapped statement just return the PreparedStatement
                trace.log(BasicLevel.DEBUG, "No room in pool");
                return ps;
            }    
        }
        
        

        // Not implemented yet - JDBC 3.0
        public PreparedStatement getPStatement(MCInfo mcinfo,
                                                Object connection,
                                                String user, String sql, 
                                               int autoGeneratedKeys)
            throws SQLException {
            throw new SQLException("Not yet implemented, for JDBC 3.0");
        }

        // Not implemented yet - JDBC 3.0
        public PreparedStatement getPStatement(MCInfo mcinfo,
                                                Object connection,
                                                String user, String sql, 
                                               int[] columnIndexes)
            throws SQLException {
            throw new SQLException("Not yet implemented, for JDBC 3.0");
        }

        // Not implemented yet - JDBC 3.0
        public PreparedStatement getPStatement(MCInfo mcinfo,
                                                Object connection,
                                                String user, String sql, 
                                               int resultSetType, 
                                               int resultSetConcurrency, 
                                               int resultSetHoldability)
            throws SQLException {
            throw new SQLException("Not yet implemented, for JDBC 3.0");
        }

        // Not implemented yet - JDBC 3.0 
        public PreparedStatement getPStatement(MCInfo mcinfo,
                                                Object connection,
                                                String user, String sql, 
                                                String[] columnNames)
            throws SQLException {
            throw new SQLException("Not yet implemented, for JDBC 3.0");
        }

       

        public void releaseResource(Object _mc) throws Exception{
            trace.log(BasicLevel.DEBUG, "MC: "+_mc);
            if (_mc instanceof ManagedConnection) {
                ManagedConnection mc = (ManagedConnection) _mc;
                MCInfo mcinfo = (MCInfo) mc2mci.get(mc);
                if (mcinfo != null) {
                    destroyPStmts(mcinfo);
                }
            }
        }
        
        public synchronized void destroyPStmts(MCInfo mcinfo) throws Exception{
            trace.log(BasicLevel.DEBUG, "MCInfo: "+mcinfo);
            if (mcinfo.pStmts.size() <= 0) {
                return;
            }
            int stmtSize = mcinfo.pStmts.size();
            try {
                for (int i=0;i<stmtSize;i++) {
                    PreparedStatementWrapper psw = (PreparedStatementWrapper)mcinfo.pStmts.remove(0);
                    psw.closePstmt();
                }
            } catch (Exception ex) {
                throw ex;
            }
        }
        
/*        public synchronized void freePStmts(MCInfo mcinfo) throws Exception{
            trace.log(BasicLevel.DEBUG, "MCInfo: "+mcinfo);
            if (mcinfo.pStmts.size() <= 0) {
                return;
            }
            int stmtSize = mcinfo.pStmts.size();
            try {
                for (int i=0;i<stmtSize;i++) {
                    PreparedStatementWrapper psw = (PreparedStatementWrapper)mcinfo.pStmts.remove(0);
                    vPstmts.add(psw);
                }
            } catch (Exception ex) {
                throw ex;
            }
        }
*/        
        

        /**
         *  IMPLEMENTATION OF INTERFACE ConnectionEventListener *
         *
         *  A ManagedConnection instance calls the connectionClosed method to notify
         *  its registered set of listeners when an application component closes a
         *  connection handle. The application server uses this connection close event
         *  to make a decision on whether or not to put the ManagedConnection instance
         *  back into the connection pool.
         *
         * see javax.resource.spi.ConnectionEventListener
         */
        public synchronized void connectionClosed(ConnectionEvent event) {
                ManagedConnection mc = (ManagedConnection) event.getSource();
                if (mc == null) {
                        trace.log(BasicLevel.ERROR, "no mc found in Event!");
                }

                MCInfo mci = (MCInfo) mc2mci.get(mc);
                if (mci == null) {
                        trace.log(BasicLevel.ERROR, "no mci found!");
                }

                if (poolTrace.isLoggable(BasicLevel.DEBUG)) {
                        poolTrace.log(BasicLevel.DEBUG, "enter\n" + getState("\t"));
                }

                mci.usedCs.remove(event.getConnectionHandle());

                if (mci.usedCs.isEmpty()) {
                        trace.log(BasicLevel.DEBUG, "Last Connection has just been removed");

                        if (mci.localTransaction) {
                                trace.log(BasicLevel.ERROR, "The managed connection is being closed while a localtransaction is not finished");
                        }

                        try {
                                Transaction currentTx = null;
                                if (tm != null) {
                                        currentTx = tm.getTransaction();
                                }

                                if (currentTx != null) {
                                        // There is a transaction
                                        // delist the connection from the transaction
                                        currentTx.delistResource((XAResource) mci.getXAResource(),
                                                        XAResource.TMSUCCESS);
                                        trace.log(BasicLevel.DEBUG, "delist XAR");
                                        mci.ctx = currentTx;

                                        if (mci.synchro == null) {
                                                currentTx.registerSynchronization(new MySynchro(this, mci));
                                                trace.log(BasicLevel.DEBUG, "registerSynchro mc=" + mci.mc);
                                        }
                                }
                                else if (mci.localTransaction) {
                                        trace.log(BasicLevel.DEBUG, "MC isn't released because local transaction is not finished");

                                }
                                else {
                                        trace.log(BasicLevel.DEBUG, "no currentTx");
                                        //Signal to the RessourceManagerEventListener mc is released
                                        rmel.connectionClosed(mci.rme);

                                        //Remove the association (used mc / mci)
                                        //mc2mci.remove(mci.mc);

                                        // Detach the connectionManager from the mc
                                        mci.mc.removeConnectionEventListener(this);

                                        mcs.remove(mci);

                                        if (mci.ctx != null) {
                                                usedMCs.remove(mci.ctx);
                                        }

                                        mci.mc.cleanup();

                                        // Free up the PreparedStatements
                                        mci.stmtHash.clear();
                                        //freePStmts(mci); Comment out for now
                                        
                                        // Release the MC from its pool
                                        poolMCs.releaseResource(mci.mc, false);
                                        
                                }
                        }
                        catch (Exception e) {
                                trace.log(BasicLevel.ERROR, "an error during delisting of ManagedConection: "+e);
                        }
                        if (poolTrace.isLoggable(BasicLevel.DEBUG)) {
                                poolTrace.log(BasicLevel.DEBUG, "exit\n" + getState("\t"));
                        }
                }
        }


        /**
         *  The connectionErrorOccurred method indicates that the associated
         *  ManagedConnection instance is now invalid and unusable. The application
         *  server handles the connection error event notification by initiating
         *  application server-specific cleanup (for example, removing
         *  ManagedConnection instance from the connection pool) and then calling
         *  ManagedConnection.destroy method to destroy the physical connection..
         *
         * see javax.resource.spi.ConnectionEventListener
         */
        public void connectionErrorOccurred(ConnectionEvent event) {
                ManagedConnection mc = (ManagedConnection) event.getSource();
                if (mc == null) {
                        trace.log(BasicLevel.ERROR, "no mc found in Event!");
                }

                MCInfo mci = (MCInfo) mc2mci.get(mc);
                if (mci == null) {
                        trace.log(BasicLevel.ERROR, "no mci found!");
                }

                if (poolTrace.isLoggable(BasicLevel.DEBUG)) {
                        poolTrace.log(BasicLevel.DEBUG, "enter\n" + getState("\t"));
                }

                mci.usedCs.clear();

                try {
                        // Get the current transaction
                        Transaction currentTx = null;
                        if (tm != null) {
                                currentTx = tm.getTransaction();
                        }

                        if (currentTx != null) {
                                // There is a transaction
                                // => unregister the connection from the transaction
                                currentTx.delistResource((XAResource) mci.getXAResource(),
                                                XAResource.TMFAIL);
                                trace.log(BasicLevel.DEBUG, "XAR delisted");
                        }
                        //Signal to the RessourceManagerEventListener mc is released
                        rmel.connectionErrorOccured(mci.rme);
                        //Remove the association (used mc / mci)
                        //mc2mci.remove(mc);
                        // Detach the connectionManager from the mc
                        mc.removeConnectionEventListener(this);
                        // Remove the association (transaction ctx / mc)
                        if (mci.ctx != null) {
                                usedMCs.remove(mci.ctx);
                        }
                        else {
                                mcs.remove(mci);
                        }

                        trace.log(BasicLevel.DEBUG, "Destroying managed connectioon (" + mc + ")");
                        // Free up the PreparedStatements
                        //freePStmts(mci);  Comment out for now
                        destroyPStmts(mci);
                                        
                        poolMCs.releaseResource(mc, true);
                if (poolTrace.isLoggable(BasicLevel.DEBUG)) {
                        poolTrace.log(BasicLevel.DEBUG, "enter\n" + getState("\t"));
                }

                }
                catch (Exception e) {
                        trace.log(BasicLevel.ERROR,
                                        "an error related during delisting of ManagedConection");
                }
        }


        /**
         *  Notifies that a Resource Manager Local Transaction was committed on the
         *  ManagedConnection instance.
         *
         * see javax.resource.spi.ConnectionEventListener
         */
        public void localTransactionCommitted(ConnectionEvent event) {
                ManagedConnection mc = (ManagedConnection) event.getSource();
                if (mc == null) {
                        trace.log(BasicLevel.ERROR, "no mc found in Event!");
                }

                MCInfo mci = (MCInfo) mc2mci.get(mc);
                if (mci == null) {
                        trace.log(BasicLevel.ERROR, "no mci found!");
                }
                mci.localTransaction = false;

                if (mci.usedCs.isEmpty()) {
                        trace.log(BasicLevel.DEBUG, "Close the managed connection");

                        //Signal to the RessourceManagerEventListener mc is released
                        rmel.connectionClosed(mci.rme);

                        //Remove the association (used mc / mci)
                        //mc2mci.remove(mci.mc);

                        // Detach the connectionManager from the mc
                        mci.mc.removeConnectionEventListener(this);

                        mcs.remove(mci);
                        if (mci.ctx != null) {
                                usedMCs.remove(mci.ctx);
                        }

                        try {
                                mci.mc.cleanup();

                                // Free up the PreparedStatements
                                mci.stmtHash.clear();
                                //freePStmts(mci); Comment out for now
                                        
                                // Release the MC from its pool
                                poolMCs.releaseResource(mci.mc, false);
                        }
                        catch (Exception e) {
                                trace.log(BasicLevel.ERROR,
                                                "an error related ManagedConection release",
                                                e, "ConnectionManagerImpl", "localTransactionCommitted");
                        }
                }
        }


        /**
         *  Notifies that a Resource Manager Local Transaction was rolled back on the
         *  ManagedConnection instance.
         *
         * see javax.resource.spi.ConnectionEventListener
         */
        public void localTransactionRolledback(ConnectionEvent event) {
                ManagedConnection mc = (ManagedConnection) event.getSource();
                if (mc == null) {
                        trace.log(BasicLevel.ERROR, "no mc found in Event!");
                }

                MCInfo mci = (MCInfo) mc2mci.get(mc);
                if (mci == null) {
                        trace.log(BasicLevel.ERROR, "no mci found!");
                }
                mci.localTransaction = false;

                if (mci.usedCs.isEmpty()) {
                        trace.log(BasicLevel.DEBUG, "Close the managed connection");

                        //Signal to the RessourceManagerEventListener mc is released
                        rmel.connectionClosed(mci.rme);

                        //Remove the association (used mc / mci)
                        //mc2mci.remove(mci.mc);

                        // Detach the connectionManager from the mc
                        mci.mc.removeConnectionEventListener(this);

                        mcs.remove(mci);
                        if (mci.ctx != null) {
                                usedMCs.remove(mci.ctx);
                        }

                        try {
                            mci.mc.cleanup();

                            // Free up the PreparedStatements
                            mci.stmtHash.clear();
                            //freePStmts(mci); Comment out for now
                                        
                            // Release the MC from its pool
                            poolMCs.releaseResource( mci.mc, false);
                        }
                        catch (Exception e) {
                                trace.log(BasicLevel.ERROR,
                                                "an error related during ManagedConection release:",
                                                e, "ConnectionManagerImpl", "localTransactionRolledback");
                        }
                }
        }


        /**
         *  Notifies that a Resource Manager Local Transaction was started on the
         *  ManagedConnection instance.
         *
         * see javax.resource.spi.ConnectionEventListener
         */
        public void localTransactionStarted(ConnectionEvent event) {
                ManagedConnection mc = (ManagedConnection) event.getSource();
                if (mc == null) {
                        trace.log(BasicLevel.ERROR, "no mc found in Event!");
                }

                MCInfo mci = (MCInfo) mc2mci.get(mc);
                if (mci == null) {
                        trace.log(BasicLevel.ERROR, "no mci found!");
                }
                mci.localTransaction = true;
        }


        /**
         *  Description of the Method
         *
         *@return    Description of the Returned Value
         */
        public String toString() {
                String m = super.toString();
                // remove package name
                int c1 = 0;
                int current = m.indexOf(".");
                while (current != -1) {
                        c1 = current;
                        current = m.indexOf(".", current + 1);
                }
                return m.substring(c1 + 1, m.length());
        }


        /**
         *  Gets the State attribute of the ConnectionManagerImpl object
         *
         *@param  prefix  Description of Parameter
         *@return         The State value
         */
        protected String getState(String prefix) {
            String res = prefix + "mcf=" + mcf + "\n";
            res += prefix + "ResourceSpec=" + rs.toString() + "\n";
            res += prefix + "size of MC pool:" + poolMCs.getSize() + "\n";
            res += prefix + "size of usedMCs:" + usedMCs.size() + "\n";
/*            res += prefix + "size of PS pool:" + vPstmts.size() + "\n";
            PreparedStatementWrapper psw = null;
            for (int i = 0; i < vPstmts.size(); i++) {
                res += prefix + "\t" + vPstmts.elementAt(i).toString() + "\n";
                psw = (PreparedStatementWrapper)vPstmts.elementAt(i);
                res += prefix + "\t  (" +psw.user + ")"+ psw.sql + "\n";
                res += prefix + "\t  (resType)" +psw.resultSetType + "(resCon)"+ psw.resultSetConcurrency + "\n";
            }
*/
            res += prefix + "mcs attached to a tx:\n";
            for (Enumeration en = usedMCs.keys(); en.hasMoreElements(); ) {
                Object tx = en.nextElement();
                MCInfo mci = (MCInfo) usedMCs.get(tx);
                res += prefix + "MCI : tx=" + tx + "\n";
                res += mci.getState(prefix + "\t");
            }
            res += prefix + "mcs not attached to a tx:\n";
            for (Enumeration en = mcs.elements(); en.hasMoreElements(); ) {
                MCInfo mci = (MCInfo) en.nextElement();
                res += mci.getState(prefix + "\t");
            }
            res += prefix + "mcs waiting for tx commit or rollback:\n";
            for (Enumeration en = synchros.elements(); en.hasMoreElements(); ) {
                MCInfo mci = (MCInfo) en.nextElement();
                res += mci.getState(prefix + "\t");
            }
            return res;
        }

}
