/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Eric Hardesty
 * 
 *
 * --------------------------------------------------------------------------
 * $Id: RAConfig.java,v 1.6 2003/09/18 22:06:57 ehardesty Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.resource;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;
import java.util.Stack;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
 import java.util.zip.ZipOutputStream;

import org.objectweb.jonas_rar.deployment.api.RarDeploymentDesc;
import org.objectweb.jonas_rar.deployment.api.RarDeploymentDescException;
import org.objectweb.jonas_rar.deployment.xml.ConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyName;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyNameImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyValue;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyValueImpl;
import org.objectweb.jonas_rar.deployment.xml.LogEnabled;
import org.objectweb.jonas_rar.deployment.xml.LogEnabledImpl;
import org.objectweb.jonas_rar.deployment.xml.LogTopic;
import org.objectweb.jonas_rar.deployment.xml.LogTopicImpl;
import org.objectweb.jonas_rar.deployment.xml.NativeLib;
import org.objectweb.jonas_rar.deployment.xml.NativeLibImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolInit;
import org.objectweb.jonas_rar.deployment.xml.PoolInitImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolMin;
import org.objectweb.jonas_rar.deployment.xml.PoolMinImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolMax;
import org.objectweb.jonas_rar.deployment.xml.PoolMaxImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolMaxAge;
import org.objectweb.jonas_rar.deployment.xml.PoolMaxAgeImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolParams;
import org.objectweb.jonas_rar.deployment.xml.PoolParamsImpl;
import org.objectweb.jonas_rar.deployment.xml.PstmtMax;
import org.objectweb.jonas_rar.deployment.xml.PstmtMaxImpl;
import org.objectweb.jonas_rar.deployment.xml.JdbcCheckLevel;
import org.objectweb.jonas_rar.deployment.xml.JdbcCheckLevelImpl;
import org.objectweb.jonas_rar.deployment.xml.JdbcConnParams;
import org.objectweb.jonas_rar.deployment.xml.JdbcConnParamsImpl;
import org.objectweb.jonas_rar.deployment.xml.JdbcTestStatement;
import org.objectweb.jonas_rar.deployment.xml.JdbcTestStatementImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasResource;
import org.objectweb.jonas_rar.deployment.xml.JonasResourceImpl;
import org.objectweb.jonas_rar.deployment.xml.Jndiname;
import org.objectweb.jonas_rar.deployment.xml.JndinameImpl;
import org.objectweb.jonas_rar.deployment.xml.Rarlink;
import org.objectweb.jonas_rar.deployment.xml.RarlinkImpl;

/**
 * This class may be used to parse a resource adapter deployment descriptor
 * and generate the corresponding properties files for the jca service
 * 
 * java org.objectweb.jonas.resource.RAConfig -j jndiname -r rarlink -verbose <input_rar>
 * java org.objectweb.jonas.resource.RAConfig -u jonas-ra.xml <input_rar>
 * java org.objectweb.jonas.resource.RAConfig -dm -p db.properties <input_rar> <output_rar>
 *
 */
public class RAConfig {
    
    RarDeploymentDesc rarDD = null;
    
    static String outputFile = "META-INF/jonas-ra.xml";
    
    // Constructor
    public RAConfig (String fileName) {
        ClassLoader curLoader = Thread.currentThread().getContextClassLoader();
        
        try {
            rarDD.setParsingWithValidation(false);
            rarDD = RarDeploymentDesc.getInstance(fileName, curLoader);
        } catch (Exception e) {
            System.err.println("RAConfig Exception caught "+e);
        }

    }

    public RarDeploymentDesc getRarDeploymentDesc(){
        return rarDD;
    }

    /* Process the ra.xml file in the input rar file and build the 
     * corresponding jonas-ra.xml.  If the -p argument is specified, then 
     * retrieve the appropriate configuration values to pre-configure the
     * corrsponding JDBC RA config properties. 
     */
    private static void buildXML(String fileInputName, String jndiname, 
                                   String rarlink, Properties dbProps) {

        RAConfig raxml = new RAConfig (fileInputName);
        ConfigProperty[] raProps = raxml.getRarDeploymentDesc().getRaConfigPropTags();
        if (raProps == null) {
            System.err.println("*** RAR file doesn't contain ra.xml.");
            System.exit(2);
        }

        JonasResource jRes = new JonasResourceImpl();
        Jndiname jJndi = new JndinameImpl();
        Rarlink jRlink = new RarlinkImpl();
        LogEnabled jLogEnab = new LogEnabledImpl();
        LogTopic jLogTopic = new LogTopicImpl();
        NativeLib jNtvLib = new NativeLibImpl();
        PoolParams jPool = new PoolParamsImpl();
        PoolInit jPoolInit = new PoolInitImpl();
        PoolMin jPoolMin = new PoolMinImpl();
        PoolMax jPoolMax = new PoolMaxImpl();
        PoolMaxAge jPoolMaxAge = new PoolMaxAgeImpl();
        PstmtMax jPstmtMax = new PstmtMaxImpl();
        JdbcConnParams jConnParams = new JdbcConnParamsImpl();
        JdbcCheckLevel jCheckLevel = new JdbcCheckLevelImpl();
        JdbcTestStatement jTestStatement = new JdbcTestStatementImpl();
        JonasConfigProperty jCfg = null;
        JonasConfigPropertyName jCfgName = null;
        JonasConfigPropertyValue jCfgValue = null;
            
        String val = "";
        jPoolInit.setValue("0");
        jPool.setPoolInit(jPoolInit);

        val = "0";
        if (dbProps.getProperty("datasource.mincon") != null &&
            dbProps.getProperty("datasource.mincon").length() > 0) {
            val = dbProps.getProperty("datasource.mincon");
        }
        jPoolMin.setValue(val);
        jPool.setPoolMin(jPoolMin);

        val = "-1";
        if (dbProps.getProperty("datasource.maxcon") != null &&
            dbProps.getProperty("datasource.maxcon").length() > 0) {
            val = dbProps.getProperty("datasource.maxcon");
        }
        jPoolMax.setValue(val);
        jPool.setPoolMax(jPoolMax);
        
        val = "0";
        if (dbProps.getProperty("jdbc.connmaxage") != null &&
            dbProps.getProperty("jdbc.connmaxage").length() > 0) {
            val = dbProps.getProperty("jdbc.connmaxage");
        }
        jPoolMaxAge.setValue(val);
        jPool.setPoolMaxAge(jPoolMaxAge);

        jPstmtMax.setValue("10");
        jPool.setPstmtMax(jPstmtMax);
        
        
        jLogEnab.setValue("false");
        jRes.setLogEnabled(jLogEnab);
        jLogTopic.setValue("");
        jRes.setLogTopic(jLogTopic);
        jNtvLib.setValue("");
        jRes.setNativeLib(jNtvLib);
        if (jndiname.length() == 0) {
            jndiname = dbProps.getProperty("datasource.name");
            if (jndiname == null) {
                jndiname = "";
            }
        }
        jJndi.setValue(jndiname);

        val = "0";
        if (dbProps.getProperty("jdbc.connchecklevel") != null &&
            dbProps.getProperty("jdbc.connchecklevel").length() > 0) {
            val = dbProps.getProperty("jdbc.connchecklevel");
        }
        jCheckLevel.setValue(val);
        jConnParams.setJdbcCheckLevel(jCheckLevel);

        val = "";
        if (dbProps.getProperty("jdbc.connteststmt") != null &&
            dbProps.getProperty("jdbc.connteststmt").length() > 0) {
            val = dbProps.getProperty("jdbc.connteststmt");
        }
        jTestStatement.setValue(val);
        jConnParams.setJdbcTestStatement(jTestStatement);


        jRes.setJndiname(jJndi);
        jRlink.setValue(rarlink);
        jRes.setRarlink(jRlink);
        jRes.setPoolParams(jPool);
        jRes.setJdbcConnParams(jConnParams);
            
        val = "";
        String str = "";
        for(int i=0; i<raProps.length;i++){
            jCfg = new JonasConfigPropertyImpl();
            jCfgName = new JonasConfigPropertyNameImpl();
            str = raProps[i].getConfigPropertyName().getValue();
            jCfgName.setValue(str);
            jCfg.setJonasConfigPropertyName(jCfgName);
            jCfgValue = new JonasConfigPropertyValueImpl();
            if (dbProps.size() > 0) {
                val = getProp(dbProps, str);
            } else {
                val = "";
            }
            jCfgValue.setValue(val);
            jCfg.setJonasConfigPropertyValue(jCfgValue);
            jRes.addJonasConfigProperty(jCfg);
        }

        try {
            jRes.marshal(new BufferedWriter(new FileWriter("jonas-ra.xml")));
        }catch(Exception e){
            System.err.println("Error writing output");
            System.exit(2);
        }
    }

    /* Mapping routine for the JDBC RA config values and the existing 
     * <DB>.properties values.
     */
    private static String getProp(Properties prop, String str) {
        String mapping[][] = { 
            {"connCheckLevel", "jdbc.connchecklevel"},
            {"connMaxAge", "jdbc.connmaxage"},
            {"connTestStmt", "jdbc.connteststmt"},
            {"URL", "datasource.url"},
            {"dsClass", "datasource.classname"},
            {"user", "datasource.username"},
            {"password", "datasource.password"},
            {"isolationLevel", "datasource.isolationlevel"},
            {"mapperName", "datasource.mapper"},
        };
        
        String ret = "";
        for (int i=0;i<mapping.length;i++) {
            if (mapping[i][0].equalsIgnoreCase(str)) {
                ret = (prop.getProperty(mapping[i][1]) == null 
                       ? "" : prop.getProperty(mapping[i][1]).trim());
            }
        }
        
        return ret;
    }
    private static void updateRAR(String infile, String outfile, String updatename,
                                    String rarlink) throws Exception
    {

        ZipOutputStream zipOutput = null;
        ZipFile zipFile = null;
        Enumeration zippedFiles = null;
        ZipEntry currEntry = null;
        ZipEntry entry = null;
        byte [] buffer = new byte[500];
        int num = 0;
        
        String filename = infile;
        String outfilename = infile;
        boolean outNew = false;
        if (outfile != null && outfile.length() != 0) {
            outfilename = outfile;
            outNew = true;
        }
        

        try {
              File file = new File(filename);
              if (file.exists()) {
                  zipFile = new ZipFile( file.getAbsolutePath() ); 
                  //get an enumeration of all existing entries 
                  zippedFiles = zipFile.entries(); 
                  //create your output zip file
                  if (!outNew) {
                      zipOutput = new ZipOutputStream( new FileOutputStream ( 
                                                    new File( file.getAbsolutePath() + "NEW") ) );
                  } else {
                      File ofile = new File(outfilename);
                      if (ofile.exists()) {
                          ofile.delete();
                      }
                      
                      zipOutput = new ZipOutputStream( new FileOutputStream ( 
                                                    new File( outfilename) ) );
                  }
                  //Get all the data out of the previously zipped files and write it to a new ZipEntry to go into a new file archive 
                  if ((outfile == null || outfile.length() == 0) && rarlink.length() == 0) {
                      while (zippedFiles.hasMoreElements()) {
                          //Retrieve entry of existing files
                          currEntry = (ZipEntry)zippedFiles.nextElement();
                          if (currEntry.getName().equalsIgnoreCase(outputFile)) {
                              continue; 
                          }
                          //Read data from existing file
                          zipOutput.putNextEntry(new ZipEntry(currEntry.getName()) ) ;
                          InputStream reader = zipFile.getInputStream( currEntry ); 
                          while ((num = reader.read(buffer)) != -1) {
                              zipOutput.write(buffer, 0, num);
                          }
                          //Commit the data
                          zipOutput.flush();
                          zipOutput.closeEntry(); 
                      }
                  }
                  //Close the old zip file
                  zipFile.close();

                  //Write the 'new' file to the archive, commit, and close 
                  entry = new ZipEntry( outputFile );
                  zipOutput.putNextEntry( entry );

                  //Read data from existing file
                  try {
                      BufferedReader reader = new BufferedReader( new FileReader(updatename) );
                      int ch = 0;
                      while ((ch = reader.read()) != -1) {
                          zipOutput.write(ch);  
                      }
                  } catch (Exception ex){
                      System.out.println("Error reading input file: "+updatename+" "+ex);
                      //Commit the data
                      zipOutput.flush();
                      zipOutput.closeEntry(); 
                      zipOutput.finish(); 
                      zipOutput.close();
                  }
                  //Commit the data
                  zipOutput.flush();
                  zipOutput.closeEntry(); 
                  zipOutput.finish(); 
                  zipOutput.close();

                  if (!outNew) {
                      //delete the old file and rename the new one
                      File toBeDeleted = new File ( file.getAbsolutePath() );
                      toBeDeleted.delete(); 
                      File toBeRenamed = new File ( file.getAbsolutePath() + "NEW"); 
                      toBeRenamed.renameTo( file );
                  }
              } else {
                  //create your output zip file
                  zipOutput = new ZipOutputStream( new FileOutputStream ( new File( outfilename ) ) );

                  //Write the file to the archive, commit, and close 
                  entry = new ZipEntry( outputFile );
                  zipOutput.putNextEntry( entry );
                  //Read data from existing file
                  BufferedReader reader = new BufferedReader( new FileReader(updatename) ); 
                  int ch = 0;
                  while ((ch = reader.read()) != -1) {
                      zipOutput.write(ch);  
                  }

                  //Commit the data
                  zipOutput.flush();
                  zipOutput.closeEntry(); 
                  zipOutput.finish(); 
                  zipOutput.close();
              }
        } catch (Exception ex) {
              System.out.println("Exception: "+ex);
        } 
    }        

    private static String indent(int count) {
        String front = "";
        String indentChars = "    ";
        for (int i = 0; i < count; i++) {
            front += indentChars;
        }
        return front;
    }

    public static void usage() {
        System.out.println("");
        System.out.println("Usage: java org.objectweb.jonas.resource.RAConfig <Options> <Input_File>");
        System.out.println("         to generate the jonas-ra.xml file for a Resource Adapter.");
        System.out.println("");
        System.out.println("           With Options:");
        System.out.println("             -? or -help to print this help message");
        System.out.println("             -dm,-ds,-pc,-xa  DriverManager, Datasource, PooledConnection,");
        System.out.println("                              XAConnection only used with -p");
        System.out.println("             -j <jndiname>  specifies the jndiname RA");
        System.out.println("             -p <property file>  specifies the database property file");
        System.out.println("                                 to process");
        System.out.println("             -r <rarlink>  specifies the rar file to link to");
        System.out.println("             -u <input name> update the input rar with the specified file");   
        System.out.println("                              unless an output_file named is specified");   
        System.out.println("             -verbose   output the contents of the  deployment descriptor ");   
        System.out.println("");
        System.out.println("           Input_File     Resource Adapter RAR file");
        System.out.println("           Output_File    Resource Adapter RAR file only used with ");
        System.out.println("                          -p(required) or -u(optional)");
        System.out.println("");
    }

    // Main Method
    public static void main (String[] args) throws Exception{
        PrintStream out = null;
        boolean  isHelp          = false;
        boolean  isVerbose       = false;
        boolean  doProperty      = false;
        boolean  doUpdate        = false;
        String fileInputName = null;
        String fileOutputName = null;
        String jndiname = "";
        String rarlink = "";
        String updatename = "";
        String jdbcRarLink = "";
        String jonasRarStr = "JOnASJDBC_"; 
        String propertyFile = "";
        // Get command args
        
        Properties dbProps = null;

        if (args.length <1) {
            System.err.println("RAConfig: Arguments missing");
            usage();
            System.exit(2);
        }
        for (int argn = 0; argn < args.length; argn++) {
            String arg = args[argn];
            if (arg.equals("-help") || arg.equals("-?")) {
                isHelp = true;
                continue;
            }
            if (arg.equals("-verbose") || arg.equals("-v")) {
                isVerbose = true;
                continue;
            }
            if (arg.equals("-update") || arg.equals("-u")) {
                doUpdate = true;
                if (argn+1 == args.length || args[++argn].startsWith("-"))
                {
                  System.err.println("*** Error with -u input");
                  usage();
                  System.exit(2);
                }  
                updatename  = args[argn];
                continue;
            }
            if (arg.equals("-jndiname") || arg.equals("-j")) {
                if (argn+1 == args.length || args[++argn].startsWith("-"))
                {
                  System.err.println("*** Error with -j input");
                  usage();
                  System.exit(2);
                }  
                jndiname  = args[argn];
                continue;
            }
            if (arg.equals("-rarlink") || arg.equals("-r")) {
                if (argn+1 == args.length || args[++argn].startsWith("-"))
                {
                  System.err.println("*** Error with -r input");
                  usage();
                  System.exit(2);
                }  
                rarlink  = args[argn];
                continue;
            }
            if (arg.equals("-property") || arg.equals("-p")) {
                doProperty = true;
                if (argn+1 == args.length || args[++argn].startsWith("-"))
                {
                  System.err.println("*** Error with -p input");
                  usage();
                  System.exit(2);
                }  
                propertyFile  = args[argn];
                continue;
            }
            if (arg.equals("-dm")) {
                jdbcRarLink = jonasRarStr + "DM";
                continue;
            }
            if (arg.equals("-ds")) {
                jdbcRarLink = jonasRarStr + "DS";
                continue;
            }
            if (arg.equals("-xa")) {
                jdbcRarLink = jonasRarStr + "XA";
                continue;
            }
            if (arg.equals("-pc")) {
                jdbcRarLink = jonasRarStr + "PC";
                continue;
            }
            if (fileInputName == null) {
                fileInputName = args[argn];
                if (!fileInputName.endsWith(".rar")) {
                    fileInputName += ".rar";
                }
            } else if (doProperty || doUpdate){
                if (fileOutputName == null) {
                    fileOutputName = args[argn];
                    if (!fileOutputName.endsWith(".rar")) {
                        fileOutputName += ".rar";
                    }
                }
            } else {
                System.err.println("*** Error multiple input files specified without -property or -update");
                usage();
                System.exit(2);
            }  
        }
        
        // Usage ?
        if (isHelp) {
            usage();
            System.exit(0);
        }
        if (fileInputName == null) {
            System.err.println("*** Error missing input file");
            usage();
            System.exit(2);
        }
        if (doProperty && fileOutputName == null) {
            System.err.println("*** Error missing output file");
            usage();
            System.exit(2);
        }
        if (fileInputName.equals(fileOutputName)) {
            System.err.println("*** Input and Output filenames cannot be the same");
            System.exit(2);
        }
        
        dbProps = new Properties();
        if (doProperty){
            if (jdbcRarLink.length() > 0 && rarlink.length() == 0) {
                rarlink = jdbcRarLink;
            }
            try {
                dbProps.load(new FileInputStream(propertyFile));
            } catch (Exception ex) {
                if (propertyFile.endsWith(".properties")) {
                    System.out.println("Error reading "+propertyFile+ " "+ex);
                } else {
                    try {
                        dbProps.load(new FileInputStream(propertyFile+".properties"));
                    } catch (Exception ex1) {
                        System.out.println("Error reading "+propertyFile+ " "+ex);
                    }
                }
            }
            buildXML(fileInputName, jndiname, rarlink, dbProps);
            if (updatename.length() == 0) {
                updatename = "jonas-ra.xml";        
            }
            updateRAR(fileInputName, fileOutputName, updatename, rarlink);
        } else if (doUpdate){
            updateRAR(fileInputName, fileOutputName, updatename, rarlink);
        } else {        
            buildXML(fileInputName, jndiname, rarlink, dbProps);
        }
    }        
}
