/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ResourceServiceImpl.java,v 1.26 2003/09/18 16:47:49 ehardesty Exp $
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.resource;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.lang.reflect.Method;

import java.net.MalformedURLException;
import java.net.URL;

import java.rmi.RemoteException;

import java.text.SimpleDateFormat;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;

import java.util.jar.JarEntry;

import javax.management.MalformedObjectNameException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;

import javax.resource.Referenceable;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;

import javax.transaction.TransactionManager;

import org.objectweb.jonas.common.JJarFile;
import org.objectweb.jonas.common.JModule;
import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.naming.CompNamingContext;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.jonas.server.JClassLoader;

import org.objectweb.jonas_rar.deployment.api.RarDeploymentDesc;
import org.objectweb.jonas_rar.deployment.xml.ConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.ConfigPropertyValue;
import org.objectweb.jonas_rar.deployment.xml.ConfigPropertyValueImpl;
import org.objectweb.jonas_rar.deployment.xml.Connector;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasResource;
import org.objectweb.jonas_rar.deployment.xml.Resourceadapter;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
/**
 * JCA resource service implmentation
 * @author Philippe Coq
 * Contributor(s):
 * JOnAS 2.4 Sebastien Chassande-Barrioz (sebastien.chassande@inrialpes.fr)
 * JOnAS 3.0 Eric Hardesty (Eric.Hardesty@bull.com)
 *
 */
public class ResourceServiceImpl extends AbsServiceImpl implements
    ResourceService,
    ResourceServiceImplMBean {

        static private Logger logger = null;
        static private Logger poolLogger = null;
        static private Logger setterLogger = null;

    // Properties for inits

    // JCA resource service configuration parameters
    public static final String RESOURCE_LIST = "jonas.service.resource.resources";
    public static final String CLASS = "jonas.service.resource.class";
    public static final String AUTOLOADDIR = "jonas.service.resource.autoloaddir";
    public static final String PARSINGWITHVALIDATION = "jonas.service.resource.parsingwithvalidation";

    public final String MANAGED_CONNECTION_FACTORY_CLASS
        = "managedconnectionfactoryclass";
    public static final String JNDI_NAME = "jndiname";
    public static final String PRINTWRITER ="printWriter";
    public static final String MONITOR_FACTORY
        = "org.objectweb.monolog.MonitorFactory";

    private class RAInfo {
        public Object cf;
        public ManagedConnectionFactory mcf;
        public ConnectionManager cm;
        public Connector racon;
        public JonasResource jonasra;
        public ConfigProperty [] cfgRaJonas;
        public boolean isInEar;
        public RAInfo(Object _cf, ManagedConnectionFactory _mcf,
                      ConnectionManager _cm, Connector _racon,
                      JonasResource _jonasra, ConfigProperty [] _cfgRaJonas,
                      boolean _isInEar) {
            cf = _cf;
            mcf = _mcf;
            cm = _cm;
            racon = _racon;
            jonasra = _jonasra;
            cfgRaJonas = _cfgRaJonas;
            isInEar = _isInEar;
        }
    }

    /**
     * This hashtable associates a jndiName to an instance of the local
     * class RAInfo which contains a reference to the connectionFactory
     * and the ManagedConnectionFactory
     */
    static private Hashtable jndiName2RA = new Hashtable();
    static private Hashtable fileName2jndiName = new Hashtable();

    /**
     * This vector is list of RAR files that will be deployed after all
     * other RAR files have been deployed.  The problem is that these RAR
     * files are associated with another RAR file that has not been processed.
     */
     private Vector delayedRAs = new Vector();
     private boolean processingDelayed = false;

    /**
     * The transaction manager in server
     */
    private TransactionManager tm = null ;

    /**
     * The ConnectionManager manages connections of resource adapters
     * and more exactly the association between logical connections,
     * physical connections, local transactions and global transaction(XA)
     */
    private ConnectionManagerImpl cm = null;

    /**
     * JNDI Context
     */
    private Context ictx = null;

    private Vector autoNames = null;
    private Vector resourceNames = null;

    /**
     *  The name of the JONAS_BASE directory
     */
    private static final String JONAS_BASE = JProp.getJonasBase();

    /**
     *  The name of the rars directory
     */
    private static final String RARS_DIR = JONAS_BASE + File.separator + "rars";


    //IMPLEMENTATION OF 'AbsServiceImpl' ABSTRACT CLASS

    /**
     * - Get the global jndi context
     * - Get the list of the resource adapters. The list is reachable in the
     * Context parameter under the name RESOURCE_LIST.
     * - Get the transaction manager into the jndi
     * - Instanciate the connection manager
     */
    public void doInit(Context ctx) {
        if(logger == null)
            logger = Log.getLogger(Log.JONAS_JCA_PREFIX+".process");
        if(poolLogger == null)
            poolLogger = Log.getLogger(Log.JONAS_JCA_PREFIX+".pool");
        if(setterLogger == null)
            setterLogger = Log.getLogger(Log.JONAS_JCA_PREFIX+".setters");
        resourceNames = new Vector();
        autoNames = new Vector();

        // Avoid using NamingManager here: performance is not a goal here.
        try {
            ictx = new InitialContext();
        } catch (NamingException e) {
            logger.log(BasicLevel.ERROR,"Cannot create initial context when Resource service initializing");
            throw new ServiceException("Cannot create initial context when Resource service initializing",e);
        }

        // Add the rars of the jonas.service.resource.autoloaddir property
        String dirValue = null;
        try {
            dirValue = (String) ctx.lookup(AUTOLOADDIR);
            logger.log(BasicLevel.DEBUG, "autoloaddir= "+dirValue);
        } catch (NamingException e) {
        }
        if (dirValue != null) {
            StringTokenizer st = new StringTokenizer(dirValue, ",");
            String dirName = null;
            while (st.hasMoreTokens()) {
                dirName = st.nextToken().trim();
                if (File.separatorChar == '/') {
                    dirName = dirName.replace('\\', File.separatorChar);
                } else {
                    dirName = dirName.replace('/', File.separatorChar);
                }
                addRars(dirName);
            }
        }

        // Get the list of the resource adapter names

        try {
            String rs = (String)ctx.lookup(RESOURCE_LIST);
            logger.log(BasicLevel.DEBUG, "resource list= "+rs);
            if (rs != null) {
                StringTokenizer st = new StringTokenizer(rs, ",");
                String resFilename = null;
                while (st.hasMoreTokens()) {
                    resFilename = st.nextToken().trim();
                    if (File.separatorChar == '/') {
                        resFilename = resFilename.replace('\\', File.separatorChar);
                    } else {
                        resFilename = resFilename.replace('/', File.separatorChar);
                    }
                    resourceNames.add(resFilename);
                }
            }
        } catch (NamingException e) {
            logger.log(BasicLevel.ERROR,"Cannot lookup the configuration context at Resource service starting");
            throw new ServiceException("Cannot lookup the configuration context at Resource service starting", e);
        }

        // Get a reference to the Transaction service
        try {
            TransactionService ts = (TransactionService) ServiceManager.getInstance().getTransactionService();
            tm = ts.getTransactionManager();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"Cannot get the Transaction service: "+e);
            throw new ServiceException("Cannot get the Transaction service: ", e);
        }

      // Set the XML parsing mode to no validation
        String parsingMode = "false";
        try {
            parsingMode = (String) ctx.lookup(PARSINGWITHVALIDATION);
        } catch (NamingException e) {
            // No problem if there is no value for 'parsingwithvalidation' (false by default)
        }
        if ("false".equalsIgnoreCase(parsingMode)) {
            RarDeploymentDesc.setParsingWithValidation(false);
            logger.log(BasicLevel.DEBUG, "XML parsing without validation");
        } else {
            logger.log(BasicLevel.DEBUG, "XML parsing with validation");
        }

        logger.log(BasicLevel.DEBUG, "ResourceService initialized");
    }

    public void doStart() throws ServiceException {
        // creates each resource
        String rarFileName = null;
        CompNamingContext ctx = null;
        for (int i = 0; i < resourceNames.size(); i++) {
            rarFileName = (String) resourceNames.elementAt(i);
            logger.log(BasicLevel.DEBUG, "rarFileName="+rarFileName);
            try {
                ctx = new CompNamingContext(rarFileName);
                ctx.rebind("rarFileName", rarFileName);
                ctx.rebind("isInEar", new Boolean(false));
                createResourceAdapter(ctx);
            } catch (Exception e) {
                logger.log(BasicLevel.ERROR,"JOnAS: Cannot create resource: "+rarFileName+" exception: "+e);
                logger.log(BasicLevel.ERROR,"Please check if "+rarFileName+" is correct");
                e.printStackTrace();
            }
        }

        // process any delayed rars
        if (!delayedRAs.isEmpty()) {
          try{
            processingDelayed = true;
            Object rarList[] = null;
            rarList = delayedRAs.toArray();
            for (int i=0;i<rarList.length;i++) {
                try {
                    createResourceAdapter((CompNamingContext) rarList[i]);
                } catch (Exception e) {
                    logger.log(BasicLevel.ERROR,"JOnAS: Cannot create resource: "+(String) rarList[i]+" exception: "+e);
                    logger.log(BasicLevel.ERROR,"Please verify that the rarlink is correct");
                    e.printStackTrace();
                }
            }
          }
          catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Error with delayed RAR file deployment\n"+e);
            throw new ServiceException("ResourceService: Error with delayed RAR file deployment", e);
          }
        }

        try {
            // Register ResourceService MBean
            ((JmxService)ServiceManager.getJmxService()).getJmxServer().registerMBean(this,
                            JonasObjectName.resourceService());
        } catch (ServiceException se) {
            // Jmx Service not available, do nothing
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Cannot start the Resource service:\n"+e);
            throw new ServiceException("ResourceService: Cannot start the Resource service", e);
        }

    }

    public void doStop() throws ServiceException {
        synchronized (jndiName2RA) {
            for (Enumeration e = jndiName2RA.keys(); e.hasMoreElements();) {
                String jndiName = (String) e.nextElement();
                RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
                try {
                    // Unbind from jndi
                    ictx.unbind(jndiName);
                } catch (NamingException ex) {
                    logger.log(BasicLevel.ERROR,"ResourceService: cannot unbind ResourceAdapter"
                                +jndiName+" :\n" + ex);
                    throw new ServiceException(ex.getMessage());
                }

                try {
                    // The connection manger must stop managing the RA
                    ((ConnectionManagerImpl)rai.cm).cleanResourceAdapter();
                } catch (Exception ex) {
                }


                try {
                    // unbind MBean
                    ((JmxService)ServiceManager.getJmxService()).getJmxServer()
                        .unregisterMBean(
                                         JonasObjectName.resourceAdapter(jndiName));
                } catch  (MBeanRegistrationException mone) {
                    //???
                } catch  (MalformedObjectNameException mone) {
                    //???
                } catch  (InstanceNotFoundException infe) {
                    //???
                } catch  (ServiceException se) {
                    // Jmx Service not available, do nothing
                }

                logger.log(BasicLevel.DEBUG,
                            "ResourceService: unbindResourceAdapter:"+jndiName);
            }
        }

        try {
            MBeanServer mbeanServer =
                    ((JmxService)ServiceManager.getJmxService()).getJmxServer();
            // unregister resource Service MBean
            mbeanServer.unregisterMBean(JonasObjectName.resourceService());
            // unregister all Resource Adaptor
            Set objectNames =
                    mbeanServer.queryNames(JonasObjectName.allResourceAdaptors(), null) ;
            Iterator it  = objectNames.iterator();
            while (it.hasNext()) {
                mbeanServer.unregisterMBean((ObjectName)it.next());
            }
        } catch (ServiceException se) {
            // Jmx Service not available, do nothing
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"EJBService: Cannot stop the Resource service:\n"+e);
            throw new ServiceException("ResourceService: Cannot stop the Resource service",e);
        }
        logger.log(BasicLevel.DEBUG, "ResourceService stopped");
    }


    private ConnectionManager createConnectionManager(String trans) throws Exception {

        ConnectionManagerImpl cm = new ConnectionManagerImpl(trans);

        Context c = new CompNamingContext("");
        c.rebind(ConnectionManagerImpl.TRANSACTION_MANAGER, tm);
        c.rebind(ConnectionManagerImpl.RESOURCE_MANAGER_EVENT_LISTENER,tm);

        // Set the MonologFactory into context of initialisation
        c.rebind(ConnectionManagerImpl.LOGGER, logger);
        c.rebind(ConnectionManagerImpl.POOL_LOGGER, poolLogger);

        cm.init(c);
        return cm;
    }

    // IMPLEMENTATION OF 'ResourceService' INTERFACE //

    /*
     * Create a new resource adapter.  This Resource Adapter is configured via
     * xml files in the rar file
     */
    public void createResourceAdapter(Context ctx) throws Exception {
        ConnectionManager cm = null;

        // Gets the 3 parameters :
        // rarFileName, isInEar and possible AltDD and earURL
        String rarFileName;
        boolean isInEar = false;
        URL earURL = null;
        JClassLoader curLoader = null;
        try {
            rarFileName = (String) ctx.lookup("rarFileName");
            isInEar = ((Boolean) ctx.lookup("isInEar")).booleanValue();
            if (isInEar == true) {
                earURL = (URL) ctx.lookup("earURL");
            }
            curLoader = (JClassLoader) Thread.currentThread().getContextClassLoader();
            ctx.rebind("classloader", curLoader);

        } catch (NamingException e) {
            String err = "Error while getting parameter from context param.";
            logger.log(BasicLevel.ERROR, err + e.getMessage());
            throw new ResourceServiceException(err, e);
        } catch (Exception ex) {
            String err = "Error while getting parameter from context param.";
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            throw new ResourceServiceException(err, ex);
        }

        logger.log(BasicLevel.DEBUG,"entering for "+rarFileName+","+cm);
        if(!rarFileName.endsWith(".rar")){
            rarFileName += ".rar";
        }

        File f = new File(rarFileName);
        if (!f.exists()) {
            boolean found = false;
            String resFileName = null;
            // In case of the name is a rar file name, check also in
            // the JONAS_BASE/rars directory
            resFileName = RARS_DIR + File.separator + rarFileName;
            File fr = new File(resFileName);
            found = fr.exists();
            if (found) {
                rarFileName = resFileName;
                ctx.rebind("rarFileName", rarFileName);
            } else {
                logger.log(BasicLevel.ERROR, "createResourceAdapter: "+resFileName+" not found");
                Exception e = new NamingException(resFileName+" not found");
                throw e;
            }
        }

        RarDeploymentDesc radesc = RarDeploymentDesc.getInstance(ctx);

        Connector conn = radesc.getConnector();
        JonasResource jonasra = radesc.getJonasResource();
        if(conn == null && jonasra == null){
            logger.log(BasicLevel.ERROR,"ResourceService.createRA: Resource ("+rarFileName+") must be a valid RAR file.");
            throw new Exception("resource input file incorrect");
        }

        // Need to check if rarlink is specified.  If yes and has been processed, then get the correct
        // Connector object and continue, if it has not been processed, then add this rar to the
        // deferred list and it will be processed at end of the rar files.  If no rarlink, then
        // make sure that the connector object is valid.
        String rarVal = (jonasra.getRarlink() == null
                         ? null
                         : jonasra.getRarlink().getValue());
        ConfigProperty [] mergeConfigs = null;
        int poolMax = -1;
        int poolMin = 0;
        int poolInit = 0;
        int pstmtMax = 0;
        long maxage = 0;
        int jdbcConnLevel = 0;
        String jdbcConnTestStmt = "";
        String tmpPool = null;

        if (rarVal != null && rarVal.length() > 0) {
            conn = getConnector(rarVal);
            if (conn == null) {
                if (processingDelayed) {
                    logger.log(BasicLevel.ERROR,"ResourceService.createRA: Resource ("+rarFileName+") with rarlink ("+rarVal+") is not valid.");
                    throw new Exception("resource input file incorrect");
                }
                delayedRAs.add(ctx);
                return;
            }
            else {
                // Get the config parameters from the rarlink
                radesc.setRaConfigPropTags(conn);
                mergeConfigs = getMergeConfigs(rarVal);
                // Setup the pool parameters from the rarlink
                JonasResource jr = getJonasResource(rarVal);
                if (jr != null) {
                    if (jr.getPoolParams() != null) {
                        if (jr.getPoolParams().getPoolInit() != null) {
                            tmpPool = jr.getPoolParams().getPoolInit().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) poolInit = Integer.parseInt(tmpPool);
                        }
                        if (jr.getPoolParams().getPoolMin() != null) {
                            tmpPool = jr.getPoolParams().getPoolMin().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) poolMin = Integer.parseInt(tmpPool);
                        }
                        if (jr.getPoolParams().getPoolMax() != null) {
                            tmpPool = jr.getPoolParams().getPoolMax().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) poolMax = Integer.parseInt(tmpPool);
                        }
                        if (jr.getPoolParams().getPstmtMax() != null) {
                            tmpPool = jr.getPoolParams().getPstmtMax().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) pstmtMax = Integer.parseInt(tmpPool);
                        }
                        if (jr.getPoolParams().getPoolMaxAge() != null) {
                            tmpPool = jr.getPoolParams().getPoolMaxAge().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) maxage = Long.parseLong(tmpPool);
                        }
                    }
                    if (jr.getJdbcConnParams() != null) {
                        if (jr.getJdbcConnParams().getJdbcCheckLevel() != null) {
                            tmpPool = jr.getJdbcConnParams().getJdbcCheckLevel().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) jdbcConnLevel = Integer.parseInt(tmpPool);
                        }
                        if (jr.getJdbcConnParams().getJdbcTestStatement() != null) {
                            jdbcConnTestStmt = jr.getJdbcConnParams().getJdbcTestStatement().getValue();
                        }
                    }
                }
            }

        }
        else if (conn == null || jonasra == null) {
            logger.log(BasicLevel.ERROR,"ResourceService.createRA: Resource ("+rarFileName+") must be a valid RAR file. Conn: "+conn+" Jonasra: "+jonasra);
            throw new Exception("resource input file incorrect");
        }

        Resourceadapter ra = conn.getResourceadapter();
        if (processingDelayed && mergeConfigs != null)
            ra.setConfigPropertyList(Arrays.asList(mergeConfigs));

        // Process the config values
        String mcfc = ra.getManagedconnectionfactoryClass().getValue();
        if(mcfc == null){
            logger.log(BasicLevel.ERROR,"ResourceService.createRA:"+MANAGED_CONNECTION_FACTORY_CLASS+" property not found");
            throw new Exception("configuration file incorrect");
        }


        // Process the RAR file adding each of the jar files into the classloader and if the
        // native-lib is defined then place all other files into that directory.

        processRar(curLoader, rarFileName, jonasra);

        // Instanciate the ra : mcf class
        Class mcfClass = Class.forName(mcfc, true, curLoader);
        ManagedConnectionFactory mcf =
            (ManagedConnectionFactory) mcfClass.newInstance();

        cm = createConnectionManager(conn.getResourceadapter().getTransactionSupport().getValue());

        // Associate the resource adapter to the connectionmanager
        tmpPool = null;
        try {
            if (jonasra.getPoolParams() != null) {
                if (jonasra.getPoolParams().getPoolInit() != null) {
                    tmpPool = jonasra.getPoolParams().getPoolInit().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) poolInit = Integer.parseInt(tmpPool);
                }
                if (jonasra.getPoolParams().getPoolMin() != null) {
                    tmpPool = jonasra.getPoolParams().getPoolMin().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) poolMin = Integer.parseInt(tmpPool);
                }
                if (jonasra.getPoolParams().getPoolMax() != null) {
                    tmpPool = jonasra.getPoolParams().getPoolMax().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) poolMax = Integer.parseInt(tmpPool);
                }
                if (jonasra.getPoolParams().getPstmtMax() != null) {
                    tmpPool = jonasra.getPoolParams().getPstmtMax().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) pstmtMax = Integer.parseInt(tmpPool);
                }
                if (jonasra.getPoolParams().getPoolMaxAge() != null) {
                    tmpPool = jonasra.getPoolParams().getPoolMaxAge().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) maxage = Long.parseLong(tmpPool);
                }
            }
            if (jonasra.getJdbcConnParams() != null) {
                if (jonasra.getJdbcConnParams().getJdbcCheckLevel() != null) {
                    tmpPool = jonasra.getJdbcConnParams().getJdbcCheckLevel().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) jdbcConnLevel = Integer.parseInt(tmpPool);
                }
                if (jonasra.getJdbcConnParams().getJdbcTestStatement() != null) {
                    jdbcConnTestStmt = jonasra.getJdbcConnParams().getJdbcTestStatement().getValue();
                }
            }
        }
        catch (Exception ex) {
                ex.printStackTrace();
                logger.log(BasicLevel.ERROR,"Invalid Pool parameter from jonas-ra.xml:  Ensure that numeric values are used. "+tmpPool);
                throw new Exception("incorrect pool parameter ");
        }


        // Get the jndi name of the ra
        String jndiName = jonasra.getJndiname().getValue();
        logger.log(BasicLevel.DEBUG, "jndiName="+ jndiName);

        String logEnabled = null;
        if (jonasra.getLogEnabled() != null) {
            logEnabled = jonasra.getLogEnabled().getValue();
        }
        String logTopic = null;
        if (jonasra.getLogTopic() != null) {
            logTopic = jonasra.getLogTopic().getValue();
        }

        // Check if logging desired and the value of the desired topic
        if (logEnabled != null) {
            logger.log(BasicLevel.DEBUG, "log-enabled="+ logEnabled);
            if (checkLogEnabled(logEnabled)) {
                if (logTopic != null && logTopic.length() > 0) {
                    logger.log(BasicLevel.DEBUG, "log-topic="+ logTopic);
                    mcf.setLogWriter(Log.getLogWriter(logTopic));
                } else {
                    logger.log(BasicLevel.DEBUG, "default log-topic="+ Log.JONAS_JCA_PREFIX);
                    mcf.setLogWriter(Log.getLogWriter(Log.JONAS_JCA_PREFIX));   // "org.objectweb.jonas.jca"
                }
            }
        }

        PrintWriter pw = mcf.getLogWriter();
        if (pw != null) {
            //Create a date format
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss z");
            String date = sdf.format(new Date());
            pw.println("MCF: "+mcf+" output starting at "+date);
            pw.flush();
        }

        // init the ra with setter method

        ConfigProperty [] cfgRa = radesc.getRaConfigPropTags();
        JonasConfigProperty [] cfgJonas = radesc.getJonasConfigPropTags();

        ConfigProperty [] cfgRaJonas = cfgMerge(radesc.getRaConfigPropTags(),
                                           radesc.getJonasConfigPropTags());
        int curParam = 0;
        while ( curParam < cfgRaJonas.length ){
            String fieldName = cfgRaJonas[curParam].getConfigPropertyName().getValue();
            String methodName = "set"+fieldName.substring(0,1).toUpperCase() +
                                fieldName.substring(1);
            String fieldType = cfgRaJonas[curParam].getConfigPropertyType().getValue();
            Method[] m = mcfClass.getMethods();

            int i=0;
            while (i<m.length
                   && (! m[i].getName().equals(methodName)
                       || m[i].getParameterTypes().length!=1
                       || ! (m[i].getParameterTypes())[0].getName().equals(fieldType))
                   ) {
                i++;
            }
            if (i<m.length) {
                Class [] paramtype = m[i].getParameterTypes();
                Object[] param = new Object[1];

                String curValue = cfgRaJonas[curParam].getConfigPropertyValue().getValue();

                setterLogger.log(BasicLevel.DEBUG, "Processing Field Name: "+fieldName+" Type: "+fieldType+" Value: "+curValue);

                if (paramtype[0].equals(java.lang.Integer.TYPE) ||
                    paramtype[0].equals(java.lang.Integer.class)){
                    param[0]  = new Integer(curValue);

                } else if (paramtype[0].equals(java.lang.Boolean.TYPE) ||
                           paramtype[0].equals(java.lang.Boolean.class) ){
                    param[0]  = new Boolean(curValue);

                } else if (paramtype[0].equals(java.lang.Double.TYPE) ||
                           paramtype[0].equals(java.lang.Double.class) ){
                    param[0]  = new Double(curValue);

                } else if (paramtype[0].equals(java.lang.Byte.TYPE) ||
                           paramtype[0].equals(java.lang.Byte.class) ){
                    param[0]  = new Byte(curValue);

                } else if (paramtype[0].equals(java.lang.Short.TYPE) ||
                           paramtype[0].equals(java.lang.Short.class)){
                    param[0]  = new Short(curValue);

                } else if (paramtype[0].equals(java.lang.Long.TYPE) ||
                           paramtype[0].equals(java.lang.Long.class) ){
                    param[0]  = new Long(curValue);

                } else if (paramtype[0].equals(java.lang.Float.TYPE) ||
                           paramtype[0].equals(java.lang.Float.class) ){
                    param[0]  = new Float(curValue);

                } else if (paramtype[0].equals(java.lang.Character.TYPE) ||
                            paramtype[0].equals(java.lang.Character.class) ){
                    param[0]  = new Character(curValue.charAt(0));

                } else if (paramtype[0].equals(java.lang.String.class)){
                    param[0]  = curValue;
                    setterLogger.log(BasicLevel.DEBUG, "Calling String method with "+curValue);

                } else {
                    logger.log(BasicLevel.ERROR,"Type unsupported for setter method:"
                                +methodName);
                    throw new Exception("incorrect type for setter method ");
                }
                // invocation of the corresponding setter method on
                // ManagedConnectionFactory instance
                try {
                    m[i].invoke(mcf,param);
                } catch (Exception e) {
                    logger.log(BasicLevel.DEBUG, "method "
                                +methodName+
                                " not found on resource adapter, check the configuration file "
                                +rarFileName+".properties");
                }
                curParam++;
            } else {
                logger.log(BasicLevel.ERROR,"Method not found: "+methodName);
                throw new Exception("incorrect method name: "+methodName);
            }
        }

        // Set hashCode for ManagedConnectionFactory
        mcf.hashCode();

        // Obtain the CF and connect the RA to the ConnectionManager
        // A connection factory must implements javax.resource.Referenceable
        Referenceable cf = (Referenceable)
            mcf.createConnectionFactory(cm);

        logger.log(BasicLevel.DEBUG, "cf="+ cf);

        ((ConnectionManagerImpl) cm).setResourceAdapter(mcf, poolInit, poolMin, poolMax, pstmtMax,
                                                        maxage, jdbcConnLevel, jdbcConnTestStmt);

        // Register into Jndi
        try {

            Reference ref =
                new Reference(
                              cf.getClass().getName(),
                              ConnectionFactoryJNDIHandler.class.getName(),
                              null);
            ref.add( new StringRefAddr(JNDI_NAME, jndiName) );
            cf.setReference(ref);
            ictx.rebind(jndiName, cf);

        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Cannot register ResourceAdapter in naming with the name " + jndiName);
            logger.log(BasicLevel.ERROR,"ResourceService: Exception caught : "+e);
            return;
        }

        synchronized (jndiName2RA) {
            // Save the association (jndiName, RA)
            jndiName2RA.put(jndiName, new RAInfo(cf, mcf, cm, conn, jonasra, cfgRaJonas, isInEar) );
            fileName2jndiName.put(rarFileName, jndiName);
        }


        // Register this ResourceAdapter with JMX
        try {
            Properties p = buildProp(cfgRa, cfgJonas);
            // MBean resourcesource registering
            ((JmxService)ServiceManager.getJmxService()).getJmxServer()
                .registerMBean(
                               new JmxResourceAdapter(p, jndiName, rarFileName, isInEar, earURL),
                               JonasObjectName.resourceAdapter(jndiName));
        } catch (ServiceException se) {
            // Jmx Service not available, do nothing
        }
        logger.log(BasicLevel.INFO, rarFileName+" available");
        logger.log(BasicLevel.DEBUG, "Mapping ResourceAdapter "+rarFileName+ " on "+jndiName);

    }


    /**
     * Process the specified rar, extracting all the jars files to be included
     * in the classloader and all the native files needed by the rar.
     * @param cls the current classloader
     * @param resFileName rar file to process
     * @param jonasra the jonas-ra.xml properties
     * @throws ResourceServiceException if an error occurs during
     * the extracting.
     */
    public void processRar(JClassLoader cls, String resFileName, JonasResource jonasra)
                 throws ResourceServiceException {

        String jarPath = RARS_DIR + File.separator + "jonas" + File.separator;
        String binPath = null;
        if (jonasra.getNativeLib() != null) {
            binPath = jonasra.getNativeLib().getValue() + File.separator;
            if (binPath.startsWith("/") || binPath.startsWith("\\") ||
                binPath.charAt(1) == ':') {
                ;
            } else {
                binPath = JONAS_BASE + File.separator + binPath;
            }

        }

        //Get RAR name
        String rarName = resFileName;
        if (rarName.endsWith(".rar")) {
            rarName = resFileName.substring(0, resFileName.lastIndexOf('.'));
        }

        int off = rarName.lastIndexOf(File.separator);
        rarName = rarName.substring(++off);
        if (File.separatorChar == '\\') {
            off = rarName.lastIndexOf("/");
            rarName = rarName.substring(++off);
        }

        JJarFile jjar = null;

        try {
            jjar = new JJarFile(resFileName);
            for (Enumeration ent = jjar.entries(); ent.hasMoreElements(); ) {
                JarEntry je = (JarEntry) ent.nextElement();
                if (!je.isDirectory() && je.getName().endsWith(".jar")) {
                    File newd = new File(jarPath);
                    if (!newd.exists()) {
                        newd.mkdirs();
                    }

                    String fileName = jarPath+"_"+rarName+"_"+je.getName();
                    jjar.extract(je, fileName);
                    cls.addURL((new File(fileName)).toURL());
                } else if (!je.isDirectory() && !je.getName().startsWith("META-INF") &&
                            binPath != null){
                    File newf = new File(binPath);
                    if (!newf.exists()) {
                        newf.mkdirs();
                    }
                    String fileName = binPath+je.getName();
                    jjar.extract(je, fileName);
                }
            }
        } catch (Exception ex) {
            String err = "Error while extracting the files from "+resFileName;
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            throw new ResourceServiceException(err, ex);
        }

    }

    /**
     * Deploy the given rars of an ear file with the specified parent
     * classloader (ear classloader). (This method is only used for
     * for ear applications).
     * @param ctx the context containing the configuration
     * to deploy the rars.<BR>
     * This context contains the following parameters :<BR>
     *    - urls the list of the urls of the rars to deploy.<BR>
     *    - earRootURL the URL of the ear application file.<BR>
     *    - earClassLoader the ear classLoader of the j2ee app.<BR>
     *    - altDDs the optional URI of deployment descriptor.<BR>
     * @throws ResourceServiceException if an error occurs during
     * the deployment.
     */
    public void deployRars(Context ctx) throws ResourceServiceException {
        // Gets the parameters from the context :
        //    - urls the list of the urls of the rars to deploy.
        //    - earRootURL the URL of the ear application file.
        //    - earClassLoader the ear classLoader of the j2ee app.
        //    - altDDs the optional URI of deployment descriptor.
        URL[] urls = null;
        URL earURL = null;
        ClassLoader earClassLoader = null;
        URL[] altDDs = null;
        try {
            urls = (URL[]) ctx.lookup("urls");
            earURL = (URL) ctx.lookup("earUrl");
            earClassLoader = (ClassLoader) ctx.lookup("earClassLoader");
            altDDs = (URL[]) ctx.lookup("altDDs");
        } catch (NamingException e) {
            String err = "Error while getting parameter from context param ";
            logger.log(BasicLevel.ERROR, err + e.getMessage());
            throw new ResourceServiceException(err, e);
        }

        delayedRAs.clear(); // reset the delayed RAs
        // Deploy all the rars of the ear application.
        for (int i = 0; i < urls.length; i++) {
            // Get the name of a rar to deploy.
            String fileName = urls[i].getFile();
            logger.log(BasicLevel.DEBUG, "Deploy rar '" + fileName + "' for the ear service");

            // The context to give for the creation of the container
            // associated to the ejb-jar.
            Context contctx = null;
            try {
                contctx = new CompNamingContext(fileName);
                contctx.rebind("rarFileName", fileName);
                contctx.rebind("isInEar", new Boolean(true));
                contctx.rebind("earURL", earURL);
                if (altDDs[i] != null) {
                    contctx.rebind("altDD", altDDs[i]);
                }
                createResourceAdapter(contctx);
            } catch (Exception e) {
                // A rar is corrupted so undeploy all the deployed rar
                // of the ear application.
                logger.log(BasicLevel.ERROR, "Error when deploying '" + fileName + "'");
                logger.log(BasicLevel.ERROR, e.getMessage());
                logger.log(BasicLevel.ERROR, "Undeploy rar of the ear application");

                for (int j = 0; j < i; j++) {
                    String rarFileName = urls[j].getFile();
                    try {
                        // Try to undeploy a rar of the ear application.
                        CompNamingContext compctx = new CompNamingContext(rarFileName);
                        compctx.rebind("rarFileName", rarFileName);
                        compctx.rebind("isInEar", new Boolean(true));
                        unRegisterRar(compctx);
                    } catch (Exception ex) {
                        // Cannot undeploy a rar of the ear application
                        // So there is an error message.
                        logger.log(BasicLevel.ERROR, "Error when undeploying '" + rarFileName + "'");
                        logger.log(BasicLevel.ERROR, ex.getMessage());
                        logger.log(BasicLevel.ERROR, "Cannot undeploy rar of the ear application");
                    }

                }
                throw new ResourceServiceException("Error during the deployment", e);
            }
        }
        // process any delayed rars
        if (!delayedRAs.isEmpty()) {
          try{
            processingDelayed = true;
            Object rarList[] = null;
            rarList = delayedRAs.toArray();
            for (int i=0;i<rarList.length;i++) {
                try {
                    createResourceAdapter((CompNamingContext) rarList[i]);
                } catch (Exception e) {
                    logger.log(BasicLevel.ERROR,"JOnAS: Cannot create resource: "+(String) rarList[i]+" exception: "+e);
                    logger.log(BasicLevel.ERROR,"Please verify that the rarlink is correct");
                    e.printStackTrace();
                }
            }
          }
          catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Error with delayed RAR file deployment\n"+e);
            throw new ServiceException("ResourceService: Error with delayed RAR file deployment", e);
          }
        }
    }


    /**
     * Undeploy the given rars of an ear file. (This method is only
     * used for the ear applications).
     * @param urls the list of the urls of the rars to undeploy.
     */
    public void unDeployRars(URL[] urls) {
        for (int i = 0; i < urls.length; i++) {
            String fileName = urls[i].getFile();
            if (fileName2jndiName.containsKey(fileName)) {
                try {
                    // Try to undeploy a rar of the ear application.
                    CompNamingContext compctx = new CompNamingContext(fileName);
                    compctx.rebind("rarFileName", fileName);
                    compctx.rebind("isInEar", new Boolean(true));
                    unRegisterRar(compctx);
                } catch (Exception ex) {
                    logger.log(BasicLevel.ERROR,"Cannot undeploy resource: "+ fileName + " "+ex);
                }
            } else {
                logger.log(BasicLevel.ERROR, "Cannot remove the non-existant rar '" + fileName + "'");
            }
        }
    }


    /*
     * Unregister the resource adapter.
     */
    public void unRegisterRar(Context ctx) throws Exception {
        // Gets the parameters :
        // rarFileName, isInEar
        String rarFileName;
        boolean isInEar = false;
        try {
            rarFileName = (String) ctx.lookup("rarFileName");
            isInEar = ((Boolean) ctx.lookup("isInEar")).booleanValue();

        } catch (NamingException e) {
            String err = "Error while getting parameter from context param.";
            logger.log(BasicLevel.ERROR, err + e.getMessage());
            throw new ResourceServiceException(err, e);
        } catch (Exception ex) {
            String err = "Error while getting parameter from context param.";
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            throw new ResourceServiceException(err, ex);
        }

        String jndiName = (String) fileName2jndiName.get(rarFileName);
        logger.log(BasicLevel.DEBUG,"Finding "+jndiName+" for file "+rarFileName+
                                    " in FileName2jndiName: "+fileName2jndiName);
        RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
        try {
            // Unbind from jndi
            ictx.unbind(jndiName);
        } catch (NamingException ex) {
            logger.log(BasicLevel.DEBUG,"ResourceService: cannot unbind ResourceAdapter"
                        +jndiName+" :\n" + ex);
            throw new ServiceException(ex.getMessage());
        }

        // The connection manger must stop managing the RA
        ((ConnectionManagerImpl)rai.cm).cleanResourceAdapter();


        try {
            // unbind MBean
            ((JmxService)ServiceManager.getJmxService()).getJmxServer()
                .unregisterMBean(
                                 JonasObjectName.resourceAdapter(jndiName));
        } catch  (MBeanRegistrationException mone) {
            //???
        } catch  (MalformedObjectNameException mone) {
            //???
        } catch  (InstanceNotFoundException infe) {
            //???
        } catch  (ServiceException se) {
            // Jmx Service not available, do nothing
        }

        // Remove rars from hashtables
        fileName2jndiName.remove(rarFileName);
        jndiName2RA.remove(jndiName);
        resourceNames.remove(rarFileName);

        logger.log(BasicLevel.INFO,
                    "ResourceService: unRegisterRar: "+rarFileName+"("+jndiName+")");
    }

    //--------------------------------------------------------------
    //           Other Method
    //--------------------------------------------------------------

    /**
     * get the ResourceAdapter  matching the raname
     * will be used in ConnectionFactoryJNDIHandler class at lookup time
     */
    public static Object getConnectionFactory(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.cf );
        }
    }

    //--------------------------------------------------------------
    //           MBean Methods
    //--------------------------------------------------------------

    /**
     * @return Integer Total Number of Resources available in JOnAS
     */
    public Integer getCurrentNumberOfResource() {
        return getCurrentNumberOfRars();
    }


    /**
     * @return Integer Total Number of Rars available in JOnAS
     */
    public Integer getCurrentNumberOfRars() {
        return new Integer(jndiName2RA.size());
    }


    /**
     * @return the list of RAR files deployed
     */
    public List getDeployedRars() {
        ArrayList al = new ArrayList();
        String rarFileName = null;
        Enumeration enum = fileName2jndiName.keys();
        while (enum.hasMoreElements()) {
            rarFileName = enum.nextElement().toString();
            try {
                al.add((new File(rarFileName)).toURL().getPath());
            }
            catch (Exception e) {
                // none
            }
        }
        return al;
    }

    /**
     * Return the list of installed RAR containers.
     * The RAR files or the directories with expanded RAR container are searched
     * in JONAS_BASE/rars and all rar directories 'autoload'.
     * @return The list of RAR files or the directories with expanded RAR container found
     * @throws Exception if the list can't be retrieved
     */
    public List getInstalledRars() throws Exception {
        // get JAR files found in JONAS_BASE/ejbjars
        ArrayList al = JModule.getInstalledContainersInDir(RARS_DIR, JModule.RAR_EXTENSION
            , JModule.RAR_CHILD_DIR, JModule.RAR_CONFIRM_FILE);
        // get RAR files found in all autoload directories
        for (int i=0; i < autoNames.size(); i++) {
            al.addAll(JModule.getInstalledContainersInDir(autoNames.get(i).toString(),
                JModule.RAR_EXTENSION, JModule.RAR_CHILD_DIR, JModule.RAR_CONFIRM_FILE));
        }
        return al;
    }

    /**
     * This method is added temporarily. It will disapear when Rars will have their associated MBeans
     * (when Rars will become manageable)
     * @return the names of the rars currently deployed in the JOnAS server
     */
    public Set getRarNames() {
        HashSet names = new HashSet();
        String rarFileName = null;
        for (int i = 0; i < resourceNames.size(); i++) {
            rarFileName = (String) resourceNames.elementAt(i);
            names.add(rarFileName);
        }
        return names;
    }


    /**
     * Deploy an RAR by delegating the operation to the createResourceAdapter method.
     * This is used for JMX management.
     * @param fileName the fileName of the rar which must be be deployed.
     * @throws RemoteException if rmi call failed.
     * @throws ResourceServiceException if the deployment of the RAR failed.
     */
    public void deployRarMBean(String fileName) throws RemoteException, ResourceServiceException {
        Context ctx = null;
        try {
            ctx = new CompNamingContext(fileName);
            ctx.rebind("rarFileName", fileName);
            ctx.rebind("isInEar", new Boolean(false));
            createResourceAdapter(ctx);
        } catch (Exception e) {
            String err = "Error when deploying the rar file ";
            logger.log(BasicLevel.ERROR, err + e.getMessage());
            throw new ResourceServiceException(err, e);
        }
    }

    /**
     * Undeploy an RAR by delegating the operation to the unRegisterRar() method.
     * This is used for JMX management.
     * @param fileName the fileName of the rar which must be be undeployed.
     * @throws RemoteException if rmi call failed.
     * @throws ResourceServiceException if the undeployment of the RAR failed.
     */
    public void unDeployRarMBean(String fileName) throws RemoteException, ResourceServiceException {

        /* We have only the name of the file, not its associated path, so we look in
           the current directory and in the rar applications directory
        */

        boolean found = false;
        logger.log(BasicLevel.DEBUG, "Trying to undeploy: "+fileName+
                                     " with the following deployed:"+fileName2jndiName);
        try {
            if (fileName2jndiName.containsKey(fileName)) {
                //We have found
                found = true;
            }

            if (!found) {
                // In case of the name is an rar check also in
                // the JONAS_BASE/rars directory
                String rarFileName = RARS_DIR + File.separator + fileName;
                if(!rarFileName.endsWith(".rar")){
                    rarFileName += ".rar";
                }

                if (fileName2jndiName.containsKey(rarFileName)) {
                    //We have found
                    found = true;
                    fileName = rarFileName;
                }
            }
            if (!found) {
                String err = "Cannot undeploy the rar '" + fileName + "', it is not deployed.";
                logger.log(BasicLevel.ERROR, err);
                throw new ResourceServiceException(err);
            }
        } catch (Exception ex) {
            String err = "Error trying to undeployRarMBean" + fileName;
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            throw new ResourceServiceException(err, ex);
        }

        //We've got the file, now we bind the params
        CompNamingContext compctx = null;
        try {
            compctx = new CompNamingContext(fileName);
            compctx.rebind("rarFileName", fileName);
            compctx.rebind("isInEar", new Boolean(false));
            //Call the function
            unRegisterRar(compctx);
        } catch (NamingException e) {
            String err = "Error when binding parameters";
            logger.log(BasicLevel.ERROR, err + e.getMessage());
            throw new ResourceServiceException(err, e);
        } catch (Exception ex) {
            String err = "Error when unRegistering rar";
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            ex.printStackTrace();
            throw new ResourceServiceException(err, ex);
        }

    }

    /**
     * Test if the specified filename is already deployed or not
     * @param fileName the name of the rar file.
     * @return true if the rar is deployed, else false.
     */
    public boolean isRarLoaded(String fileName) {

        String updateName = null;
        boolean isLoaded = false;
        try {
            if (fileName2jndiName.containsKey(fileName)) {
                isLoaded = true;
            }
            // If not found check the exact filename
            if (!isLoaded) {
                // construct name for rars in JONAS_BASE/rars directory
                updateName= RARS_DIR + File.separator + fileName;

                //Check if the rar is already deployed or not
                if (fileName2jndiName.containsKey(updateName)) {
                    isLoaded = true;
                }
            }
        } catch (Exception e) {
            String err = "Cannot determine if the rar is deployed or not";
            logger.log(BasicLevel.ERROR, err);
            return false;
        }

        return isLoaded;
    }


    /**
     * Return the list of installed RAR container ready to deploy.
     *
     * @return The list of deployable RAR container
     * @throws Exception
     */
    public List getDeployableRars() throws Exception {
        List al = getInstalledRars();
        al.removeAll(getDeployedRars());
        return al;
    }

    /**
     * Return the list of "autoload" directories for RAR containers.
     * @return The list of all "autoload" directories
     */
    public List getAutoloadDirectories() {
        ArrayList al = new ArrayList();
        for (int i=0; i < autoNames.size(); i++) {
            try {
                al.add((new File(autoNames.get(i).toString())).toURL().getPath());
            }
            catch (Exception e) {
                // none
            }
        }
        return al;
    }

    /**
     * Return the Rars directory.
     * @return The Rars directory
     */
    public String getRarsDirectory() {
        String sRet = null;
        try {
            sRet = (new File(RARS_DIR)).toURL().getPath();
        }
        catch (Exception e) {
            // none
        }
        return sRet;
    }

    //--------------------------------------------------------------
    //           Private Methods
    //--------------------------------------------------------------

    /**
     * Add recursively the rars of the specified directory.
     * If the dir has a relative path, this path is relative from where
     * the Application Server is launched. If the dir is not found it will
     * be searched in $JONAS_BASE/rars/ directory.
     * @param dirPath the path to the directory containing the rars to
     * load.
     */
    private void addRars(String dirPath) {
        boolean found = false;

        // Look the directory relative to where the JOnAS server is launch or
        // if the path is absolute.
        File dir = new File(dirPath);
        found = dir.isDirectory();

        if (!found) {
            // Look the directory relative to the $JONAS_BASE/rars directory
            dir = new File(RARS_DIR + File.separator + dirPath);
            found = dir.isDirectory();
        }

        if (found) {
            autoNames.add(dir);
            addRarsFrom(dir);
        } else {
            String err = "Warning: Cannot load dir: '" + dirPath + "' ";
            err += "is not a directory or directory doesn't exist";
            logger.log(BasicLevel.WARN, err);
        }
    }

    /**
     * Add the rars of the specified directory.
     * @param dir the directory from which the rars are loaded.
     * @throws ResourceServiceException if the argument is not a directory
     */
    private void addRarsFrom(File dir) throws ResourceServiceException {
        try {
            if (dir.isDirectory()) {
                File[] files = dir.listFiles();
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        if (files[i].getPath().toLowerCase().endsWith(".rar")) {
                            resourceNames.add(files[i].getCanonicalPath());
                        }
                    } else {
                        autoNames.add(files[i]);
                        addRarsFrom(files[i]);
                    }
                }
            } else {
                String err = "Cannot load dir: '" + dir.getPath();
                err += "' is not a directory";
                logger.log(BasicLevel.ERROR, err);
                throw new ResourceServiceException(err);
            }
        } catch (IOException e) {
            String err = "Invalid file name '" + dir.getPath();
            logger.log(BasicLevel.ERROR, err);
            throw new ResourceServiceException(err, e);
        }
    }

    private boolean checkLogEnabled(String inp) {
        if (inp.equals("1") || inp.equalsIgnoreCase("on") ||
            inp.equalsIgnoreCase("t") || inp.equalsIgnoreCase("true") ||
            inp.equalsIgnoreCase("y") || inp.equalsIgnoreCase("yes"))
            return true;
         return false;
    }

    private Properties buildProp(ConfigProperty [] cfgRa, JonasConfigProperty [] cfgJonas)
    {
      Properties ret = new Properties();
      for (int i=0;i<cfgRa.length;i++){
        String val = cfgRa[i].getConfigPropertyValue().getValue()!=null ? cfgRa[i].getConfigPropertyValue().getValue() : "";
        ret.setProperty(cfgRa[i].getConfigPropertyName().getValue(), val);
      }
      for (int i=0;i<cfgJonas.length;i++){
        String val = cfgJonas[i].getJonasConfigPropertyValue().getValue()!=null ? cfgJonas[i].getJonasConfigPropertyValue().getValue() : "";
        ret.setProperty(cfgJonas[i].getJonasConfigPropertyName().getValue(), val);
      }

      return ret;
    }

    private ConfigProperty [] cfgMerge(ConfigProperty [] cfgRa, JonasConfigProperty [] cfgJonas)
    {
      ConfigProperty [] configs = cfgRa;

      for (int i=0;i<cfgJonas.length;i++){
          String name = cfgJonas[i].getJonasConfigPropertyName().getValue();
          String val = cfgJonas[i].getJonasConfigPropertyValue().getValue();
          if (val != null) {
              for (int j=0;j<configs.length;j++) {
                  if (name.equalsIgnoreCase(configs[j].getConfigPropertyName().getValue())) {
                      if (configs[j].getConfigPropertyValue() == null) {
                          ConfigPropertyValue cfgValue = new ConfigPropertyValueImpl();
                          cfgValue.setValue(val);
                          configs[j].setConfigPropertyValue(cfgValue);
                      }
                      else {
                          configs[j].getConfigPropertyValue().setValue(val);
                      }
                      break;
                  }
              }
          }
      }

      return configs;
    }
    /**
     * get the ResourceAdapter Connector object matching the jndiname
     */
    private Connector getConnector(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.racon );
        }
    }

    /**
     * get the JonasResource object matching the jndiname
     */
    private JonasResource getJonasResource(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.jonasra );
        }
    }

    /**
     * get the ResourceAdapter ConfigProperty [] object matching the jndiname
     */
    private ConfigProperty [] getMergeConfigs(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.cfgRaJonas );
        }
    }
}


