/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JonasTreeBuilder.java,v 1.39 2003/10/16 12:09:48 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.HashMap;

import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.util.MessageResources;

import org.objectweb.jonas.webapp.taglib.TreeBuilder;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.service.container.ContainerItem;
import org.objectweb.jonas.webapp.jonasadmin.service.container.ContainerItemByFile;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItem;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItemByNameComparator;
import org.objectweb.jonas.webapp.jonasadmin.catalina.ConnectorItem;
import org.objectweb.jonas.webapp.jonasadmin.catalina.ConnectorItemByPort;
import org.objectweb.jonas.webapp.jonasadmin.service.container.WebAppItem;
import org.objectweb.jonas.webapp.jonasadmin.service.container.WebAppItemByPathContext;
import org.objectweb.jonas.webapp.jonasadmin.logging.LoggerItem;

/**
 * Implementation of <code>TreeBuilder</code> that adds the nodes required
 * for administering the Jonas server.
 *
 * @author Michel-Ange ANTON
 * Contributors:
 * Adriana Danes : add nodes for JMS service
 */
public class JonasTreeBuilder implements TreeBuilder {
// ----------------------------------------------------- Instance Variables

// ---------------------------------------------------- TreeBuilder Methods

    /**
     * Add the required nodes to the specified <code>treeControl</code>
     * instance.
     *
     * @param treeControl The <code>TreeControl</code> to which we should
     *  add our nodes
     * @param servlet The controller servlet for the admin application
     * @param request The servlet request we are processing
     */
    public void buildTree(TreeControl treeControl, ActionServlet servlet
        , HttpServletRequest request) {
        try {
            TreeControlNode rootNode = treeControl.getRoot();
            MessageResources resources = (MessageResources) servlet.getServletContext().
                getAttribute(Action.MESSAGES_KEY);
            getServer(rootNode, resources, request);
        }
        catch (Throwable t) {
            t.printStackTrace(System.out);
        }
    }

// ------------------------------------------------------ Protected Methods

    /**
     * Append nodes for a defined Jonas server.
     *
     * @param p_RootNode Root node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServer(TreeControlNode p_RootNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Add server node
        String sServerName = (String) JonasAdminJmx.getJonasServerName();
        String sNodeLabel = p_Resources.getMessage("treenode.jonas.server") + " ( " + sServerName
            + " )";
        TreeControlNode serverNode = new TreeControlNode(sServerName, "icon/BalJonasTree.gif"
            , sNodeLabel, "EditJonasServer.do?select=" + URLEncoder.encode(sServerName), "content", true);
        p_RootNode.addChild(serverNode);
        // Add children
        getMonitoring(serverNode, p_Resources);
        getLogging(serverNode, p_Resources, p_Request);
        getProtocols(serverNode, p_Resources, p_Request);
        getServices(serverNode, p_Resources, p_Request);
        getDeployments(serverNode, p_Resources);
        getResources(serverNode, p_Resources);
        getSecurity(serverNode, p_Resources, p_Request);
    }

    /**
     * Append nodes for Monitoring.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *  messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getMonitoring(TreeControlNode p_ServerNode, MessageResources p_Resources)
        throws Exception {
        // Monitoring server Service node
        TreeControlNode node = new TreeControlNode(p_ServerNode.getName() + ";monitoring"
            , "icon/monitoring.gif", p_Resources.getMessage("treenode.jonas.server.monitoring")
            , "EditMonitoring.do", "content", false);
        p_ServerNode.addChild(node);
    }

    /**
     * Append nodes for Logging.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getLogging(TreeControlNode p_ServerNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Logging server Service node
        TreeControlNode node = new TreeControlNode(p_ServerNode.getName() + ";logging"
            , "icon/loggers.gif", p_Resources.getMessage("treenode.jonas.server.logging")
            , "ListLoggers.do", "content", false);
        p_ServerNode.addChild(node);
        // Add all loggers
        getLoggers(node, p_Resources, p_Request);
    }

    /**
     * Append nodes for each logger.
     *
     * @param p_LoggingNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getLoggers(TreeControlNode p_LoggingNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Get WhereAreYou instance in session
        WhereAreYou oWhere = (WhereAreYou) p_Request.getSession().getAttribute(WhereAreYou.
            SESSION_NAME);
        // Get list of loggers with Action name
        ArrayList al = JonasAdminJmx.getLoggers(p_Resources, oWhere, true);
        // Build the node of each logger
        LoggerItem oLog;
        String sIcon = "icon/loggers.gif";
        for (int i = 0; i < al.size(); i++) {
            oLog = (LoggerItem) al.get(i);
            if (LoggerItem.LOGGER_JONAS.equals(oLog.getType()) == true) {
                sIcon = "icon/logger_jonas.gif";
            }
            else if (LoggerItem.LOGGER_CATALINA_ACCESS.equals(oLog.getType()) == true) {
                sIcon = "icon/logger_http.gif";
            }
            // Add a logger node
            TreeControlNode node = new TreeControlNode(p_LoggingNode.getName() + ";" + oLog.getType()
                , sIcon, oLog.getName(), oLog.getForward(), "content", false);
            p_LoggingNode.addChild(node);
        }
    }

    /**
     * Append nodes for all defined services for the specified server.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServices(TreeControlNode p_ServerNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Services node
        TreeControlNode nodeAllServices = new TreeControlNode(p_ServerNode.getName() + ";services"
            , "icon/services.gif", p_Resources.getMessage("treenode.jonas.server.services")
            , "ListServices.do", "content", true);
        p_ServerNode.addChild(nodeAllServices);

        // Detail Service node
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.earService()) == true) {
            getServiceApp(nodeAllServices, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.ejbService()) == true) {
            getServiceEjbContainer(nodeAllServices, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.webContainerService()) == true) {
            getServiceWeb(nodeAllServices, p_Resources, p_Request);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.databaseService()) == true) {
            getServiceDatabase(nodeAllServices, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.resourceService()) == true) {
            getServiceResourceAdapter(nodeAllServices, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.transactionService()) == true) {
            getServiceTransaction(nodeAllServices, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.jmsService()) == true) {
            getServiceJms(nodeAllServices, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.mailService()) == true) {
            getServiceMail(nodeAllServices, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.securityService()) == true) {
            getServiceSecurity(nodeAllServices, p_Resources);
        }
    }

    /**
     * Append node for Ear service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceApp(TreeControlNode p_NodeServices, MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ";ear"
            , "icon/service_application.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.ear"), "ListAppContainers.do"
            , "content", false);
        p_NodeServices.addChild(nodeService);

        // Add App Containers nodes
        getAppContainers(nodeService, p_Resources);
    }

    /**
     * Append node for all App Containers.
     *
     * @param p_NodeParent Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getAppContainers(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        String sPath;
        String sFile;
        String sNodeName;
        ArrayList al = new ArrayList();

        // Get Container List
        Iterator itNames = ((java.util.Set) JonasManagementRepr.getAttribute(JonasObjectName.
            earService(), "EarNames")).iterator();
        while (itNames.hasNext()) {
            sPath = itNames.next().toString();
            sFile = JonasAdminJmx.extractFilename(sPath);
            if (sFile != null) {
                sNodeName = p_NodeParent.getName() + ";" + sFile;
                al.add(new ContainerItem(sFile, sPath, sNodeName));
            }
        }
        // Sort list
        Collections.sort(al, new ContainerItemByFile());
        // Build nodes
        ContainerItem oItem;
        for (int i = 0; i < al.size(); i++) {
            oItem = (ContainerItem) al.get(i);
            TreeControlNode node = new TreeControlNode(oItem.getName(), "icon/ear.gif"
                , oItem.getFile(), "EditEar.do?path=" + oItem.getPath(), "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append node for EJBContainer service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *  messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceEjbContainer(TreeControlNode p_NodeServices
        , MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName()
            + ";ejbContainers", "icon/service_ejb.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.ejbContainers")
            , "ListContainers.do", "content", false);
        p_NodeServices.addChild(nodeService);
        // Add Containers nodes
        getContainers(nodeService, p_Resources);
    }

    /**
     * Append node for all Containers.
     *
     * @param p_NodeParent Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getContainers(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        String sPath;
        String sFile;
        String sName;
        String sNodeName;
        ObjectName onContainer;
        ArrayList al = new ArrayList();

        // Get Container List
        Iterator itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allContainers()).iterator();
        while (itNames.hasNext()) {
            sPath = JonasAdminJmx.extractValueMbeanName("fname", itNames.next().toString());
            sFile = JonasAdminJmx.extractFilename(sPath);
            if (sFile != null) {
                sNodeName = p_NodeParent.getName() + ";" + sFile;
                al.add(new ContainerItem(sFile, sPath, sNodeName));
            }
        }
        // Sort list
        Collections.sort(al, new ContainerItemByFile());
        // Build nodes
        ContainerItem oItem;
        for (int i = 0; i < al.size(); i++) {
            oItem = (ContainerItem) al.get(i);
            TreeControlNode node = new TreeControlNode(oItem.getName(), "icon/jar.gif"
                , oItem.getFile(), "EditContainer.do?path=" + oItem.getPath(), "content", false);
            p_NodeParent.addChild(node);
            // Add ejb node
            getEjbs(node, p_Resources, oItem.getPath());
        }
    }

    /**
     * Append node for all Ejbs for a Container.
     *
     * @param p_NodeParent Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Filename Filename of container
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getEjbs(TreeControlNode p_NodeParent, MessageResources p_Resources
        , String p_Filename)
        throws Exception {
        String sName;
        Iterator itNames;
        ArrayList al = new ArrayList();

        // get BMP (Entity with Bean-Managed Persistence)
        itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbBmp(p_Filename)).iterator();
        while (itNames.hasNext()) {
            sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
            al.add(new EjbItem(EjbItem.EJB_BMP, sName, p_Filename));
        }

        // get CMP (Entity with Container-Managed Persistence)
        itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbCmp(p_Filename)).iterator();
        while (itNames.hasNext()) {
            sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
            al.add(new EjbItem(EjbItem.EJB_CMP, sName, p_Filename));
        }

        // get SBF (Stateful Session Bean)
        itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbSbf(p_Filename)).iterator();
        while (itNames.hasNext()) {
            sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
            al.add(new EjbItem(EjbItem.EJB_SBF, sName, p_Filename));
        }

        // get SBL (Stateless Session Bean)
        itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbSbl(p_Filename)).iterator();
        while (itNames.hasNext()) {
            sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
            al.add(new EjbItem(EjbItem.EJB_SBL, sName, p_Filename));
        }

        // get MDB (Message Driven Bean)
        itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbMdb(p_Filename)).iterator();
        while (itNames.hasNext()) {
            sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
            al.add(new EjbItem(EjbItem.EJB_MDB, sName, p_Filename));
        }

        // Sort by name
        Collections.sort(al, new EjbItemByNameComparator());

        // Add all node Ejb
        String sNodeName;
        String sAction;
        TreeControlNode nodeEjb;
        EjbItem oEjb;
        for (int i = 0; i < al.size(); i++) {
            oEjb = (EjbItem) al.get(i);
            sNodeName = p_NodeParent.getName() + ";" + oEjb.getName();
            sAction = "EditEjb" + oEjb.getTypeString() + ".do?type=" + oEjb.getTypeString()
                + "&name=" + URLEncoder.encode(oEjb.getName()) + "&file="
                + URLEncoder.encode(p_Filename);
            // Add node Ejb
            nodeEjb = new TreeControlNode(sNodeName, "icon/bean.gif", oEjb.getName(), sAction
                , "content", false);
            p_NodeParent.addChild(nodeEjb);
        }
    }

    /**
     * Append node for Database service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *  messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceDatabase(TreeControlNode p_NodeServices, MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ";database"
            , "icon/service_database.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.database"), "ListDatabases.do"
            , "content", false);
        p_NodeServices.addChild(nodeService);
        // Add Datasources nodes
        getDatasources(nodeService, p_Resources);
    }

    /**
     * Append node for all deployed Datasources.
     *
     * @param p_NodeParent Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getDatasources(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        String sPath;
        String sFile;
        String sNodeName;
        ArrayList al = new ArrayList();

        // Get List
        Iterator itNames = JonasAdminJmx.getDatasourceFilesDeployed().iterator();
        while (itNames.hasNext()) {
            al.add(itNames.next().toString());
        }
        // Sort list
        Collections.sort(al);
        // Build nodes
        for (int i = 0; i < al.size(); i++) {
            TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";"
                + al.get(i).toString(), "icon/Datasource.gif", al.get(i).toString()
                , "EditDatasource.do?name=" + al.get(i).toString(), "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append node for Web service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceWeb(TreeControlNode p_NodeServices, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ";web"
            , "icon/service_web.gif", p_Resources.getMessage("treenode.jonas.server.services.web")
            , "ListWebContainers.do", "content", false);
        p_NodeServices.addChild(nodeService);
        // Add Web Containers nodes
        getWebContainers(nodeService, p_Resources, p_Request);
    }

    /**
     * Append node for all Web Containers.
     *
     * @param p_NodeParent Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getWebContainers(TreeControlNode p_NodeParent, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        WhereAreYou oWhere = (WhereAreYou) p_Request.getSession().getAttribute(WhereAreYou.
            SESSION_NAME);

        ArrayList al = new ArrayList();
        // Detect Catalina items present
        if (oWhere.isCatalinaServer() == true) {
            // Get the list of Catalina contexts
            Iterator itOnContexts = JonasAdminJmx.getListMbean(CatalinaObjectName.catalinaContexts(
                oWhere.getCurrentCatalinaServiceName()
                , oWhere.getCurrentCatalinaDefaultHostName())).iterator();
            while (itOnContexts.hasNext()) {
                al.add(new WebAppItem(((ObjectName) itOnContexts.next()).getKeyProperty("path")));
            }
        }
        // Detect Jetty items present
        else if (oWhere.isJettyServer() == true) {
            ObjectName on;
            String sPathContext;
            HashMap hWebAppItems = new HashMap();

            // Get the list of Jetty contexts
            Iterator itOnContexts = JonasAdminJmx.getListMbean(JettyObjectName.jettyContexts()).
                iterator();
            while (itOnContexts.hasNext()) {
                on = (ObjectName) itOnContexts.next();
                sPathContext = on.getKeyProperty("context");
                if (sPathContext != null) {
                    if (hWebAppItems.containsKey(sPathContext) == false) {
                        hWebAppItems.put(sPathContext, new WebAppItem(sPathContext, on.toString()));
                    }
                }
            }
            // Prepare to display
            al.addAll(hWebAppItems.values());
        }
        // Sort list
        Collections.sort(al, new WebAppItemByPathContext());
        // Build nodes
        WebAppItem oItem;
        for (int i = 0; i < al.size(); i++) {
            oItem = (WebAppItem) al.get(i);
            TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";"
                + oItem.getPathContext(), "icon/war.gif", oItem.getPathContextLabel()
                , "EditWebApp.do?context=" + oItem.getPathContext(), "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append node for Transaction service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *  messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceTransaction(TreeControlNode p_NodeServices
        , MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ";transaction"
            , "icon/service_transaction.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.transaction")
            , "EditJtmService.do", "content", false);
        p_NodeServices.addChild(nodeService);
    }

    /**
     * Append node for Jms service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceJms(TreeControlNode p_NodeServices, MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ";jms"
            , "icon/service_jms.gif", p_Resources.getMessage("treenode.jonas.server.services.jms")
            , "EditJmsService.do", "content", false);
        p_NodeServices.addChild(nodeService);
        // Add JMS administred objects (resources)
        getJmsResources(nodeService, p_Resources);
    }

    /**
     * Append 3 subnodes for the JMS resource types + subnodes for all the ressources
     *
     * @param p_NodeParent JMS service node
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getJmsResources(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        // Add node for Connection Factories
        TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";cf"
            , "icon/jms_connector.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.jms.cf")
            , "ListConnectionFactories.do", "content", false);
        p_NodeParent.addChild(node);
        getJmsConnectionFactories(node, p_Resources);

        // Add node for Queues
        node = new TreeControlNode(p_NodeParent.getName() + ";queue", "icon/jms_queue.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.jms.queues"), "ListQueues.do"
            , "content", false);
        p_NodeParent.addChild(node);
        getJmsQueues(node, p_Resources);

        // Add node for Topics
        node = new TreeControlNode(p_NodeParent.getName() + ";topic", "icon/jms_topic.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.jms.topics"), "ListTopics.do"
            , "content", false);
        p_NodeParent.addChild(node);
        getJmsTopics(node, p_Resources);
    }

    /**
     * Append subnodes for ConnectionFactory resources
     *
     * @param p_NodeParent CFs subnode
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getJmsConnectionFactories(TreeControlNode p_NodeParent
        , MessageResources p_Resources)
        throws Exception {
        // Get the 3 connection factories created by default:
        // - default ConnectionFactory , default Topic ConnectionFactory, default Queue ConnectionFactory,
        // This corresponds to the current implementation of the JMS service ..... ???
        int nbConnFacts = 3;
        String cfNames[] = new String[3];
        ObjectName jmsServMB = JonasObjectName.jmsService();
        cfNames[0] = (String) JonasManagementRepr.getAttribute(jmsServMB
            , "DefaultConnectionFactoryName");
        cfNames[1] = (String) JonasManagementRepr.getAttribute(jmsServMB
            , "DefaultTopicConnectionFactoryName");
        cfNames[2] = (String) JonasManagementRepr.getAttribute(jmsServMB
            , "DefaultQueueConnectionFactoryName");
        TreeControlNode node;
        for (int i = 0; i < nbConnFacts; i++) {
            node = new TreeControlNode(p_NodeParent.getName() + ";" + cfNames[i]
                , "icon/jms_connector.gif", cfNames[i]
                , "PresentConnectionFactory.do?name=" + cfNames[i], "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append subnodes node for Queue destinations
     *
     * @param p_NodeParent Queue subnode
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getJmsQueues(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        // Get queues list
        Iterator itNames = ((java.util.Set) JonasManagementRepr.getAttribute(JonasObjectName.
            jmsService(), "AllJmsQueueDestinationNames")).iterator();
        while (itNames.hasNext()) {
            String sNodeName = (String) itNames.next();
            String sQueueName = sNodeName;
            // Add node for each queue
            TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";" + sNodeName
                , "icon/jms_queue.gif", sQueueName, "PresentQueue.do?name=" + sQueueName, "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append subnodes node for Topic destinations
     *
     * @param p_NodeParent Topic subnode
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getJmsTopics(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        // Get topics list
        Iterator itNames = ((java.util.Set) JonasManagementRepr.getAttribute(JonasObjectName.
            jmsService(), "AllJmsTopicDestinationNames")).iterator();
        while (itNames.hasNext()) {
            String sNodeName = (String) itNames.next();
            String sTopicName = sNodeName;
            // Add node for each topic
            TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";" + sNodeName
                , "icon/jms_topic.gif", sTopicName, "PresentTopic.do?name=" + sTopicName, "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append node for Mail service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *  messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceMail(TreeControlNode p_NodeServices, MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ";mail"
            , "icon/service_mail.gif", p_Resources.getMessage("treenode.jonas.server.services.mail")
            , "EditMailService.do", "content", false);
        p_NodeServices.addChild(nodeService);
        // Add nodes for MailFactories
        getAllSessionMailFactories(nodeService, p_Resources);
        getAllMimePartDSMailFactories(nodeService, p_Resources);
    }

    public void getAllSessionMailFactories(TreeControlNode p_NodeParent
        , MessageResources p_Resources)
        throws Exception {
        // Add parent node for all the Session Mail Factories
        TreeControlNode nodeSessionMailFactories = new TreeControlNode(p_NodeParent.getName()
            + ";session", "icon/mail_session.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.mail.session")
            , "ListSessionMailFactories.do", "content", false);
        p_NodeParent.addChild(nodeSessionMailFactories);
        // Add a node for each Session Mail Factory
        getSessionMailFactories(nodeSessionMailFactories, p_Resources);
    }

    public void getAllMimePartDSMailFactories(TreeControlNode p_NodeParent
        , MessageResources p_Resources)
        throws Exception {
        // Add parent node for all the Session Mail Factories
        TreeControlNode nodeMimePartMailFactories = new TreeControlNode(p_NodeParent.getName()
            + ";mimepart", "icon/mail_mime.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.mail.mimepart")
            , "ListMimePartMailFactories.do", "content", false);
        p_NodeParent.addChild(nodeMimePartMailFactories);
        // Add a node for each Session Mail Factory
        getMimePartMailFactories(nodeMimePartMailFactories, p_Resources);
    }

    protected void getSessionMailFactories(TreeControlNode p_NodeParent
        , MessageResources p_Resources)
        throws Exception {
        String sPath;
        String sFile;
        String sNodeName;
        ArrayList al = new ArrayList();

        // Get List
        Iterator itNames = JonasAdminJmx.getSessionMailFactoriesDeployed().iterator();
        while (itNames.hasNext()) {
            al.add(itNames.next().toString());
        }
        // Sort list
        Collections.sort(al);
        // Build nodes
        for (int i = 0; i < al.size(); i++) {
            TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";"
                + al.get(i).toString(), "icon/mail_session.gif", al.get(i).toString()
                , "EditSessionMailFactory.do?name=" + al.get(i).toString(), "content", false);
            p_NodeParent.addChild(node);
        }
    }

    protected void getMimePartMailFactories(TreeControlNode p_NodeParent
        , MessageResources p_Resource)
        throws Exception {
        String sPath;
        String sFile;
        String sNodeName;
        ArrayList al = new ArrayList();

        // Get List
        Iterator itNames = JonasAdminJmx.getMimeMailPartFactoriesDeployed().iterator();
        while (itNames.hasNext()) {
            al.add(itNames.next().toString());
        }
        // Sort list
        Collections.sort(al);
        // Build nodes
        for (int i = 0; i < al.size(); i++) {
            TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";"
                + al.get(i).toString(), "icon/mail_mime.gif", al.get(i).toString()
                , "EditMimePartMailFactory.do?name=" + al.get(i).toString(), "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append node for Resource service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceResourceAdapter(TreeControlNode p_NodeServices
        , MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName()
            + ";resourceAdapter", "icon/service_resource.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.resource")
            , "ListResourcesAdapters.do", "content", false);
        p_NodeServices.addChild(nodeService);
        // Add Containers nodes
        getResourcesAdapters(nodeService, p_Resources);
    }

    /**
     * Append node for all Resources Adapters.
     *
     * @param p_NodeParent Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getResourcesAdapters(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        String sName;
        ObjectName on;
        ArrayList al = new ArrayList();

        // Get Container List
        Iterator it = JonasAdminJmx.getListMbean(JonasObjectName.resourceAdapters()).iterator();
        while (it.hasNext()) {
            on = (ObjectName) it.next();
            al.add(on.getKeyProperty("name"));
        }
        // Sort list
        Collections.sort(al);
        // Build nodes
        for (int i = 0; i < al.size(); i++) {
            sName = al.get(i).toString();
            TreeControlNode node = new TreeControlNode(p_NodeParent.getName() + ";" + sName
                , "icon/resourceAdapter.gif", sName, "EditResourceAdapter.do?name=" + sName
                , "content", false);
            p_NodeParent.addChild(node);
        }
    }

    /**
     * Append node for Security service for the JOnAS services.
     *
     * @param p_NodeServices Services node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *  messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getServiceSecurity(TreeControlNode p_NodeServices, MessageResources p_Resources)
        throws Exception {
        // Add Service node
        TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ";security"
            , "icon/service_security.gif"
            , p_Resources.getMessage("treenode.jonas.server.services.security")
            , "EditServiceSecurity.do", "content", false);
        p_NodeServices.addChild(nodeService);
    }

    /**
     * Append nodes for Deployments services for the specified server.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @exception Exception if an exception occurs building the tree
     */
    protected void getDeployments(TreeControlNode p_ServerNode, MessageResources p_Resources)
        throws Exception {
        // Deployments node
        TreeControlNode nodeDeployments = new TreeControlNode(p_ServerNode.getName()
            + ";deployments", "icon/deploy.gif"
            , p_Resources.getMessage("treenode.jonas.server.deployments"), null, "content", true);
        p_ServerNode.addChild(nodeDeployments);

        // Detail Service node
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.earService()) == true) {
            getDeploymentEar(nodeDeployments, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.ejbService()) == true) {
            getDeploymentEjb(nodeDeployments, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.webContainerService()) == true) {
            getDeploymentWebAppli(nodeDeployments, p_Resources);
        }
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.resourceService()) == true) {
            getDeploymentRar(nodeDeployments, p_Resources);
        }
    }

    /**
     * Append node for Ear deployment.
     *
     * @param p_NodeDeployments Deployment node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getDeploymentEar(TreeControlNode p_NodeDeployments, MessageResources p_Resources)
        throws Exception {
        // Add node
        TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + ";"
            + WhereAreYou.DEPLOYMENT_STRING_EAR, "icon/deploy_ear.gif"
            , p_Resources.getMessage("treenode.jonas.server.deployments."
            + WhereAreYou.DEPLOYMENT_STRING_EAR)
            , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_EAR, "content", false);
        p_NodeDeployments.addChild(oNode);
    }

    /**
     * Append node for Ejb deployment.
     *
     * @param p_NodeDeployments Deployment node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getDeploymentEjb(TreeControlNode p_NodeDeployments, MessageResources p_Resources)
        throws Exception {
        // Add node
        TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + ";"
            + WhereAreYou.DEPLOYMENT_STRING_JAR, "icon/deploy_jar.gif"
            , p_Resources.getMessage("treenode.jonas.server.deployments."
            + WhereAreYou.DEPLOYMENT_STRING_JAR)
            , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_JAR, "content", false);
        p_NodeDeployments.addChild(oNode);

        // Add list files
        //getDeploymentJarFiles(oNode, p_Resources);
    }

    /**
     * Append node for Ejb Files deployment.
     *
     * @param p_NodeParent Deployment node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getDeploymentEjbFiles(TreeControlNode p_NodeParent, MessageResources p_Resources)
        throws Exception {
        String sFile;
        Iterator it = JonasAdminJmx.getJarFilesDeployable().iterator();
        while (it.hasNext()) {
            sFile = it.next().toString();
            // Add node
            TreeControlNode oNode = new TreeControlNode(p_NodeParent.getName() + ";" + sFile
                , "icon/JonasQuestion.gif", sFile, null, "content", false);
            p_NodeParent.addChild(oNode);
        }

    }

    /**
     * Append node for Web deployment.
     *
     * @param p_NodeDeployments Deployment node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getDeploymentWebAppli(TreeControlNode p_NodeDeployments
        , MessageResources p_Resources)
        throws Exception {
        // Add node
        TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + ";"
            + WhereAreYou.DEPLOYMENT_STRING_WAR, "icon/deploy_war.gif"
            , p_Resources.getMessage("treenode.jonas.server.deployments."
            + WhereAreYou.DEPLOYMENT_STRING_WAR)
            , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_WAR, "content", false);
        p_NodeDeployments.addChild(oNode);
    }

    /**
     * Append node for Rar deployment.
     *
     * @param p_NodeDeployments Deployment node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getDeploymentRar(TreeControlNode p_NodeDeployments, MessageResources p_Resources)
        throws Exception {
        // Add node
        TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + ";"
            + WhereAreYou.DEPLOYMENT_STRING_RAR, "icon/deploy_rar.gif"
            , p_Resources.getMessage("treenode.jonas.server.deployments."
            + WhereAreYou.DEPLOYMENT_STRING_RAR)
            , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_RAR, "content", false);
        p_NodeDeployments.addChild(oNode);
    }

    /**
     * Append nodes for Resources services for the specified server.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @exception Exception if an exception occurs building the tree
     */
    protected void getResources(TreeControlNode p_ServerNode, MessageResources p_Resources)
        throws Exception {
        // Resources node
        TreeControlNode nodeDeployments = new TreeControlNode(p_ServerNode.getName() + ";resources"
            , "icon/resources.gif", p_Resources.getMessage("treenode.jonas.server.resources"), null
            , "content", true);
        p_ServerNode.addChild(nodeDeployments);

        // Detail Service node
        // - database resources
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.databaseService()) == true) {
            getResourceDatabase(nodeDeployments, p_Resources);
        }
        // - jms resources
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.jmsService()) == true) {
            getJmsResource(nodeDeployments, p_Resources);
        }
        // - mail resources
        if (JonasAdminJmx.hasMBeanName(JonasObjectName.mailService()) == true) {
            getMailResource(nodeDeployments, p_Resources);
        }
    }

    /**
     * Append nodes for Mail Resources for the specified server.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @exception Exception if an exception occurs building the tree
     */
    protected void getMailResource(TreeControlNode p_ServerNode, MessageResources p_Resources)
        throws Exception {
        // Mail resources node
        TreeControlNode nodeDeployments = new TreeControlNode(p_ServerNode.getName() + ";mail"
            , "icon/mail.gif", p_Resources.getMessage("treenode.jonas.server.resources.mail")
            , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_MAILFACTORY, "content", false);
        p_ServerNode.addChild(nodeDeployments);
    }

    /**
     * Append nodes for Jms Resource for the specified server.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @exception Exception if an exception occurs building the tree
     */
    protected void getJmsResource(TreeControlNode p_ServerNode, MessageResources p_Resources)
        throws Exception {
        // Jms resources node
        TreeControlNode nodeDeployments = new TreeControlNode(p_ServerNode.getName() + ";jms"
            , "icon/jms.gif", p_Resources.getMessage("treenode.jonas.server.resources.jms")
            , "ListJmsResources.do?", "content", false);
        p_ServerNode.addChild(nodeDeployments);
    }

    /**
     * Append nodes for Database Resource for the specified server.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @exception Exception if an exception occurs building the tree
     */
    protected void getResourceDatabase(TreeControlNode p_ServerNode, MessageResources p_Resources)
        throws Exception {
        // Database Resources node
        TreeControlNode nodeDeployments = new TreeControlNode(p_ServerNode.getName() + ";database"
            , "icon/Database.gif"
            , p_Resources.getMessage("treenode.jonas.server.resources.database")
            , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_DATASOURCE, "content", false);
        p_ServerNode.addChild(nodeDeployments);
    }

    /**
     * Append nodes for Security for the specified server.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     * @exception Exception if an exception occurs building the tree
     */
    protected void getSecurity(TreeControlNode p_ServerNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Detect JOnAS item presents
        boolean bJonasSecurityService = JonasAdminJmx.hasMBeanName(JonasObjectName.securityService());
        // Detect Catalina items present
        WhereAreYou oWhere = (WhereAreYou) p_Request.getSession().getAttribute(WhereAreYou.
            SESSION_NAME);
        boolean bCatalinaRealm = false;
        if (oWhere.isCatalinaServer() == true) {
            bCatalinaRealm = JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaRealm(oWhere.
                getCurrentCatalinaServiceName()));
        }
        // Add nodes
        if ((bJonasSecurityService == true) || (bCatalinaRealm == true)) {
            // Active action
            String sAction = null;
            if (bJonasSecurityService == true) {
                sAction = "ListRealms.do?realm=all";
            }
            else if (bCatalinaRealm == true) {
                sAction = "EditSecurityCatalinaRealm.do";
            }
            // Security node
            TreeControlNode nodeSecurity = new TreeControlNode(p_ServerNode.getName() + ";security"
                , "icon/security.gif", p_Resources.getMessage("treenode.jonas.server.security")
                , sAction, "content", false);
            p_ServerNode.addChild(nodeSecurity);
            // Detail Security node for JOnAS service
            if (bJonasSecurityService == true) {
                getSecurityFactories(nodeSecurity, p_Resources);
            }
        }
    }

    /**
     * Append node for the Security catalina realm
     *
     * @param p_SecurityNode Catalina Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getSecurityCatalinaRealm(TreeControlNode p_SecurityNode
        , MessageResources p_Resources, HttpServletRequest p_Request)
        throws Exception {

        // Add node
        TreeControlNode oNode = new TreeControlNode(p_SecurityNode.getName() + ";catalina.realm"
            , "icon/security.gif"
            , p_Resources.getMessage("treenode.jonas.server.security.catalina.realm"), null
            , "content", false);
        p_SecurityNode.addChild(oNode);
    }

    /**
     * Append nodes for Security factories for the specified server.
     *
     * @param p_SecurityNode Security node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @exception Exception if an exception occurs building the tree
     */
    public void getSecurityFactories(TreeControlNode p_SecurityNode, MessageResources p_Resources)
        throws Exception {
        ArrayList al = null;
        String sName = null;
        TreeControlNode nodeSecurityFactories = null;
        TreeControlNode nodeSecurityFactory = null;

        // Security Memory factories
        al = JonasAdminJmx.getSecurityMemoryFactories();
        if (al.size() > 0) {
            // Security factories node
            nodeSecurityFactories = new TreeControlNode(p_SecurityNode.getName()
                + ";factory.memory", "icon/security_memory.gif"
                , p_Resources.getMessage("treenode.jonas.server.security.factory.memory")
                , "ListRealms.do?realm=memory", "content", false);
            p_SecurityNode.addChild(nodeSecurityFactories);
            // List Security factories
            for (int i = 0; i < al.size(); i++) {
                sName = al.get(i).toString();
                // Security factory node
                nodeSecurityFactory = new TreeControlNode(nodeSecurityFactories.getName() + ";"
                    + sName, "icon/security_memory.gif", sName
                    , "EditMemoryRealm.do?resource=" + sName, "content", false);
                nodeSecurityFactories.addChild(nodeSecurityFactory);
                // Add detail to memory security node
                //getSecurityMemoryFactory(nodeSecurityFactory, p_Resources, sName);
            }
        }
        // Security Datasource factories
        al = JonasAdminJmx.getSecurityDatasourceFactories();
        if (al.size() > 0) {
            // Security factories node
            nodeSecurityFactories = new TreeControlNode(p_SecurityNode.getName()
                + ";factory.datasource", "icon/security_database.gif"
                , p_Resources.getMessage("treenode.jonas.server.security.factory.datasource")
                , "ListRealms.do?realm=datasource", "content", false);
            p_SecurityNode.addChild(nodeSecurityFactories);
            // List Security factories
            for (int i = 0; i < al.size(); i++) {
                sName = al.get(i).toString();
                // Security factory node
                nodeSecurityFactory = new TreeControlNode(nodeSecurityFactories.getName() + ";"
                    + sName, "icon/security_database.gif", sName
                    , "EditDatasourceRealm.do?resource=" + sName, "content", false);
                nodeSecurityFactories.addChild(nodeSecurityFactory);
            }
        }
        // Security Ldap factories
        al = JonasAdminJmx.getSecurityLdapFactories();
        if (al.size() > 0) {
            // Security factories node
            nodeSecurityFactories = new TreeControlNode(p_SecurityNode.getName() + ";factory.ldap"
                , "icon/security_ldap.gif"
                , p_Resources.getMessage("treenode.jonas.server.security.factory.ldap")
                , "ListRealms.do?realm=ldap", "content", false);
            p_SecurityNode.addChild(nodeSecurityFactories);
            // List Security factories
            for (int i = 0; i < al.size(); i++) {
                sName = al.get(i).toString();
                // Security factory node
                nodeSecurityFactory = new TreeControlNode(nodeSecurityFactories.getName() + ";"
                    + sName, "icon/security_ldap.gif", sName, "EditLdapRealm.do?resource=" + sName
                    , "content", false);
                nodeSecurityFactories.addChild(nodeSecurityFactory);
            }
        }
    }

    /**
     * Append nodes for Memory Security Factory for the specified server.
     *
     * @param p_SecurityMemoryNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_ResourceFactory The name of resource
     * @exception Exception if an exception occurs building the tree
     */
    protected void getSecurityMemoryFactory(TreeControlNode p_SecurityMemoryNode
        , MessageResources p_Resources, String p_ResourceFactory)
        throws Exception {
        TreeControlNode node = null;
        // Users node
        node = new TreeControlNode(p_SecurityMemoryNode.getName() + ";users"
            , "icon/security_memory.gif"
            , p_Resources.getMessage("treenode.jonas.server.security.factory.memory.users")
            , "EditMemoryRealmUsers.do?resource=" + p_ResourceFactory, "content", false);
        p_SecurityMemoryNode.addChild(node);
        // Groups node
        node = new TreeControlNode(p_SecurityMemoryNode.getName() + ";groups"
            , "icon/security_memory.gif"
            , p_Resources.getMessage("treenode.jonas.server.security.factory.memory.groups")
            , "EditMemoryRealmGroups.do?resource=" + p_ResourceFactory, "content", false);
        p_SecurityMemoryNode.addChild(node);
        // Roles node
        node = new TreeControlNode(p_SecurityMemoryNode.getName() + ";roles"
            , "icon/security_memory.gif"
            , p_Resources.getMessage("treenode.jonas.server.security.factory.memory.roles")
            , "EditMemoryRealmRoles.do?resource=" + p_ResourceFactory, "content", false);
        p_SecurityMemoryNode.addChild(node);
    }

    /**
     * Append nodes for the Servlet server
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getProtocols(TreeControlNode p_ServerNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Catalina server
        if (JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaServer()) == true) {
            getCatalinaProtocols(p_ServerNode, p_Resources, p_Request);
        }
    }

    /**
     * Append nodes for the Catalina server
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getCatalinaProtocols(TreeControlNode p_ServerNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        // Add Catalina server node
        TreeControlNode catalinaNode = new TreeControlNode(p_ServerNode.getName() + ";protocols"
            , "icon/protocol.gif", p_Resources.getMessage("treenode.jonas.server.protocols")
            , "ListCatalinaConnectors.do", "content", false);
        p_ServerNode.addChild(catalinaNode);
        // Add Connectors nodes
        getCatalinaConnector(catalinaNode, p_Resources, p_Request);
    }

    /**
     * Append node for the Connector catalina server
     *
     * @param p_CatalinaServerNode Catalina Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getCatalinaConnector(TreeControlNode p_CatalinaServerNode
        , MessageResources p_Resources, HttpServletRequest p_Request)
        throws Exception {

        // Add Connector node
        TreeControlNode connectorNode = new TreeControlNode(p_CatalinaServerNode.getName()
            + ";connectors", "icon/connectors.gif"
            , p_Resources.getMessage("treenode.jonas.server.protocols.connectors")
            , "ListCatalinaConnectors.do", "content", false);
        p_CatalinaServerNode.addChild(connectorNode);

        getCatalinaDetailConnectors(connectorNode, p_Resources, p_Request);

    }

    /**
     * Append nodes for all the Connectors catalina server
     *
     * @param p_ConnectorNode Connector node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getCatalinaDetailConnectors(TreeControlNode p_ConnectorNode
        , MessageResources p_Resources, HttpServletRequest p_Request)
        throws Exception {

        // Get WhereAreYou
        WhereAreYou oWhere = (WhereAreYou) p_Request.getSession().getAttribute(WhereAreYou.
            SESSION_NAME);

        // Get the catalina service name
        String sServiceName = oWhere.getCurrentCatalinaServiceName();
        // Get the list of connectors
        ArrayList al = new ArrayList();
        Iterator it = JonasAdminJmx.getListMBeanName(CatalinaObjectName.catalinaConnectors(
            sServiceName)).iterator();
        while (it.hasNext()) {
            al.add(new ConnectorItem(new ObjectName(it.next().toString())
                , oWhere.getApplicationServerPort()));
        }
        // Sort
        Collections.sort(al, new ConnectorItemByPort());
        // Build the nodes
        TreeControlNode node;
        ConnectorItem oConnector;
        String sLabel;
        for (int i = 0; i < al.size(); i++) {
            oConnector = (ConnectorItem) al.get(i);
            node = new TreeControlNode(p_ConnectorNode.getName() + ";" + sServiceName + ";"
                + oConnector.getPort() + "." + oConnector.getAddress(), "icon/connector.gif"
                , oConnector.getLabel()
                , "EditCatalinaConnector.do?select=" + oConnector.getObjectName(), "content", false);
            p_ConnectorNode.addChild(node);
        }
    }
}
