/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WhereAreYou.java,v 1.16 2003/10/16 12:09:48 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionServlet;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;

/**
 * @author Michel-Ange ANTON
 */
public class WhereAreYou {

// --------------------------------------------------------- Public Constants

    public final static String SESSION_NAME = "WhereAreYou";

    public final static String TAGS_IMAGES_ROOT_KEY = "tagsimagesroot";
    public final static String CONTEXT_MACRO = "${CONTEXT}";
    public final static String TREE_PAGE_REFRESH = "viewTree.do?select=";

    public final static int DEPLOYMENT_EAR = 1;
    public final static int DEPLOYMENT_JAR = 2;
    public final static int DEPLOYMENT_WAR = 3;
    public final static int DEPLOYMENT_RAR = 4;
    public final static int DEPLOYMENT_DATASOURCE = 5;
    public final static int DEPLOYMENT_JMS = 6;
    public final static int DEPLOYMENT_MAIL = 7;

    public final static String DEPLOYMENT_STRING_EAR = "ear";
    public final static String DEPLOYMENT_STRING_JAR = "jar";
    public final static String DEPLOYMENT_STRING_WAR = "war";
    public final static String DEPLOYMENT_STRING_RAR = "rar";
    public final static String DEPLOYMENT_STRING_DATASOURCE = "datasource";
    public final static String DEPLOYMENT_STRING_JMS = "jms";
    public final static String DEPLOYMENT_STRING_MAILFACTORY = "mail";

// --------------------------------------------------------- Properties variable

    private String imagesRoot = null;
    private TreeControl treeControl = null;
    private boolean treeToRefresh = false;

    private String applicationJonasServerName = null;
    private int applicationServerPort = 0;
    private String applicationContextPath = null;

    private String currentJonasServerName = null;
    private boolean currentApplicationJonasServerName = false;

    private boolean catalinaServer = false;
    private String currentCatalinaServiceName = null;
    private String currentCatalinaEngineName = null;
    private String currentCatalinaDefaultHostName = null;
    private boolean jettyServer = false;

    /**
     * Type of deployment in progress : JAR, EAR, WAR, RAR, DATASOURCE.
     */
    private int currentJonasDeploymentType = 0;

// --------------------------------------------------------- Constructor

    public WhereAreYou() {
        super();
    }

// --------------------------------------------------------- Public Methods

    /**
     * Initialize the properties in only reading.
     *
     * @param p_Servlet The instance of servlet to access to the parameter of the servlet config
     * @param p_Request The request to access to the HTTP informations
     */
    public void initialize(ActionServlet p_Servlet, HttpServletRequest p_Request) {
        // Tags Images Root directory
        String sImagesRoot = (String) p_Servlet.getServletConfig().getInitParameter(
            TAGS_IMAGES_ROOT_KEY);
        if (sImagesRoot != null) {
            sImagesRoot = JonasAdminJmx.replace(sImagesRoot, CONTEXT_MACRO
                , p_Request.getContextPath());
        }
        this.imagesRoot = sImagesRoot;
        // Initialize Servers properties
        refreshServers(p_Request);

        /*
         java.util.Properties oProps = System.getProperties();
         //oProps.list(System.out);
         String sKey;
         java.util.ArrayList alKey = new java.util.ArrayList(oProps.keySet());
         java.util.Collections.sort(alKey);
         java.util.Iterator it = alKey.iterator();
         System.out.println("------------------------------------------------------------");
         while (it.hasNext())
         {
            sKey = it.next().toString();
            System.out.println(sKey);
            System.out.println(oProps.getProperty(sKey));
            System.out.println("------------------------------------------------------------");
         }
         */

    }

    /**
     * Return the name of the selected node in the tree.
     *
     * @return The name
     */
    public String getSelectedNameNode() {
        if (treeControl != null) {
            TreeControlNode oNode = treeControl.getSelected();
            if (oNode != null) {
                return oNode.getName();
            }
        }
        return null;
    }

    /**
     * Return the URL to refresh the selected node in the tree.
     *
     * @param p_Request HTTP request
     * @param p_Response HTTP response
     * @return The URL
     */
    public String getUrlToRefreshSelectedNode(HttpServletRequest p_Request
        , HttpServletResponse p_Response) {
        if (treeControl != null) {
            TreeControlNode oNode = treeControl.getSelected();
            if (oNode != null) {
                // Disable refresh
                treeToRefresh = false;
                // Build URL refresh
                StringBuffer sb = new StringBuffer();
                sb.append(p_Request.getContextPath());
                sb.append("/");
                sb.append(TREE_PAGE_REFRESH);
                sb.append(oNode.getName());

                return p_Response.encodeURL(sb.toString());
            }
        }
        return null;
    }

    /**
     * Select the node in the tree.
     *
     * @param p_Name The name of node to select
     * @param p_Expand Expand parent branch
     */
    public void selectNameNode(String p_Name, boolean p_Expand) {
        if (treeControl != null) {
            // Changing node ?
            if (p_Name.equals(getSelectedNameNode()) == false) {
                // Change node
                treeControl.selectNode(p_Name);
                if (p_Expand == true) {
                    treeControl.expandSelectedParents();
                }
                treeToRefresh = true;
            }
            // Query node not found
            if ((getSelectedTreeControlNode() == null) && (p_Name != null) && (p_Name.length() > 0)) {
                // Try to select the parent node
                String s = p_Name.substring(0, p_Name.lastIndexOf(';'));
                if ((s != null) && (s.length() > 0)) {
                    selectNameNode(s, p_Expand);
                }
            }
        }
    }

    /**
     * Return the selected node in the tree.
     *
     * @return The selected node
     */
    public TreeControlNode getSelectedTreeControlNode() {
        if (treeControl != null) {
            return treeControl.getSelected();
        }
        return null;
    }

    /**
     * Return the state of current display of tree.
     *
     * @return True if the current tree is not correctly displaying
     */
    public boolean isTreeToRefresh() {
        return treeToRefresh;
    }

    /**
     * Change the state of current display of tree.
     *
     * @param treeToRefresh True to refresh
     */
    public void setTreeToRefresh(boolean treeToRefresh) {
        this.treeToRefresh = treeToRefresh;
    }

    /**
     * Refresh all the servers properties.
     *
     * @param p_Request The request to access to the HTTP informations
     */
    public void refreshServers(HttpServletRequest p_Request) {
        applicationServerPort = p_Request.getServerPort();
        applicationContextPath = p_Request.getContextPath();
        refreshCatalina(p_Request);
        refreshJetty(p_Request);
        refreshJonas(p_Request);
    }

    /**
     * Refresh the Catalina server properties.
     *
     * @param p_Request The request to access to the HTTP informations
     */
    public void refreshCatalina(HttpServletRequest p_Request) {
        resetCatalina();
        try {
            //catalinaServer = JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaServer());
            String sServer = getServletServerName().toLowerCase();
            catalinaServer = (sServer.indexOf("tomcat") > -1);

            if (catalinaServer == true) {
                // Service : the first !
                ObjectName on = JonasAdminJmx.getFirstMbean(CatalinaObjectName.catalinaServices());
                currentCatalinaServiceName = on.getKeyProperty("name");
                // Engine of found service
                on = JonasAdminJmx.getFirstMbean(CatalinaObjectName.catalinaEngine(
                    currentCatalinaServiceName));
                currentCatalinaEngineName = (String) JonasManagementRepr.getAttribute(on, "name");
                // Default Host
                currentCatalinaDefaultHostName = (String) JonasManagementRepr.getAttribute(on
                    , "defaultHost");
            }

        }
        catch (Exception e) {
            // none
        }
    }

    /**
     * Refresh the Jetty server properties.
     *
     * @param p_Request The request to access to the HTTP informations
     */
    public void refreshJetty(HttpServletRequest p_Request) {
        resetJetty();
        try {
            String sServer = getServletServerName().toLowerCase();
            jettyServer = (sServer.indexOf("jetty") > -1);
            if (jettyServer == true) {
                // none action
            }
        }
        catch (Exception e) {
            // none
        }
    }

    /**
     * Refresh the JOnAS server properties.
     *
     * @param p_Request The request to access to the HTTP informations
     */
    public void refreshJonas(HttpServletRequest p_Request) {
        resetJonas();
        try {
            // Get the name of the Jonas Server who is running the application
            org.objectweb.jonas.common.JProp oJProp = org.objectweb.jonas.common.JProp.getInstance();
            applicationJonasServerName = oJProp.getValue("jonas.name");
            // Get the name of the managed Jonas Server
            currentJonasServerName = JonasAdminJmx.getJonasServerName();
            // Detect if the current managed JOnAS server is the one where the application is running
            currentApplicationJonasServerName = applicationJonasServerName.equals(
                currentJonasServerName);
        }
        catch (Exception e) {
            // none
        }
    }

    /**
     * Return in a string the contents of the instance for each propertie.
     * Debug utility.
     *
     * @return The string contents
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("imagesRoot=[").append(imagesRoot).append("] ");
        sb.append("treeControl=[").append(treeControl).append("] ");
        sb.append("catalinaServer=[").append(catalinaServer).append("] ");
        return sb.toString();
    }

// --------------------------------------------------------- Protected Methods

    /**
     * Return the name of the servlet server.
     * @return The name of the servlet server.
     */
    protected String getServletServerName() {
        return (String) JonasManagementRepr.getAttribute(JonasObjectName.webContainerService()
            , "ServerName");
    }

    /**
     * Reset the Catalina server properties.
     */
    protected void resetCatalina() {
        catalinaServer = false;
        currentCatalinaServiceName = null;
    }

    /**
     * Reset the Jetty server properties.
     */
    protected void resetJetty() {
        jettyServer = false;
    }

    /**
     * Reset the JOnAS server properties.
     */
    protected void resetJonas() {
        applicationJonasServerName = null;
        currentJonasServerName = null;
        currentApplicationJonasServerName = false;
    }

// --------------------------------------------------------- Properties Methods

    public String getImagesRoot() {
        return imagesRoot;
    }

    public TreeControl getTreeControl() {
        return treeControl;
    }

    public void setTreeControl(TreeControl treeControl) {
        this.treeControl = treeControl;
    }

    public boolean isCatalinaServer() {
        return catalinaServer;
    }

    public String getCurrentCatalinaServiceName() {
        return currentCatalinaServiceName;
    }

    public boolean isCurrentApplicationJonasServerName() {
        return currentApplicationJonasServerName;
    }

    public int getCurrentJonasDeploymentType() {
        return currentJonasDeploymentType;
    }

    public void setCurrentJonasDeploymentType(int currentJonasDeploymentType) {
        this.currentJonasDeploymentType = currentJonasDeploymentType;
    }

    public int getApplicationServerPort() {
        return applicationServerPort;
    }

    public String getCurrentJonasServerName() {
        return currentJonasServerName;
    }

    public String getApplicationJonasServerName() {
        return applicationJonasServerName;
    }

    public String getApplicationContextPath() {
        return applicationContextPath;
    }

    public String getCurrentCatalinaDefaultHostName() {
        return currentCatalinaDefaultHostName;
    }

    public String getCurrentCatalinaEngineName() {
        return currentCatalinaEngineName;
    }

    public boolean isJettyServer() {
        return jettyServer;
    }
}
