/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ApplyConnectorAction.java,v 1.5 2003/09/10 08:45:43 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.catalina;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.webapp.jonasadmin.CatalinaObjectName;
import org.objectweb.jonas.webapp.jonasadmin.Jlists;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.jmx.JonasManagementRepr;

/**
 * @author Michel-Ange ANTON
 */
public class ApplyConnectorAction extends CatalinaBaseAction {

    /**
     * Signature for the <code>createStandardConnector</code> operation.
     */
    private String sa_CreateStandardConnectorTypes[] = {
        "java.lang.String", "java.lang.String", "int" // port
    };

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {
        // Next forward
        String sForward = "Catalina Connector";
        // Identify the requested action
        ConnectorForm oForm = (ConnectorForm) p_Form;
        String sAction = oForm.getAction();
        String sObjectName = oForm.getObjectName();
        String sConnectorType = oForm.getConnectorType();

        // Populate
        try {

            // Perform a "Create Connector" transaction (if requested)
            if ("create".equals(sAction) == true) {
                String operation = null;
                Object values[] = null;

                //CatalinaForm oCatalinaForm = getCatalinaForm();

                ObjectName onConnector = CatalinaObjectName.catalinaConnector(m_WhereAreYou.
                    getCurrentCatalinaServiceName(), oForm.getPortText(), oForm.getAddress());

                // Ensure that the requested connector name is unique
                if (JonasAdminJmx.hasMBeanName(onConnector) == true) {
                    m_Errors.add("connector", new ActionError("error.catalina.connector.exists"));
                    saveErrors(p_Request, m_Errors);
                    return (new ActionForward(p_Mapping.getInput()));
                }

                // Look up the Catalina MBeanFactory
                ObjectName onFactory = CatalinaObjectName.catalinaFactory();

                // Create a new Connector object
                values = new Object[3];
                // parent
                values[0] = CatalinaObjectName.catalinaService(m_WhereAreYou.
                    getCurrentCatalinaServiceName()).toString();
                values[1] = oForm.getAddress();
                values[2] = new Integer(oForm.getPortText());

                if ("HTTP".equalsIgnoreCase(sConnectorType)) {
                    operation = "createHttpConnector"; // HTTP
                }
                else if ("HTTPS".equalsIgnoreCase(sConnectorType)) {
                    operation = "createHttpsConnector"; // HTTPS
                }
                else {
                    operation = "createAjpConnector"; // AJP(HTTP)
                }

                sObjectName = (String) JonasManagementRepr.invoke(onFactory, operation, values
                    , sa_CreateStandardConnectorTypes);

                // refresh tree
                refreshTree(p_Request);
                // Force the node selected in tree
                m_WhereAreYou.selectNameNode(getTreeBranchName(1) + ";protocols;connectors;"
                    + m_WhereAreYou.getCurrentCatalinaServiceName() + ";" + oForm.getPortText()
                    + "." + oForm.getAddress(), true);

            }

            ObjectName oObjectName = new ObjectName(sObjectName);

            setIntegerAttribute(oObjectName, "debug", oForm.getDebugLvl());
            setIntegerAttribute(oObjectName, "acceptCount", oForm.getAcceptCountText());
            setIntegerAttribute(oObjectName, "connectionTimeout", oForm.getConnTimeOutText());
            setIntegerAttribute(oObjectName, "bufferSize", oForm.getBufferSizeText());
            setBooleanAttribute(oObjectName, "enableLookups", oForm.isEnableLookups());
            setIntegerAttribute(oObjectName, "redirectPort", oForm.getRedirectPortText());
            setIntegerAttribute(oObjectName, "minProcessors", oForm.getMinProcessorsText());
            setIntegerAttribute(oObjectName, "maxProcessors", oForm.getMaxProcessorsText());

            // proxy name and port do not exist for AJP connector
            if ("AJP".equalsIgnoreCase(sConnectorType) == false) {
                setStringAttribute(oObjectName, "proxyName", oForm.getProxyName());
                setIntegerAttribute(oObjectName, "proxyPort", oForm.getProxyPortText());
            }

            // HTTPS specific properties
            if ("HTTPS".equalsIgnoreCase(sConnectorType) == true) {
                setBooleanAttribute(oObjectName, "clientAuth", oForm.isClientAuthentication());
                setStringAttribute(oObjectName, "keystoreFile", oForm.getKeyStoreFileName());
                setStringAttribute(oObjectName, "keystorePass", oForm.getKeyStorePassword());
            }

            if ("create".equals(sAction) == true) {
                m_Session.removeAttribute(p_Mapping.getAttribute());
                sForward = "ActionListCatalinaConnectors";
            }

            // Save in configuration file
            if (oForm.isSave() == true) {
                oForm.setSave(false);
                p_Request.setAttribute("forward", sForward);
                sForward = "ActionEditServletServer";
                //ObjectName on = CatalinaObjectName.catalinaServer();
                //JonasManagementRepr.invoke(on, "store", null, null);
            }
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }

        // Forward to the connector display page or the list if create
        return p_Mapping.findForward(sForward);
    }
}
