/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EditConnectorAction.java,v 1.3 2003/08/28 16:24:03 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.catalina;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.webapp.jonasadmin.Jlists;

/**
 * @author Michel-Ange ANTON
 */
public class EditConnectorAction extends CatalinaBaseAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Set up the object names of the MBeans we are manipulating
        ObjectName oObjectName = null;
        try {
            oObjectName = new ObjectName(p_Request.getParameter("select"));
        }
        catch (Exception e) {
            m_Errors.add("select", new ActionError("error.catalina.connector.select.bad"
                , p_Request.getParameter("select")));
            addGlobalError(e);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }

        // Fill in the form values for display and editing
        ConnectorForm oForm = new ConnectorForm();
        m_Session.setAttribute("catalinaConnectorForm", oForm);
        oForm.setAction("edit");
        oForm.setObjectName(oObjectName.toString());
        oForm.setDebugLvlVals(Jlists.getDebugLevels());
        oForm.setBooleanVals(Jlists.getBooleanValues());

        // Populate
        try {

            // General properties
            oForm.setScheme(getStringAttribute(oObjectName, "scheme"));
            String sHandlerClassName = getStringAttribute(oObjectName, "protocolHandlerClassName");
            int period = sHandlerClassName.lastIndexOf('.');
            String sHandlerType = sHandlerClassName.substring(period + 1);
            oForm.setConnectorType("HTTPS");
            if ("JkCoyoteHandler".equalsIgnoreCase(sHandlerType)) {
                oForm.setConnectorType("AJP");
            }
            else if (("Http11Protocol".equalsIgnoreCase(sHandlerType) == true)
                && ("http".equalsIgnoreCase(oForm.getScheme()) == true)) {
                oForm.setConnectorType("HTTP");
            }

            oForm.setAcceptCountText(toStringIntegerAttribute(oObjectName, "acceptCount"));
            oForm.setConnTimeOutText(toStringIntegerAttribute(oObjectName, "connectionTimeout"));
            oForm.setDebugLvl(toStringIntegerAttribute(oObjectName, "debug"));
            oForm.setBufferSizeText(toStringIntegerAttribute(oObjectName, "bufferSize"));
            oForm.setEnableLookups(getBooleanAttribute(oObjectName, "enableLookups"));
            oForm.setAddress(getStringAttribute(oObjectName, "address"));

            // Ports
            oForm.setPortText(toStringIntegerAttribute(oObjectName, "port"));
            oForm.setRedirectPortText(toStringIntegerAttribute(oObjectName, "redirectPort"));

            // Processors
            oForm.setMinProcessorsText(toStringIntegerAttribute(oObjectName, "minProcessors"));
            oForm.setMaxProcessorsText(toStringIntegerAttribute(oObjectName, "maxProcessors"));

            // Supported by HTTP and HTTPS only
            if (!("AJP".equalsIgnoreCase(oForm.getConnectorType()))) {
                oForm.setProxyName(getStringAttribute(oObjectName, "proxyName"));
                oForm.setProxyPortText(toStringIntegerAttribute(oObjectName, "proxyPort"));
            }

            if ("HTTPS".equalsIgnoreCase(oForm.getConnectorType())) {
                // Initialize rest of variables.
                // These are set only for SSL connectors.
                oForm.setClientAuthentication(getBooleanAttribute(oObjectName, "clientAuth"));
                oForm.setKeyStoreFileName(getStringAttribute(oObjectName, "keystoreFile"));
                oForm.setKeyStorePassword(getStringAttribute(oObjectName, "keystorePass"));
            }
            // Force the node selected in tree
            m_WhereAreYou.selectNameNode(getTreeBranchName(1) + ";protocols;connectors;"
                + m_WhereAreYou.getCurrentCatalinaServiceName() + ";" + oForm.getPortText() + "."
                + oForm.getAddress(), true);
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the connector display page
        return (p_Mapping.findForward("Catalina Connector"));
    }
}
