/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BaseDeployAction.java,v 1.19 2003/10/16 12:09:49 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.deploy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;

import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;
import org.objectweb.jonas.webapp.jonasadmin.JonasTreeBuilder;
import org.objectweb.jonas.webapp.jonasadmin.MBeanTreeBuilder;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;

/**
 * @author Michel-Ange ANTON
 */

abstract public class BaseDeployAction extends JonasBaseAction {

// --------------------------------------------------------- Protected Methods

    protected int getCurrentJonasDeployment()
        throws Exception {
        int iRet = 0;
        iRet = m_WhereAreYou.getCurrentJonasDeploymentType();
        return iRet;
    }

    protected void setCurrentJonasDeployment(HttpServletRequest p_Request)
        throws Exception {
        int iDeployment = 0;
        String sDeployment = p_Request.getParameter("type");
        if (sDeployment != null) {
            // Dispatch
            if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_EAR) == true) {
                iDeployment = WhereAreYou.DEPLOYMENT_EAR;
            }
            else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_JAR) == true) {
                iDeployment = WhereAreYou.DEPLOYMENT_JAR;
            }
            else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_WAR) == true) {
                iDeployment = WhereAreYou.DEPLOYMENT_WAR;
            }
            else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_RAR) == true) {
                iDeployment = WhereAreYou.DEPLOYMENT_RAR;
            }
            else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_DATASOURCE) == true) {
                iDeployment = WhereAreYou.DEPLOYMENT_DATASOURCE;
            }
            else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_MAILFACTORY) == true) {
                iDeployment = WhereAreYou.DEPLOYMENT_MAIL;
            }
            else {
                throw new Exception("Unknown type deployment");
            }
            // Storing
            m_WhereAreYou.setCurrentJonasDeploymentType(iDeployment);
        }
    }

    /**
     * Return the edit forward string.
     *
     * @return The edit forward string
     */
    protected String getForwardEdit() {
        String sForward = null;
        try {
            switch (getCurrentJonasDeployment()) {
                case WhereAreYou.DEPLOYMENT_EAR:
                case WhereAreYou.DEPLOYMENT_JAR:
                case WhereAreYou.DEPLOYMENT_WAR:
                case WhereAreYou.DEPLOYMENT_RAR:
                    sForward = "Deploy";
                    break;
                case WhereAreYou.DEPLOYMENT_DATASOURCE:
                    sForward = "Deploy Datasource";
                    break;
                case WhereAreYou.DEPLOYMENT_MAIL:
                    sForward = "Deploy Mail Factory";
                    break;
            }
        }
        catch (Exception e) {
            // none
        }
        return sForward;
    }

    /**
     * Return the list of deployable files in using the type of current deployment
     * storing in WhereAreYou instance in session.
     *
     * @return The list of deployable files
     * @throws Exception
     */
    protected ArrayList getListDeployableFiles()
        throws Exception {
        ArrayList al = null;
        switch (getCurrentJonasDeployment()) {
            case WhereAreYou.DEPLOYMENT_EAR:
                al = JonasAdminJmx.getEarFilesDeployable();
                break;
            case WhereAreYou.DEPLOYMENT_JAR:
                al = JonasAdminJmx.getJarFilesDeployable();
                break;
            case WhereAreYou.DEPLOYMENT_WAR:
                al = JonasAdminJmx.getWarFilesDeployable();
                break;
            case WhereAreYou.DEPLOYMENT_RAR:
                al = JonasAdminJmx.getRarFilesDeployable();
                break;
            case WhereAreYou.DEPLOYMENT_DATASOURCE:
                al = JonasAdminJmx.getDatasourceFilesDeployable();
                break;
            case WhereAreYou.DEPLOYMENT_MAIL:
                al = JonasAdminJmx.getMailFilesDeployable();
                break;
        }
        return al;
    }

    /**
     * Return the list of deployed files in using the type of current deployment
     * storing in WhereAreYou instance in session.
     *
     * @return The list of deployed files
     * @throws Exception
     */
    protected ArrayList getListDeployedFiles()
        throws Exception {
        ArrayList al = null;
        switch (getCurrentJonasDeployment()) {
            case WhereAreYou.DEPLOYMENT_EAR:
                al = JonasAdminJmx.getEarFilesDeployed();
                break;
            case WhereAreYou.DEPLOYMENT_JAR:
                al = JonasAdminJmx.getJarFilesDeployed();
                break;
            case WhereAreYou.DEPLOYMENT_RAR:
                al = JonasAdminJmx.getRarFilesDeployed();
                break;
            case WhereAreYou.DEPLOYMENT_WAR:
                al = JonasAdminJmx.getWarFilesDeployed();
                break;
            case WhereAreYou.DEPLOYMENT_DATASOURCE:
                al = JonasAdminJmx.getDatasourceFilesDeployed();
                break;
            case WhereAreYou.DEPLOYMENT_MAIL:
                al = JonasAdminJmx.getMailFilesDeployed();
                break;
        }
        return al;
    }

    /**
     * Undeploy a file in using the type of current deployment
     * storing in WhereAreYou instance in session.
     *
     * @param p_Filename Name of file to undeploy
     * @throws Exception
     */
    protected void undeploy(String p_Filename)
        throws Exception {
        String[] asParam = new String[1];
        asParam[0] = p_Filename;
        String[] asSignature = new String[1];
        asSignature[0] = "java.lang.String";

        switch (getCurrentJonasDeployment()) {
            case WhereAreYou.DEPLOYMENT_EAR:
                JonasManagementRepr.invoke(JonasObjectName.earService(), "unDeployEarMBean"
                    , asParam, asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_JAR:
                JonasManagementRepr.invoke(JonasObjectName.ejbService(), "removeContainer", asParam
                    , asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_RAR:
                JonasManagementRepr.invoke(JonasObjectName.resourceService(), "unDeployRarMBean"
                    , asParam, asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_WAR:
                JonasManagementRepr.invoke(JonasObjectName.webContainerService()
                    , "unRegisterWarMBean", asParam, asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_DATASOURCE:
                undeployDataSource(p_Filename);
                break;
            case WhereAreYou.DEPLOYMENT_MAIL:
                undeployMailFactory(p_Filename);
                break;
        }
    }

    /**
     * Undeploy a datasource file.
     *
     * @param p_Filename Name of file to undeploy
     * @throws Exception
     */
    protected void undeployDataSource(String p_Filename)
        throws Exception {
        ArrayList al = JonasAdminJmx.getDatasourceDependences(p_Filename);
        if (al.size() == 0) {
            String[] asParam = new String[1];
            asParam[0] = p_Filename;
            String[] asSignature = new String[1];
            asSignature[0] = "java.lang.String";
            JonasManagementRepr.invoke(JonasObjectName.databaseService(), "unloadDataSource"
                , asParam, asSignature);
        }
        else {
            throw new Exception(m_Resources.getMessage("error.undeploy.datasource.dependences"
                , al.toString()));
        }
    }

    /**
     * Undeploy a mail factory.
     *
     * @param p_Filename Name of the factory (same as the props file name configuring it)
     * @throws Exception
     */
    protected void undeployMailFactory(String p_Filename)
        throws Exception {
        ArrayList al = JonasAdminJmx.getMailFactoryDependences(p_Filename);
        if (al.size() == 0) {
            String[] asParam = new String[1];
            asParam[0] = p_Filename;
            String[] asSignature = new String[1];
            asSignature[0] = "java.lang.String";
            JonasManagementRepr.invoke(JonasObjectName.mailService(), "unbindMailFactoryMBean"
                , asParam, asSignature);
        }
        else {
            throw new Exception(m_Resources.getMessage("error.undeploy.mailfactory.dependences"
                , al.toString()));
        }
    }

    /**
     * Deploy a file in using the type of current deployment
     * storing in WhereAreYou instance in session.
     *
     * @param p_Filename Name of file to deploy
     * @throws Exception
     */
    protected void deploy(String p_Filename)
        throws Exception {
        String[] asParam = new String[1];
        asParam[0] = p_Filename;
        String[] asSignature = new String[1];
        asSignature[0] = "java.lang.String";

        switch (getCurrentJonasDeployment()) {
            case WhereAreYou.DEPLOYMENT_EAR:
                JonasManagementRepr.invoke(JonasObjectName.earService(), "deployEarMBean", asParam
                    , asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_JAR:
                JonasManagementRepr.invoke(JonasObjectName.ejbService(), "createContainer", asParam
                    , asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_RAR:
                JonasManagementRepr.invoke(JonasObjectName.resourceService(), "deployRarMBean", asParam
                    , asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_WAR:
                JonasManagementRepr.invoke(JonasObjectName.webContainerService()
                    , "registerWarMBean", asParam, asSignature);
                break;
            case WhereAreYou.DEPLOYMENT_DATASOURCE:
                deployDataSource(p_Filename);
                break;
            case WhereAreYou.DEPLOYMENT_MAIL:
                deployMailFactory(p_Filename);
                break;
        }
    }

    /**
     * Deploy a datasource file.
     *
     * @param p_Filename Name of file to deploy
     * @throws Exception
     */
    protected void deployDataSource(String p_Filename)
        throws Exception {

        String[] asParam = new String[1];
        String[] asSignature = new String[1];
        asParam[0] = p_Filename;
        asSignature[0] = "java.lang.String";

        ObjectName onDb = JonasObjectName.databaseService();
        // Verify if datasource already loaded
        Boolean oLoaded = (Boolean) JonasManagementRepr.invoke(onDb, "isLoadedDataSource", asParam
            , asSignature);
        if (oLoaded.booleanValue() == false) {
            // Get Datasource properties file
            Properties oProps = (Properties) JonasManagementRepr.invoke(onDb
                , "getDataSourcePropertiesFile", asParam, asSignature);
            // Load Datasource
            Object[] aoParam = {
                p_Filename, oProps, new Boolean(true)};
            String[] asSign_3 = {
                "java.lang.String", "java.util.Properties", "java.lang.Boolean"};
            JonasManagementRepr.invoke(onDb, "loadDataSource", aoParam, asSign_3);
        }
        else {
            throw new Exception(m_Resources.getMessage("error.deploy.datasource.isloaded"));
        }
    }

    /**
     * Deploy a mail factory.
     *
     * @param p_Filename Name of the mail factory whch is also the name of the file
     * @throws Exception
     */
    protected void deployMailFactory(String p_Filename)
        throws Exception {

        String[] asParam = new String[1];
        String[] asSignature = new String[1];
        asParam[0] = p_Filename;
        asSignature[0] = "java.lang.String";

        ObjectName onMail = JonasObjectName.mailService();
        // Get mail factory properties file
        Properties oProps = (Properties) JonasManagementRepr.invoke(onMail
                , "getMailFactoryPropertiesFile", asParam, asSignature);
        // Load mail factory
        Object[] aoParam = {p_Filename, oProps, new Boolean(true)};
        String[] asSign_3 = {"java.lang.String", "java.util.Properties", "java.lang.Boolean"};
        JonasManagementRepr.invoke(onMail, "createMailFactoryMBean", aoParam, asSign_3);
    }


    /**
     * Run garbage collector.
     *
     * @throws Exception
     */
    protected void runGC()
        throws Exception {
        ObjectName oObjectName = JonasObjectName.ejbServer();
        JonasManagementRepr.invoke(oObjectName, "runGC", null, null);
    }

    /**
     * Refresh the tree.
     *
     * @param p_Request The current HTTP Request
     * @throws Exception
     */
    protected void refreshTree(HttpServletRequest p_Request)
        throws Exception {
        // Refresh Service Tree
        switch (getCurrentJonasDeployment()) {
            case WhereAreYou.DEPLOYMENT_EAR:
                refreshServiceTree(WhereAreYou.DEPLOYMENT_EAR, p_Request);
                if (JonasAdminJmx.hasMBeanName(JonasObjectName.ejbService()) == true) {
                    refreshServiceTree(WhereAreYou.DEPLOYMENT_JAR, p_Request);
                }
                if (JonasAdminJmx.hasMBeanName(JonasObjectName.webContainerService()) == true) {
                    refreshServiceTree(WhereAreYou.DEPLOYMENT_WAR, p_Request);
                }
                if (JonasAdminJmx.hasMBeanName(JonasObjectName.resourceService()) == true) {
                    refreshServiceTree(WhereAreYou.DEPLOYMENT_RAR, p_Request);
                }
                break;
            case WhereAreYou.DEPLOYMENT_JAR:
                refreshServiceTree(WhereAreYou.DEPLOYMENT_JAR, p_Request);
                break;
            case WhereAreYou.DEPLOYMENT_WAR:
                refreshServiceTree(WhereAreYou.DEPLOYMENT_WAR, p_Request);
                break;
            case WhereAreYou.DEPLOYMENT_RAR:
                refreshServiceTree(WhereAreYou.DEPLOYMENT_RAR, p_Request);
                break;
            case WhereAreYou.DEPLOYMENT_DATASOURCE:
                refreshServiceTree(WhereAreYou.DEPLOYMENT_DATASOURCE, p_Request);
                break;
            case WhereAreYou.DEPLOYMENT_MAIL:
                refreshServiceTree(WhereAreYou.DEPLOYMENT_MAIL, p_Request);
                break;
        }
        // Refresh MBeans Tree
        refreshMBeansTree();
        // Force display to refresh
        m_WhereAreYou.setTreeToRefresh(true);
    }

    /**
     * Refresh the service tree.
     *
     * @param p_Deployment The type of deployment
     * @param p_Request The current HTTP Request
     * @throws Exception
     */
    protected void refreshServiceTree(int p_Deployment, HttpServletRequest p_Request)
        throws Exception {
        // Get the server name
        ObjectName oObjectName = JonasObjectName.ejbServer();
        String sNameServer = getStringAttribute(oObjectName, "JonasName");
        // Get the service name
        String sCurrentNodeNameServiceItem = null;
        switch (p_Deployment) {
            case WhereAreYou.DEPLOYMENT_EAR:
                sCurrentNodeNameServiceItem = sNameServer + ";services;ear";
                break;
            case WhereAreYou.DEPLOYMENT_JAR:
                sCurrentNodeNameServiceItem = sNameServer + ";services;ejbContainers";
                break;
            case WhereAreYou.DEPLOYMENT_RAR:
                sCurrentNodeNameServiceItem = sNameServer + ";services;resourceAdapter";
                break;
            case WhereAreYou.DEPLOYMENT_WAR:
                sCurrentNodeNameServiceItem = sNameServer + ";services;web";
                break;
            case WhereAreYou.DEPLOYMENT_DATASOURCE:
                sCurrentNodeNameServiceItem = sNameServer + ";services;database";
                break;
            case WhereAreYou.DEPLOYMENT_MAIL:
                sCurrentNodeNameServiceItem = sNameServer + ";services;mail";
                break;
            case WhereAreYou.DEPLOYMENT_JMS:
                sCurrentNodeNameServiceItem = sNameServer + ";services;jms";
                break;
        }
        // Get current tree
        TreeControl oControl = m_WhereAreYou.getTreeControl();
        // Get service node
        TreeControlNode oServiceNode = oControl.findNode(sCurrentNodeNameServiceItem);
        // Enable auto-refresh mode
        oControl.enableAutoRefresh();
        // Remove old children
        TreeControlNode[] aoNodes = oServiceNode.findChildren();
        for (int i = 0; i < aoNodes.length; i++) {
            aoNodes[i].remove();
        }
        // Build node for the Service
        JonasTreeBuilder oBuilder = new JonasTreeBuilder();
        switch (p_Deployment) {
            case WhereAreYou.DEPLOYMENT_EAR:
                oBuilder.getAppContainers(oServiceNode, m_Resources);
                break;
            case WhereAreYou.DEPLOYMENT_JAR:
                oBuilder.getContainers(oServiceNode, m_Resources);
                break;
            case WhereAreYou.DEPLOYMENT_RAR:
                oBuilder.getResourcesAdapters(oServiceNode, m_Resources);
                break;
            case WhereAreYou.DEPLOYMENT_WAR:
                oBuilder.getWebContainers(oServiceNode, m_Resources, p_Request);
                break;
            case WhereAreYou.DEPLOYMENT_DATASOURCE:
                oBuilder.getDatasources(oServiceNode, m_Resources);
                break;
            case WhereAreYou.DEPLOYMENT_JMS:
                oBuilder.getJmsResources(oServiceNode, m_Resources);
                break;
            case WhereAreYou.DEPLOYMENT_MAIL:
                oBuilder.getAllSessionMailFactories(oServiceNode, m_Resources);
                oBuilder.getAllMimePartDSMailFactories(oServiceNode, m_Resources);
                break;
        }
        // Disable auto-refresh mode
        oControl.disableAutoRefresh();
    }

    /**
     * Refresh the MBeans tree.
     *
     * @throws Exception
     */
    protected void refreshMBeansTree()
        throws Exception {
        // Get current tree
        TreeControl oControl = m_WhereAreYou.getTreeControl();
        // MBeans node present ?
        TreeControlNode oMBeansNode = oControl.findNode("mbeans");
        if (oMBeansNode != null) {
            // Enable auto-refresh mode
            oControl.enableAutoRefresh();
            // Remove node
            oMBeansNode.remove();
            // Build node and his children
            MBeanTreeBuilder oBuilder = new MBeanTreeBuilder();
            oBuilder.getMBeans(oControl.getRoot(), m_Resources);
            // Disable auto-refresh mode
            oControl.disableAutoRefresh();
        }
    }
}
