/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BaseSecurityAction.java,v 1.3 2003/10/16 12:09:49 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.security;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;

import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;
import org.objectweb.jonas.webapp.jonasadmin.JonasTreeBuilder;
import org.objectweb.jonas.webapp.jonasadmin.MBeanTreeBuilder;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;
import org.objectweb.jonas.webapp.jonasadmin.deploy.BaseDeployAction;

/**
 * @author Michel-Ange ANTON
 */

abstract public class BaseSecurityAction extends BaseDeployAction {

// --------------------------------------------------------- Protected Methods

    /**
     * Refresh the tree.
     *
     * @throws Exception
     */
    protected void refreshTree()
        throws Exception {
        // Refresh Security Tree
        refreshSecurityTree();
        // Refresh MBeans Tree
        refreshMBeansTree();
        // Force display to refresh
        m_WhereAreYou.setTreeToRefresh(true);
    }

    /**
     * Refresh the security branch tree.
     *
     * @throws Exception
     */
    protected void refreshSecurityTree()
        throws Exception {
        // Get the server name
        ObjectName oObjectName = JonasObjectName.ejbServer();
        String sNameServer = getStringAttribute(oObjectName, "JonasName");
        // Get the service name
        String sCurrentNodeNameItem = sNameServer + ";security";
        // Get current tree
        TreeControl oControl = m_WhereAreYou.getTreeControl();
        // Get branch root node
        TreeControlNode oBranchRootNode = oControl.findNode(sCurrentNodeNameItem);
        // Enable auto-refresh mode
        oControl.enableAutoRefresh();
        // Remove old children
        TreeControlNode[] aoNodes = oBranchRootNode.findChildren();
        for (int i = 0; i < aoNodes.length; i++) {
            aoNodes[i].remove();
        }
        // Build node for the Service
        JonasTreeBuilder oBuilder = new JonasTreeBuilder();
        oBuilder.getSecurityFactories(oBranchRootNode, m_Resources);
        // Disable auto-refresh mode
        oControl.disableAutoRefresh();
    }

    /**
     * Refresh the MBeans tree.
     *
     * @throws Exception
     */
    /*
    protected void refreshMBeansTree()
        throws Exception {
        // Get current tree
        TreeControl oControl = m_WhereAreYou.getTreeControl();
        // MBeans node present ?
        TreeControlNode oMBeansNode = oControl.findNode("mbeans");
        if (oMBeansNode != null) {
            // Enable auto-refresh mode
            oControl.enableAutoRefresh();
            // Remove node
            oMBeansNode.remove();
            // Build node and his children
            MBeanTreeBuilder oBuilder = new MBeanTreeBuilder();
            oBuilder.getMBeans(oControl.getRoot(), m_Resources);
            // Disable auto-refresh mode
            oControl.disableAutoRefresh();
        }
    }
        */
}
