/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ApplyWebAppCatalinaAction.java,v 1.4 2003/09/10 08:45:44 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.container;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.CatalinaObjectName;
import org.objectweb.jonas.webapp.jonasadmin.Jlists;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminException;

/**
 * @author Michel-Ange ANTON
 */
public class ApplyWebAppCatalinaAction extends BaseWebAppAction {

    /**
     * Signature for the <code>createStandardConnector</code> operation.
     */
    private String sa_CreateJOnASStandardContext[] = {
        "java.lang.String", "java.lang.String", "java.lang.String"
    };

    private static String sDefaultForward = "ActionEditWebAppCatalina";

// --------------------------------------------------------- Protected Variables

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Default forward
        ActionForward oForward = null;

        // Identify the requested action
        WebAppCatalinaForm oForm = (WebAppCatalinaForm) p_Form;

        // Populate
        try {
            // Perform a "Create WebApp" transaction (if requested)
            if ("create".equals(oForm.getAction()) == true) {
                oForward = createWebApplication(oForm, p_Mapping, p_Request);
            }
            // Save in memory the new datas
            else if ("edit".equals(oForm.getAction()) == true) {
                oForward = populateMbean(oForm, p_Mapping.findForward(sDefaultForward), p_Mapping
                    , p_Request);
            }
            // Reload the web app
            else if ("reload".equals(oForm.getAction()) == true) {
                oForward = reloadWebApplication(oForm, p_Mapping);
            }
            // Stop the web app
            else if ("stop".equals(oForm.getAction()) == true) {
                oForward = stopWebApplication(oForm, p_Mapping);
            }
            // Start the web app
            else if ("start".equals(oForm.getAction()) == true) {
                oForward = startWebApplication(oForm, p_Mapping);
            }
        }
        catch (JonasAdminException e) {
            // Build error
            m_Errors.add("webapp", new ActionError(e.getId()));
            saveErrors(p_Request, m_Errors);
            // Return to the current page
            oForward = new ActionForward(p_Mapping.getInput());
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            oForward = p_Mapping.findForward("Global Error");
        }

        // Next Forward
        return oForward;
    }

    /**
     * Create a new web application (Context, Manager, Loader).
     * @param p_Form The current form
     * @param p_Mapping The current mapping
     * @param p_Request The current request
     * @return The forward to go to the next page
     * @throws Exception
     */
    protected ActionForward createWebApplication(WebAppCatalinaForm p_Form, ActionMapping p_Mapping
        , HttpServletRequest p_Request)
        throws Exception {
        Object values[] = null;

        // Ensure that the requested context is unique
        ObjectName onContext = CatalinaObjectName.catalinaContext(m_WhereAreYou.
            getCurrentCatalinaServiceName(), m_WhereAreYou.getCurrentCatalinaDefaultHostName()
            , p_Form.getPathContext());
        if (JonasAdminJmx.hasMBeanName(onContext) == true) {
            throw new JonasAdminException("error.webapp.context.exists");
        }

        // Look up the JOnAS Web service
        ObjectName onWebService = JonasObjectName.webContainerService();
        // Create a new JOnASStandardContext object
        values = new Object[3];
        // parent
        values[0] = CatalinaObjectName.catalinaHost(m_WhereAreYou.getCurrentCatalinaServiceName()
            , m_WhereAreYou.getCurrentCatalinaDefaultHostName()).toString();
        values[1] = p_Form.getPathContext();
        values[2] = "";
        p_Form.setObjectName((String) JonasManagementRepr.invoke(onWebService
            , "createJOnASStandardContext", values, sa_CreateJOnASStandardContext));

        // Populate
        ActionForward oForward = populateMbean(p_Form
            , p_Mapping.findForward("ActionListWebContainers"), p_Mapping, p_Request);

        // Stop the WebApp (Tomcat start it at creating)
        try {
            stopWebApplication(p_Form, p_Mapping);
        }
        catch (Exception e) {
            // none
        }

        // refresh tree
        refreshTree(p_Request);
        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(1) + ";services;web;"
            + p_Form.getPathContext(), true);

        return oForward;
    }

    /**
     * Populate the Mbean web application (Context, Manager, Loader).
     * @param p_Form The current form
     * @param p_Forward The current forward
     * @param p_Mapping The current mapping
     * @param p_Request The current request
     * @return If 'save' is requested then return the new forward to save
     * else return the current forward
     * @throws Exception
     */
    protected ActionForward populateMbean(WebAppCatalinaForm p_Form, ActionForward p_Forward
        , ActionMapping p_Mapping, HttpServletRequest p_Request)
        throws Exception {
        ActionForward oForward = p_Forward;
        // Context
        ObjectName on = new ObjectName(p_Form.getObjectName());
        setBooleanAttribute(on, "available", p_Form.isAvailable());
        setBooleanAttribute(on, "cookies", p_Form.isCookies());
        setBooleanAttribute(on, "crossContext", p_Form.isCrossContext());
        setBooleanAttribute(on, "reloadable", p_Form.isReloadable());
        setBooleanAttribute(on, "swallowOutput", p_Form.isSwallowOutput());
        setBooleanAttribute(on, "override", p_Form.isOverride());
        try {
            setIntegerAttribute(on, "sessionTimeout", p_Form.getSessionTimeout());
            p_Form.setExistSessionTimeout(true);
        }
        catch (Exception e) {
            p_Form.setExistSessionTimeout(false);
        }
        // Save in configuration file
        if (p_Form.isSave() == true) {
            //ObjectName onServer = CatalinaObjectName.catalinaServer();
            //JonasManagementRepr.invoke(onServer, "store", null, null);
            p_Form.setSave(false);
            p_Request.setAttribute("forward", p_Forward.getName());
            oForward = p_Mapping.findForward("ActionEditServletServer");
        }
        return oForward;
    }

    /**
     * Reload the current web application.
     * @param p_Form The current form
     * @param p_Mapping The current mapping
     * @return The forward to go to the next page
     * @throws Exception
     */
    protected ActionForward reloadWebApplication(WebAppCatalinaForm p_Form, ActionMapping p_Mapping)
        throws Exception {
        // Reload
        try {
            ObjectName onContext = new ObjectName(p_Form.getObjectName());
            JonasManagementRepr.invoke(onContext, "reload", null, null);
        }
        catch (Throwable t) {
            throw new JonasAdminException("error.webapp.reload");
        }
        // Forward
        ActionForward oForward = null;
        if (p_Form.getPathContext().equals(m_WhereAreYou.getApplicationContextPath()) == true) {
            oForward = p_Mapping.findForward("Main Index");
        }
        else {
            oForward = p_Mapping.findForward(sDefaultForward);
        }
        return oForward;
    }

    /**
     * Start the current web application.
     * @param p_Form The current form
     * @param p_Mapping The current mapping
     * @return The forward to go to the next page
     * @throws Exception
     */
    protected ActionForward startWebApplication(WebAppCatalinaForm p_Form, ActionMapping p_Mapping)
        throws Exception {
        try {
            ObjectName onContext = new ObjectName(p_Form.getObjectName());
            JonasManagementRepr.invoke(onContext, "start", null, null);
        }
        catch (Throwable t) {
            throw new JonasAdminException("error.webapp.start");
        }
        return p_Mapping.findForward(sDefaultForward);
    }

    /**
     * Stop the current web application.
     * @param p_Form The current form
     * @param p_Mapping The current mapping
     * @return The forward to go to the next page
     * @throws Exception
     */
    protected ActionForward stopWebApplication(WebAppCatalinaForm p_Form, ActionMapping p_Mapping)
        throws Exception {
        try {
            ObjectName onContext = new ObjectName(p_Form.getObjectName());
            JonasManagementRepr.invoke(onContext, "stop", null, null);
        }
        catch (Throwable t) {
            throw new JonasAdminException("error.webapp.stop");
        }
        return p_Mapping.findForward(sDefaultForward);
    }
}
