/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BaseWebAppAction.java,v 1.4 2003/09/23 14:53:03 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.container;

import java.net.URL;
import java.util.Iterator;

import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.CatalinaObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JettyObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.webapp.jonasadmin.deploy.BaseDeployAction;

/**
 * @author Michel-Ange ANTON
 */

abstract public class BaseWebAppAction extends BaseDeployAction {

// --------------------------------------------------------- Protected variables

    protected WebAppForm m_WebAppForm;
    protected WarForm m_WarForm;

// --------------------------------------------------------- Public Methods

// --------------------------------------------------------- Protected Methods

    /**
     * Initialize the forms and parameters.
     * @param p_Mapping Mapping to reset form
     * @param p_Request Http request used
     * @return true if the forms must be populated
     */
    protected boolean initialize(ActionMapping p_Mapping, HttpServletRequest p_Request) {

        boolean bPopulate = false;

        // Global form used
        m_WebAppForm = null;
        m_WarForm = null;

        // Action
        String sParamAction = p_Request.getParameter("action");
        // Selected WebApp
        String sParamPathContext = p_Request.getParameter("context");
        // Selected War
        String sParamPathWar = p_Request.getParameter("war");

        // Detect create action
        if ((sParamAction != null) && ("create".equals(sParamAction) == true)) {
            // Create a new WebApp
            m_WebAppForm = createWebAppForm(p_Mapping, p_Request);
            m_WebAppForm.setAction("create");
            // Remove WarForm
            m_Session.removeAttribute("warForm");
        }
        // Build forms with the context
        else if (sParamPathContext != null) {
            m_WebAppForm = createWebAppForm(p_Mapping, p_Request);
            m_WebAppForm.setPathContext(sParamPathContext);
            bPopulate = true;
            // Remove WarForm
            m_Session.removeAttribute("warForm");
        }
        else if (sParamPathWar != null) {
            // Create a new War form
            m_WarForm = createWarForm(p_Mapping, p_Request);
            m_WarForm.setPath(sParamPathWar);
            m_WarForm.setFilename(JonasAdminJmx.extractFilename(sParamPathWar));
            bPopulate = true;
        }
        else {
            // Edit last form in session
            m_WebAppForm = (WebAppForm) m_Session.getAttribute("webAppForm");
            if (m_WebAppForm != null) {
                m_WebAppForm.setAction("edit");
            }
            m_WarForm = (WarForm) m_Session.getAttribute("warForm");
            // Force populate
            if (("reload".equals(sParamAction) == true) || ("start".equals(sParamAction) == true)
                || ("stop".equals(sParamAction) == true)) {
                bPopulate = true;
            }
        }
        return bPopulate;
    }

    /**
     * Create a new WebApp form in function of Web Server.
     * @param p_Mapping Mapping to reset form
     * @param p_Request Http request used
     * @return The new form or null if web server unknown
     */
    protected WebAppForm createWebAppForm(ActionMapping p_Mapping, HttpServletRequest p_Request) {
        WebAppForm oForm = null;
        if (m_WhereAreYou.isCatalinaServer() == true) {
            oForm = new WebAppCatalinaForm();
            oForm.reset(p_Mapping, p_Request);
            ((WebAppCatalinaForm)oForm).setJonasStandardContext(true);
            ((WebAppCatalinaForm)oForm).setExistSessionTimeout(true);
            m_Session.setAttribute("webAppForm", oForm);
            m_Session.setAttribute("webAppCatalinaForm", oForm);
        }
        else if (m_WhereAreYou.isJettyServer() == true) {
            oForm = new WebAppJettyForm();
            oForm.reset(p_Mapping, p_Request);
            m_Session.setAttribute("webAppForm", oForm);
            m_Session.setAttribute("webAppJettyForm", oForm);
        }
        return oForm;
    }

    /**
     * Create a new War form
     * @param p_Mapping Mapping to reset form
     * @param p_Request Http request used
     * @return The new form or null if web server unknown
     */
    protected WarForm createWarForm(ActionMapping p_Mapping, HttpServletRequest p_Request) {
        WarForm oForm = new WarForm();
        oForm.reset(p_Mapping, p_Request);
        m_Session.setAttribute("warForm", oForm);
        return oForm;
    }

    /**
     * Populate a WebApp form in function of Web Server.
     * @param p_PathContext The path context
     * @param p_Form The form to populate
     * @throws Exception
     */
    protected void populateWebApp(String p_PathContext, WebAppForm p_Form)
        throws Exception {
        if (m_WhereAreYou.isCatalinaServer() == true) {
            populateWebAppCatalina(p_PathContext, (WebAppCatalinaForm) p_Form);
        }
        else if (m_WhereAreYou.isJettyServer() == true) {
            populateWebAppJetty(p_PathContext, (WebAppJettyForm) p_Form);
        }
    }

    /**
     * Populate a form with the Catalina Context MBean
     * and with the JOnAS War MBean if it's found.
     * @param p_PathContext The path context
     * @param p_Form The form to populate
     * @throws Exception
     */
    protected void populateWebAppCatalina(String p_PathContext, WebAppCatalinaForm p_Form)
        throws Exception {
        // Context
        ObjectName on = CatalinaObjectName.catalinaContext(m_WhereAreYou.
            getCurrentCatalinaServiceName(), m_WhereAreYou.getCurrentCatalinaDefaultHostName()
            , p_PathContext);
        p_Form.setPathContext(p_PathContext);
        p_Form.setObjectName(on.toString());
        p_Form.setHost(on.getKeyProperty("host"));
        p_Form.setAvailable(getBooleanAttribute(on, "available"));
        p_Form.setCookies(getBooleanAttribute(on, "cookies"));
        p_Form.setReloadable(getBooleanAttribute(on, "reloadable"));
        p_Form.setSwallowOutput(getBooleanAttribute(on, "swallowOutput"));
        p_Form.setCrossContext(getBooleanAttribute(on, "crossContext"));
        p_Form.setOverride(getBooleanAttribute(on, "override"));
        try {
            p_Form.setSessionTimeout(toStringIntegerAttribute(on, "sessionTimeout"));
            p_Form.setExistSessionTimeout(true);
            if (p_Form.getSessionTimeout() == null) {
                p_Form.setSessionTimeout("30");
            }
        }
        catch (Exception e) {
            p_Form.setExistSessionTimeout(false);
        }
        // ClassName
        p_Form.setJonasStandardContext(false);
        try {
            p_Form.setClassName(getStringAttribute(on, "className"));
            if (p_Form.getClassName().indexOf("JOnASStandardContext") > -1) {
                p_Form.setJonasStandardContext(true);
            }
        }
        catch (Exception e) {
            // nothing
        }
    }

    /**
     * Populate a form with the Jetty Context MBean
     * and with the JOnAS War MBean if it's found.
     * @param p_PathContext The path context
     * @param p_Form The form to populate
     * @throws Exception
     */
    protected void populateWebAppJetty(String p_PathContext, WebAppJettyForm p_Form)
        throws Exception {
        ObjectName on = null;
        // Find context
        Iterator it = JonasAdminJmx.getListMbean(JettyObjectName.jettyContexts(p_PathContext)).iterator();
        while (it.hasNext() == true) {
            on = (ObjectName) it.next();
            if (on.getKeyProperty("WebApplicationHandler") == null) {
                // Context
                p_Form.setPathContext(p_PathContext);
                p_Form.setObjectName(on.toString());
                //p_Form.setHost(getStringAttribute(on, "host"));
                p_Form.setStarted(getBooleanAttribute(on, "started"));
                p_Form.setResourceBase(getStringAttribute(on, "resourceBase"));
                p_Form.setDisplayName(getStringAttribute(on, "displayName"));
                break;
            }
        }
    }

    /**
     * Return the JOnAS MBean War.
     * @param p_PathContext The path context to find
     * @return The JOnAS MBean War or null if not found
     * @throws Exception
     */
    protected ObjectName findJonasMbeanWar(String p_PathContext)
        throws Exception {
        ObjectName onRet = null;
        ObjectName on;
        String sPathContext;
        StringBuffer sb;

        Iterator itOnWars = JonasAdminJmx.getListMbean(JonasObjectName.allWars()).iterator();
        while (itOnWars.hasNext()) {
            on = (ObjectName) itOnWars.next();
            sPathContext = getStringAttribute(on, "ContextRoot");
            if (sPathContext.charAt(0) != '/') {
                sb = new StringBuffer("/");
                sb.append(sPathContext);
                sPathContext = sb.toString();
            }
            // Detect the good War
            if (p_PathContext.equals(sPathContext) == true) {
                onRet = on;
                break;
            }
        }
        return onRet;
    }

    /**
     * Populate a form with the JOnAS War MBean given.
     * @param p_Form The form to populate
     * @param p_ObjectName The JOnAS War MBean
     * @throws Exception
     */
    protected void populateWar(ObjectName p_ObjectName, WarForm p_Form)
        throws Exception {
        if (p_ObjectName != null) {
            p_Form.setPath(p_ObjectName.getKeyProperty("fname"));
            p_Form.setFilename(JonasAdminJmx.extractFilename(p_Form.getPath()));

            p_Form.setHostName(getStringAttribute(p_ObjectName, "HostName"));
            p_Form.setInEarCase(getBooleanAttribute(p_ObjectName, "InEarCase"));
            p_Form.setContextRoot(getStringAttribute(p_ObjectName, "ContextRoot"));
            p_Form.setJava2DelegationModel(getBooleanAttribute(p_ObjectName, "Java2DelegationModel"));
            p_Form.setXmlContent(getStringAttribute(p_ObjectName, "XmlContent"));
            p_Form.setJonasXmlContent(getStringAttribute(p_ObjectName, "JOnASXmlContent"));
            p_Form.setServletsName((String[]) JonasManagementRepr.getAttribute(p_ObjectName
                , "ServletsName"));
            p_Form.setWarPath((URL) JonasManagementRepr.getAttribute(p_ObjectName, "WarURL"));
            p_Form.setEarPath((URL) JonasManagementRepr.getAttribute(p_ObjectName, "EarURL"));
        }
    }

    /**
     * Populate a form with the JOnAS War path given.
     * @param p_Path The JOnAS War MBean path
     * @param p_Form The form to populate
     * @throws Exception
     */
    protected void populateWar(String p_Path, WarForm p_Form)
        throws Exception {
        populateWar(JonasObjectName.war(p_Path), p_Form);
    }
}
