/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EarForm.java,v 1.5 2003/09/15 16:02:14 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.container;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;

/**
 * @author Michel-Ange ANTON
 */
public class EarForm extends ActionForm {

// --------------------------------------------------------- Properties variables

    private String path = null;
    private String filename = null;
    private String earPath = null;
    private String xmlDeploymentDescriptor = null;
    private ArrayList listWars = new ArrayList();
    private ArrayList listJars = new ArrayList();
    private ArrayList listRars = new ArrayList();

// --------------------------------------------------------- Public Methods

    /**
     * Reset all properties to their default values.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */

    public void reset(ActionMapping mapping, HttpServletRequest request) {
        earPath = null;
        xmlDeploymentDescriptor = null;
        listWars = new ArrayList();
        listJars = new ArrayList();
        listRars = new ArrayList();
    }

    /**
     * Validate the properties that have been set from this HTTP request,
     * and return an <code>ActionErrors</code> object that encapsulates any
     * validation errors that have been found.  If no errors are found, return
     * <code>null</code> or an <code>ActionErrors</code> object with no
     * recorded error messages.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
        ActionErrors oErrors = new ActionErrors();
        return oErrors;
    }

// --------------------------------------------------------- Properties Methods

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public String getFilename() {
        return filename;
    }

    public void setFilename(String filename) {
        this.filename = filename;
    }

    public String getEarPath() {
        return earPath;
    }

    public void setEarPath(URL p_Url) {
        this.earPath = null;
        if (p_Url != null) {
            this.earPath = p_Url.getPath();
        }
    }

    public ArrayList getListJars() {
        return listJars;
    }

    /**
     * Set list of Ejb Jar with Jar given in a array of <code>URL</code>.
     * Each item of list is a <code>ContainerItem</code>.
     *
     * @param pa_Url Array of Url Jar
     */
    public void setListJars(URL[] pa_Url) {
        listJars.clear();
        if (pa_Url != null) {
            for (int i = 0; i < pa_Url.length; i++) {
                listJars.add(new ContainerItem(JonasAdminJmx.extractFilename(pa_Url[i].getPath())
                    , pa_Url[i].getPath()));
            }
            // Sort list
            Collections.sort(listJars, new ContainerItemByFile());
        }
    }

    public ArrayList getListRars() {
        return listRars;
    }

    /**
     * Set list of Ejb Rar with Rar given in a array of <code>URL</code>.
     * Each item of list is a <code>ContainerItem</code>.
     *
     * @param pa_Url Array of Rar
     */
    public void setListRars(URL[] pa_Url) {
        listRars.clear();
        if (pa_Url != null) {
            for (int i = 0; i < pa_Url.length; i++) {
                listRars.add(new ContainerItem(JonasAdminJmx.extractFilename(pa_Url[i].getPath())
                    , pa_Url[i].getPath()));
            }
            // Sort list
            Collections.sort(listRars, new ContainerItemByFile());
        }
    }

    public ArrayList getListWars() {
        return listWars;
    }

    /**
     * Set list of Ejb War with War given in a array of <code>URL</code>.
     * Each item of list is a <code>ContainerItem</code>.
     *
     * @param pa_Url Array of War
     */
    public void setListWars(URL[] pa_Url) {
        listWars.clear();
        if (pa_Url != null) {
            for (int i = 0; i < pa_Url.length; i++) {
                listWars.add(new ContainerItem(JonasAdminJmx.extractFilename(pa_Url[i].getPath())
                    , pa_Url[i].getPath()));
            }
            // Sort list
            Collections.sort(listWars, new ContainerItemByFile());
        }
    }

    public String getXmlDeploymentDescriptor() {
        return xmlDeploymentDescriptor;
    }

    public void setXmlDeploymentDescriptor(String xmlDeploymentDescriptor) {
        this.xmlDeploymentDescriptor = xmlDeploymentDescriptor;
    }

}