/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ListWebContainersAction.java,v 1.10 2003/09/23 14:53:03 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.container;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.HashMap;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.management.ObjectName;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.CatalinaObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JettyObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;

/**
 * @author Michel-Ange ANTON
 */

public class ListWebContainersAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(1) + ";services;web", true);
        // Force the deployment type to use the refreshing method
        m_WhereAreYou.setCurrentJonasDeploymentType(m_WhereAreYou.DEPLOYMENT_WAR);
        // Delete last list if used
        m_Session.removeAttribute("itemsWebContainersForm");

        // no Form used
        try {
            ArrayList al = new ArrayList();
            ObjectName on;
            String sPathContext;
            HashMap hWebAppItems = new HashMap();

            // Get the list of Catalina contexts
            if (m_WhereAreYou.isCatalinaServer() == true) {
                Iterator itOnContexts = JonasAdminJmx.getListMbean(CatalinaObjectName.
                    catalinaContexts(m_WhereAreYou.getCurrentCatalinaServiceName()
                    , m_WhereAreYou.getCurrentCatalinaDefaultHostName())).iterator();
                while (itOnContexts.hasNext()) {
                    on = (ObjectName) itOnContexts.next();
                    sPathContext = on.getKeyProperty("path");
                    hWebAppItems.put(sPathContext, new WebAppItem(sPathContext, on.toString()));
                }
            }
            // Get the list of Jetty contexts
            else if (m_WhereAreYou.isJettyServer() == true) {
                Iterator itOnContexts = JonasAdminJmx.getListMbean(JettyObjectName.
                    jettyContexts()).iterator();
                while (itOnContexts.hasNext()) {
                    on = (ObjectName) itOnContexts.next();
                    sPathContext = on.getKeyProperty("context");
                    if (sPathContext != null) {
                        if (hWebAppItems.containsKey(sPathContext) == false) {
                            hWebAppItems.put(sPathContext, new WebAppItem(sPathContext, on.toString()));
                        }
                    }
                }
            }

            // Get the list of JOnAS War
            String sPath;
            String sFile;
            WebAppItem oWeb;
            Iterator itOnWars = JonasAdminJmx.getListMbean(JonasObjectName.allWars()).iterator();
            while (itOnWars.hasNext()) {
                on = (ObjectName) itOnWars.next();
                sPath = on.getKeyProperty("fname");
                sFile = JonasAdminJmx.extractFilename(sPath);
                sPathContext = getStringAttribute(on, "ContextRoot");
                if (sPathContext.charAt(0) != '/') {
                    sPathContext = "/" + sPathContext;
                }
                oWeb = (WebAppItem) hWebAppItems.get(sPathContext);
                if (oWeb != null) {
                    oWeb.setFile(sFile);
                    oWeb.setPath(sPath);
                }
            }
            // Prepare to display and sort
            al.addAll(hWebAppItems.values());
            Collections.sort(al, new WebAppItemByPathContext());

            // Set list in the request
            p_Request.setAttribute("listWebContainers", al);
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp.
        return (p_Mapping.findForward("Web Containers"));
    }
}
