/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ApplyDatasourceAction.java,v 1.4 2003/09/11 15:45:39 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.db;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;

/**
 * @author Michel-Ange ANTON
 */

public class ApplyDatasourceAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Form used
        DatasourceForm oForm = (DatasourceForm) p_Form;
        try {
            populate(oForm);
            if (oForm.getAction().equals("save") == true) {
                save(oForm);
            }
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp and add the parameter 'name' with the good value.
        return new ActionForward(p_Mapping.findForward("ActionEditDatasource").getPath() + "?name="
            + oForm.getDatasourceName());
    }

// --------------------------------------------------------- Protected Methods

    protected void populate(DatasourceForm p_Form)
        throws Exception {
        // Object name used
        ObjectName oObjectName = JonasObjectName.datasource(p_Form.getDatasourceName());
        // Populate
        setIntegerAttribute(oObjectName, "JDBCConnCheckLevel", p_Form.getJdbcConnCheckLevel());
        setIntegerAttribute(oObjectName, "JDBCConnMaxAge", p_Form.getJdbcConnMaxAge());
        setStringAttribute(oObjectName, "JDBCTestStatement", p_Form.getJdbcTestStatement());
        setIntegerAttribute(oObjectName, "JDBCMaxConnPool", p_Form.getJdbcMaxConnPool());
        setIntegerAttribute(oObjectName, "JDBCMinConnPool", p_Form.getJdbcMinConnPool());
    }

    protected void save(DatasourceForm p_Form)
        throws Exception {
        // Save
        JonasManagementRepr.invoke(JonasObjectName.datasource(p_Form.getDatasourceName())
            , "saveConfig", null, null);
    }

}
