/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DatasourceForm.java,v 1.4 2003/10/03 09:13:59 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.db;

import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.webapp.jonasadmin.Jlists;

/**
 * @author Michel-Ange ANTON
 */
public class DatasourceForm extends ActionForm {

// --------------------------------------------------------- Constants

// --------------------------------------------------------- Properties variables

    private String action = null;
    private String className = null;
    private String datasourceDescription = null;
    private String datasourceFactory = null;
    private String datasourceName = null;
    private String datasourceType = null;
//   private String datasourceXADataName = null;
    private String dsName = null;
    private String jdbcTestStatement = null;
    private String password = null;
    private String url = null;
    private String userName = null;
    private String datasourceMapper = null;
//   private String jdbcSleepTimeOut = null;
//   private String jdbcLoopWaitPool = null;
//   private String jdbcDeadLockPool = null;
    private java.util.List checkingLevels = Jlists.getJdbcConnectionCheckingLevels();
    private String currentNumberOfJDBCConnectionOpen = null;
    private String jdbcConnCheckLevel = null;
    private String jdbcConnMaxAge = null;
    private String jdbcMaxConnPool = null;
    private String jdbcMinConnPool = null;

    private ArrayList listUsedByEjb = new ArrayList();

// --------------------------------------------------------- Public Methods

    /**
     * Reset all properties to their default values.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */

    public void reset(ActionMapping mapping, HttpServletRequest request) {
        action = "apply";
    }

    /**
     * Validate the properties that have been set from this HTTP request,
     * and return an <code>ActionErrors</code> object that encapsulates any
     * validation errors that have been found.  If no errors are found, return
     * <code>null</code> or an <code>ActionErrors</code> object with no
     * recorded error messages.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     * @return List of errors
     */
    public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
        ActionErrors oErrors = new ActionErrors();

        validateInteger(oErrors, jdbcConnMaxAge, "jdbcConnMaxAge"
            , "error.database.datasource.jdbcConnMaxAge.numberformat");
        validateInteger(oErrors, jdbcMaxConnPool, "jdbcMaxConnPool"
            , "error.database.datasource.jdbcMaxConnPool.numberformat");
        validateInteger(oErrors, jdbcMinConnPool, "jdbcMinConnPool"
            , "error.database.datasource.jdbcMinConnPool.numberformat");

        return oErrors;
    }

    protected void validateInteger(ActionErrors p_Errors, String p_Value, String p_Tag
        , String p_ResError) {
        try {
            Integer.parseInt(p_Value);
        }
        catch (NumberFormatException e) {
            p_Errors.add(p_Tag, new ActionError(p_ResError));
        }
    }

// --------------------------------------------------------- Properties Methods

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public String getCurrentNumberOfJDBCConnectionOpen() {
        return currentNumberOfJDBCConnectionOpen;
    }

    public void setCurrentNumberOfJDBCConnectionOpen(String currentNumberOfJDBCConnectionOpen) {
        this.currentNumberOfJDBCConnectionOpen = currentNumberOfJDBCConnectionOpen;
    }

    public String getDatasourceDescription() {
        return datasourceDescription;
    }

    public void setDatasourceDescription(String datasourceDescription) {
        this.datasourceDescription = datasourceDescription;
    }


    public String getDatasourceName() {
        return datasourceName;
    }

    public void setDatasourceName(String datasourceName) {
        this.datasourceName = datasourceName;
    }

    public String getDatasourceType() {
        return datasourceType;
    }

    public void setDatasourceType(String datasourceType) {
        this.datasourceType = datasourceType;
    }

    public String getDatasourceMapper() {
        return datasourceMapper;
    }

    public void setDatasourceMapper(String datasourceMapper) {
        this.datasourceMapper = datasourceMapper;
    }

    /*
       public String getDatasourceXADataName() {
          return datasourceXADataName;
       }
       public void setDatasourceXADataName(String datasourceXADataName) {
          this.datasourceXADataName = datasourceXADataName;
       }
     */
    public String getDsName() {
        return dsName;
    }

    public void setDsName(String dsName) {
        this.dsName = dsName;
    }

    public String getJdbcConnCheckLevel() {
        return jdbcConnCheckLevel;
    }

    public void setJdbcConnCheckLevel(String jdbcConnCheckLevel) {
        this.jdbcConnCheckLevel = jdbcConnCheckLevel;
    }

    public String getJdbcConnMaxAge() {
        return jdbcConnMaxAge;
    }

    public void setJdbcConnMaxAge(String jdbcConnMaxAge) {
        this.jdbcConnMaxAge = jdbcConnMaxAge;
    }

    /*
       public String getJdbcDeadLockPool() {
          return jdbcDeadLockPool;
       }
       public void setJdbcDeadLockPool(String jdbcDeadLockPool) {
          this.jdbcDeadLockPool = jdbcDeadLockPool;
       }
     */
    /*
       public String getJdbcLoopWaitPool() {
          return jdbcLoopWaitPool;
       }
       public void setJdbcLoopWaitPool(String jdbcLoopWaitPool) {
          this.jdbcLoopWaitPool = jdbcLoopWaitPool;
       }
     */
    public String getJdbcMaxConnPool() {
        return jdbcMaxConnPool;
    }

    public void setJdbcMaxConnPool(String jdbcMaxConnPool) {
        this.jdbcMaxConnPool = jdbcMaxConnPool;
    }

    public String getJdbcMinConnPool() {
        return jdbcMinConnPool;
    }

    public void setJdbcMinConnPool(String jdbcMinConnPool) {
        this.jdbcMinConnPool = jdbcMinConnPool;
    }

    /*
       public String getJdbcSleepTimeOut() {
          return jdbcSleepTimeOut;
       }
       public void setJdbcSleepTimeOut(String jdbcSleepTimeOut) {
          this.jdbcSleepTimeOut = jdbcSleepTimeOut;
       }
     */
    public String getJdbcTestStatement() {
        return jdbcTestStatement;
    }

    public void setJdbcTestStatement(String jdbcTestStatement) {
        this.jdbcTestStatement = jdbcTestStatement;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getAction() {
        return action;
    }

    public void setAction(String action) {
        this.action = action;
    }

    public java.util.List getCheckingLevels() {
        return checkingLevels;
    }

    public ArrayList getListUsedByEjb() {
        return listUsedByEjb;
    }

    public void setListUsedByEjb(ArrayList listUsedByEjb) {
        this.listUsedByEjb = listUsedByEjb;
    }

}
