/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EditDatasourceAction.java,v 1.5 2003/10/03 09:13:59 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.db;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItem;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItemByNameComparator;

/**
 * @author Michel-Ange ANTON
 */

public class EditDatasourceAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Datasource to edit
        String sName = p_Request.getParameter("name");

        // Form used
        DatasourceForm oForm = null;
        if (sName != null) {
            // Editing a new
            oForm = new DatasourceForm();
            oForm.reset(p_Mapping, p_Request);
            m_Session.setAttribute("datasourceForm", oForm);
            oForm.setDatasourceName(sName);
        }
        else {
            // Editing that which is in session
            oForm = (DatasourceForm) m_Session.getAttribute("datasourceForm");
        }

        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(1) + ";services;database;"
            + oForm.getDatasourceName(), true);

        // Populate
        try {
            if (sName != null) {
                populate(oForm);
            }
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp.
        return (p_Mapping.findForward("Datasource"));
    }

// --------------------------------------------------------- Protected Methods

    protected void populate(DatasourceForm p_Form)
        throws Exception {

        // Object name used
        ObjectName oObjectName = JonasObjectName.datasource(p_Form.getDatasourceName());
        // Populate
        p_Form.setClassName(getStringAttribute(oObjectName, "ClassName"));
        p_Form.setCurrentNumberOfJDBCConnectionOpen(toStringIntegerAttribute(oObjectName
            , "CurrentNumberOfJDBCConnectionOpen"));
        p_Form.setDatasourceDescription(getStringAttribute(oObjectName, "DataSourceDescription"));
        p_Form.setDatasourceName(getStringAttribute(oObjectName, "DatasourceName"));
        p_Form.setDatasourceType(getStringAttribute(oObjectName, "DataSourceType"));
        p_Form.setDatasourceMapper(getStringAttribute(oObjectName, "MapperName"));
//      p_Form.setDatasourceXADataName(getStringAttribute(oObjectName,"DataSourceXADataName"));
        p_Form.setDsName(getStringAttribute(oObjectName, "DSName"));
        p_Form.setJdbcConnCheckLevel(toStringIntegerAttribute(oObjectName, "JDBCConnCheckLevel"));
        p_Form.setJdbcConnMaxAge(toStringIntegerAttribute(oObjectName, "JDBCConnMaxAge"));
        p_Form.setJdbcTestStatement(getStringAttribute(oObjectName, "JDBCTestStatement"));
        p_Form.setPassword(getStringAttribute(oObjectName, "Password"));
        p_Form.setUrl(getStringAttribute(oObjectName, "Url"));
        p_Form.setUserName(getStringAttribute(oObjectName, "UserName"));
        //p_Form.setJdbcDeadLockPool(getIntegerAttribute(oObjectName,"JDBCDeadLockPool"));
        //p_Form.setJdbcLoopWaitPool(getIntegerAttribute(oObjectName,"JDBCLoopWaitPool"));
        p_Form.setJdbcMaxConnPool(toStringIntegerAttribute(oObjectName, "JDBCMaxConnPool"));
        p_Form.setJdbcMinConnPool(toStringIntegerAttribute(oObjectName, "JDBCMinConnPool"));
        //p_Form.setJdbcSleepTimeOut(getIntegerAttribute(oObjectName,"JDBCSleepTimeOut"));

        // Build list of Ejb which used this datasource
        ArrayList al = new ArrayList();
        String[] asParam = new String[1];
        String[] asSignature = new String[1];
        asSignature[0] = "java.lang.String";
        asParam[0] = p_Form.getDsName();
        if (JonasManagementRepr.isRegistered(JonasObjectName.ejbService()) == true) {
            java.util.Iterator it = ((java.util.Set) JonasManagementRepr.invoke(JonasObjectName.
                ejbService(), "getDataSourceDependence", asParam, asSignature)).iterator();
            while (it.hasNext()) {
                al.add(new EjbItem((ObjectName) it.next()));
            }
        }
        // Sort by name
        Collections.sort(al, new EjbItemByNameComparator());
        // Set list in form
        p_Form.setListUsedByEjb(al);
    }
}
