/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ExecuteEjbAction.java,v 1.1 2003/09/16 12:42:03 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.ejb;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;
import org.objectweb.jonas.webapp.jonasadmin.service.container.ContainerForm;

/**
 * @author Michel-Ange ANTON
 */

public class ExecuteEjbAction extends JonasBaseAction {

// --------------------------------------------------------- Protected variables

    protected int m_Type = 0;
    protected String m_Action = null;

// --------------------------------------------------------- Public Methods

    /**
     * Execute a action.
     *
     * @param p_Mapping <code>ActionForward</code> instance
     * @param p_Form <code>ActionForm</code> instance
     * @param p_Request <code>HttpServletRequest</code> instance
     * @param p_Response <code>HttpServletResponse</code> instance
     * @return <code>ActionForward</code> instance
     * @throws IOException
     * @throws ServletException
     */
    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Get Http parameters
        m_Action = p_Request.getParameter("action");
        // Form used
        EjbForm oForm = getEjbForm();
        // Type Ejb used
        m_Type = getIntegerType(oForm);

        try {
            // Execute a method of Ejb
            executeEjb(oForm);
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }

        // Ejb Forward
        String sForward = getEjbForward(oForm, p_Request);
        // Forward to the jsp or the action
        return (p_Mapping.findForward(sForward));
    }

// --------------------------------------------------------- Protected Methods

    /**
     * Dispath executing on Ejb.
     *
     * @param p_Form Current form
     * @throws Exception
     */
    protected void executeEjb(EjbForm p_Form)
        throws Exception {
        // Object name used
        ObjectName on = getEjbObjectName(p_Form);
        switch (m_Type) {
            case EjbItem.EJB_BMP:
            case EjbItem.EJB_CMP:
                if ("synchronize".equals(m_Action) == true) {
                    JonasManagementRepr.invoke(on, "synchronize", null, null);
                }
                else if ("reduceCache".equals(m_Action) == true) {
                    JonasManagementRepr.invoke(on, "reduceCache", null, null);
                }
                break;
            case EjbItem.EJB_SBF:
                break;
            case EjbItem.EJB_SBL:
                if ("reducePool".equals(m_Action) == true) {
                    JonasManagementRepr.invoke(on, "reducePoolToMinSize", null, null);
                }
                break;
            case EjbItem.EJB_MDB:
                break;
        }
    }

    /**
     * The integer type of a Ebj.
     *
     * @param p_Form Current form
     * @return The integer type
     */
    protected int getIntegerType(EjbForm p_Form) {
        int iType = 0;
        String sType = p_Form.getType();
        if (EjbItem.EJB_BMP_STRING.equals(sType) == true) {
            iType = EjbItem.EJB_BMP;
        }
        else if (EjbItem.EJB_CMP_STRING.equals(sType) == true) {
            iType = EjbItem.EJB_CMP;
        }
        else if (EjbItem.EJB_SBF_STRING.equals(sType) == true) {
            iType = EjbItem.EJB_SBF;
        }
        else if (EjbItem.EJB_SBL_STRING.equals(sType) == true) {
            iType = EjbItem.EJB_SBL;
        }
        else if (EjbItem.EJB_MDB_STRING.equals(sType) == true) {
            iType = EjbItem.EJB_MDB;
        }
        return iType;
    }

    /**
     * The global forward to go.
     *
     * @param p_Form The current form
     * @param p_Request The current HTTP request
     * @return Forward
     */
    protected String getEjbForward(EjbForm p_Form, HttpServletRequest p_Request) {
        String sForward = null;
        switch (m_Type) {
            case EjbItem.EJB_BMP:
                sForward = "ActionEditEjbBmp";
                break;
            case EjbItem.EJB_CMP:
                sForward = "ActionEditEjbCmp";
                break;
            case EjbItem.EJB_SBF:
                break;
            case EjbItem.EJB_SBL:
                sForward = "ActionEditEjbSbl";
                break;
            case EjbItem.EJB_MDB:
                break;
        }
        p_Request.setAttribute("type", p_Form.getType());
        p_Request.setAttribute("name", p_Form.getName());
        p_Request.setAttribute("file", p_Form.getFile());
        return sForward;
    }

    /**
     * Return a <code>EjbForm</code> instance associate to the EJB of the session.
     *
     * @return A form instance
     */
    protected EjbForm getEjbForm() {
        return ((EjbForm) m_Session.getAttribute("ejbForm"));
    }

    /**
     * Get a <code>ObjectName</code> instance for the Ejb given.
     *
     * @param p_Form Current form
     * @return The ObjectName instance
     * @throws Exception
     */
    protected ObjectName getEjbObjectName(EjbForm p_Form)
        throws Exception {
        ObjectName on = null;
        switch (m_Type) {
            case EjbItem.EJB_BMP:
                on = JonasObjectName.ejbbmp(p_Form.getFile(), p_Form.getName());
                break;
            case EjbItem.EJB_CMP:
                on = JonasObjectName.ejbcmp(p_Form.getFile(), p_Form.getName());
                break;
            case EjbItem.EJB_SBF:
                on = JonasObjectName.ejbsbf(p_Form.getFile(), p_Form.getName());
                break;
            case EjbItem.EJB_SBL:
                on = JonasObjectName.ejbsbl(p_Form.getFile(), p_Form.getName());
                break;
            case EjbItem.EJB_MDB:
                on = JonasObjectName.ejbmdb(p_Form.getFile(), p_Form.getName());
                break;
        }
        return on;
    }

}
