/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.jtm;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import javax.management.ObjectName;

import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;

import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;

/**
 * Principal action for JTM service management.
 * @author Adriana Danes
 */
public class EditJtmServiceAction extends JonasBaseAction {

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(1) + ";services;transaction", true);

         // Form used
        JtmServiceForm oForm = (JtmServiceForm) p_Form;
        try {
            // Object name used
            ObjectName oObjectName = JonasObjectName.transactionService();

            String location;
            if(getBooleanAttribute(oObjectName, "JTMLocal")) {
                // Collacated JTM
                location = m_Resources.getMessage("tab.jtm.collocated");
                oForm.setJtmLocation(location);
            } else {
                // Distant MOM
                location = m_Resources.getMessage("tab.jtm.distant");
                oForm.setJtmLocation(location);
            }

            String host = (String) getStringAttribute(oObjectName, "JTMHostName");
            oForm.setJtmHost(host);

            Integer port = (Integer) JonasManagementRepr.getAttribute(oObjectName, "JTMPortNumber");
            String portValue = null;
            if (port.intValue() == 0) {
                // Undefined value
                portValue = m_Resources.getMessage("tab.jtm.undefined");
                oForm.setJtmPort(portValue);
            } else {
                oForm.setJtmPort(port.toString());
            }

            int timeOut = getIntegerAttribute(oObjectName, "CurrentTimeOut");
            oForm.setTimeOut(timeOut);

        } catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }

        // Forward to the jsp.
        return (p_Mapping.findForward("Jtm Service"));
    }

}
