/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Delplanque Xavier & Sauthier Guillaume
 * --------------------------------------------------------------------------
 * $Id: WebServicesService.java,v 1.6 2003/09/16 14:15:55 sauthieg Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas.ws;

import org.objectweb.jonas.service.Service;

import org.objectweb.jonas_ws.deployment.api.WSDLFile;

import javax.naming.Context;


/**
 * WebServices Service interface.
 *
 * @author Guillaume Sauthier
 */
public interface WebServicesService extends Service {
    /**
     * explore the context classloader to find Web service deployment
     * descriptor (for endpoints) and service-ref element within standard
     * deployment descriptor (for clients). It registers each endpoints in WS
     * engine publishing relative WSDLs, and it instanciates and binds in the
     * registry each clients classes.
     *
     * @param ctx the context containing the configuration  to deploy the
     *        wars.<BR> This context contains the following parameters :<BR> -
     *        jarUrls the list of the urls of the jars to deploy.<BR> -
     *        warUrls the list of the urls of the wars to deploy.<BR> -
     *        parentClassLoader the parent classLoader of the wars.<BR> -
     *        earClassLoader the ear classLoader of the j2ee app.<BR> - altDDs
     *        the optional URI of deployment descriptor.<BR>
     *
     * @throws WSServiceException if an error occurs during  the deployment.
     */
    void deployWebServices(Context ctx) throws WSServiceException;

    /**
     * DOCUMENT ME!
     */
    void locateWSDL();

    /**
     * DOCUMENT ME!
     *
     * @param wsdl DOCUMENT ME!
     */
    void publishWSDL(WSDLFile wsdl);

    /**
     * Configure the given JaxRpc Service file with the Context  Configuration
     * parameter. Used when a port-component-link has been  specified in
     * service-ref. Its a engine specific process (has to be implemented in 
     * specific WebServicesService).
     *
     * @param service The Service to configure
     * @param ctx The Context where find informations for Context Configuration
     *
     * @return The configured Service
     *
     * @throws WSServiceException When configuration fails.
     */
    javax.xml.rpc.Service configureJaxRpcService(
        javax.xml.rpc.Service service,
        Context               ctx) throws WSServiceException;

    /**
     * Return the Deafult Servlet Name tio use for SSB WebServices.
     *
     * @return the Deafult Servlet Name tio use for SSB WebServices.
     */
    String getDefaultServletName();
}
