/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JStatelessContext.java,v 1.8 2003/09/11 15:44:44 durieuxp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.rmi.RemoteException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.TimerService;
import javax.xml.rpc.handler.MessageContext;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class extends JSessionContext in case of Stateless Session Bean.
 * @author Philippe Durieux
 */
public class JStatelessContext extends JSessionContext {

    /**
     * constructor
     * @param bf The Bean Factory
     * @param sb The Session Bean instance.
     */
    public JStatelessContext(JSessionFactory bf, SessionBean sb) {
        super(bf, sb);
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
    }

    /**
     * Get access to the EJB Timer Service.
     * @return the EJB Timer Service 
     * @throws IllegalStateException Thrown if the instance is not
     * allowed to use this method
     */
    public TimerService getTimerService() throws IllegalStateException {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        return bf.getTimerService();
    }

    /**
     * set this instance as removed
     */
    public void setRemoved() throws RemoteException, RemoveException {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        // Set a flag to finish remove at postInvoke.
        ismarkedremoved = true;
    }

    /**
     * Obtain a reference to the JAX-RPC MessageContext. 
     * @return The MessageContext for this web service invocation. 
     * @throws java.lang.IllegalStateException - the instance is in a state 
     * that does not allow access to this method.
     */
    public MessageContext getMessageContext() throws java.lang.IllegalStateException {
        throw new IllegalStateException("Not Implemented");
    }
}
