/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): S.Chassande_________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id: MapperManager.java,v 1.8 2003/10/14 15:02:18 chassand Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container.jorm;

import org.objectweb.jonas_ejb.container.TraceEjb;
import org.objectweb.jonas_ejb.container.JContainer;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.lib.PTypeSpacePAAH;
import org.objectweb.jorm.lib.Mapper;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.compiler.lib.JormCompiler;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.medor.eval.prefetch.lib.PrefetchCacheImpl;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import javax.naming.Context;

/**
 * This class is an implemention of JOnAS service in order to manage Jorm
 * mappers. This class creates and references also the PTypeSpace and the Meta
 * information manager.
 *
 * @author Sebastien Chassande-Barrioz
 */
public class MapperManager {

    /**
     * The singleton instance of the class
     */
    private static MapperManager singleton = null;

    /**
     * It retrieves the unique instance of MapperManager.
     */
    public static MapperManager getInstance() {
        if (singleton == null) {
            singleton = new MapperManager();
        }
        return singleton;
    }

    /**
     * The Logger used in this class
     */
    private Logger logger = null;

    /**
     * This fields contains the association between a mapper ant its name.
     * key: String mapper name
     * Value: PMapper a mapper instance
     */
    private HashMap mappers = new HashMap();

    /**
     *  Initial Context for Naming
     */
    private Context ictx = null;

    /**
     * permits to configure Jorm and its mappers
     */
    private JormConfigurator jormConfigurator;

    protected MapperManager() {
        logger = TraceEjb.logger;
        JormCompiler jormcompiler = new JormCompiler();
        jormConfigurator = jormcompiler.getCompilerConfigurator();
        Properties prop = new Properties();
        prop.put("jorm.generator", "org.objectweb.jorm.generator.lib.JormGenerator");
        prop.put("jorm.mimanager","org.objectweb.jorm.metainfo.lib.JormManager");
        prop.put("jorm.parser", "org.objectweb.jorm.xml2mi.lib.BasicDomParser");
        prop.put("jorm.verifier", "org.objectweb.jorm.verifier.lib.JormVerifier");
        prop.put("jorm.writer", "org.objectweb.jorm.mi2xml.lib.BasicDomWriter");
        prop.put("jorm.mapper.list", "rdb");
        prop.put("jorm.mapper.mifactory.rdb", "org.objectweb.jorm.mapper.rdb.metainfo.RdbMappingFactory");
        prop.put("jorm.mapper.mopfactory.rdb", "org.objectweb.jorm.mapper.rdb.generator.RdbMOPFactory");
        prop.put("jorm.mapper.gcmapping.rdb", "org.objectweb.jorm.mapper.rdb.genclass.RdbGenClassMapping");
        prop.put("jorm.mapper.schmgr.rdb", "org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager");
        jormConfigurator.configure(prop);
        jormConfigurator.setLoggerFactory(TraceEjb.loggerFactory);
        logger.log(BasicLevel.DEBUG, "JormConfigurator created");
    }

    public PMapper getMapper(String mappername, JContainer c) {
        Map m = (Map) mappers.get(c);
        return (m == null ? null : (Mapper) m.get(mappername));
    }

    public PMapper addMapper(PMapper m, JContainer c) throws PException {
        Map map = (Map) mappers.get(c);
        PMapper pm =  null;
        if (map == null) {
            map = new HashMap();
            mappers.put(c, map);
        } else {
            pm = (PMapper) map.get(m.getMapperName());
        }
        if (pm == null) {
            pm = m;
            String mappername = m.getMapperName();

            //Allocate a new Cache of PreftechBuffer
            m.setPrefetchCache(new PrefetchCacheImpl());
            
            //Add a logger
            if (TraceEjb.loggerFactory == null) {
                m.setLogger(logger);
            } else {
                m.setLogger(TraceEjb.loggerFactory.getLogger(TraceEjb.prefix
                        + ".mapper." + mappername));
                if (m instanceof Loggable) {
                    ((Loggable) m).setLoggerFactory(TraceEjb.loggerFactory);
                }
            }
            // start the mapper
            m.start();

            map.put(mappername, m);
            logger.log(BasicLevel.DEBUG, "Mapper '" + mappername + "' initialized");
        }
        return pm;
    }

    public Logger getLogger() {
        return logger;
    }

    public JormConfigurator getJormConfigurator() {
        return jormConfigurator;
    }
}
