/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id: EjbqlVariableVisitor.java,v 1.16 2003/09/08 09:10:10 joaninh Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.lib;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;

import org.objectweb.jonas_ejb.container.TraceEjb;
import org.objectweb.jonas_ejb.deployment.api.DeploymentCmp2Desc;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTAbstractSchemaName;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionMemberDeclaration;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEJBQL;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFromClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTIdentifier;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTPath;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTRangeVariableDeclaration;
import org.objectweb.jonas_ejb.deployment.ejbql.ParseException;
import org.objectweb.jonas_ejb.deployment.ejbql.SimpleNode;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSelectClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTIdentificationVariable;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedNavigation;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTWhereClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBetweenExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTInExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTLikeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTNullComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEmptyCollectionComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionMemberExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTStringExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTDatetimeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBooleanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEntityBeanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningStrings;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningNumerics;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTOrderByClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTOrderByItem;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.query.jorm.lib.NavigatorNodeFactory;
import org.objectweb.medor.query.lib.QueryTreePrinter;
import org.objectweb.medor.filter.lib.NavigatorOperator;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.util.monolog.api.BasicLevel;


/**
 * Implementation of a visitor that creates a map of pairs [name,QueryTreeField] for all defined
 * identifiers of the query.
 * @author  Christophe Ney  [cney@batisseurs.com]
 * contributor(s);
 * - Helene Joanin: In the visit of the SELECT clause, do not forget to add the id
 *                  in the ids structure if its not present.
 *                  Indeed, the definition of this id is done in the FROM clause which
 *                  is visited later.
 *                  This is needed for ejbSelect query which allows path_expression in
 *                  the SELECT clause.
 * - Helene Joanin: Use of the NavigatorNodeFactory.getIn() for IN in the FROM part
 *                  instead of NavigatorNodeFactory.getNavigatorNode()
 * - Helene Joanin: Some modifications needed to take into account CollectionValuedPathExpression
 *                  for the member of and is empty operators.
 * - Helene Joanin: For the MemberOf implementation, do not more build an unique QueryTree,
 *                  but a Map of pairs [name,QueryTreeField]
 * - Helene Joanin: Add the ORDER BY clause.
 * - Helene Joanin: Take into account the new interface of NavigatorNodeFactory.
 * - Helene Joanin: IS EMPTY / IS NOT EMPTY
 */
public class EjbqlVariableVisitor extends EjbqlAbstractVisitor {

    private static final byte NAVOP_IN = 1;
    private static final byte NAVOP_NAV = 2;
    private static final byte NAVOP_MEMBER_OF = 3;
    private static final byte NAVOP_IS_EMPTY = 4;
    private static final byte NAVOP_IS_NOT_EMPTY = 5;

    private Manager manager;
    private DeploymentCmp2Desc dd;
    private HashMap ids = new HashMap();
    private HashMap fields = new HashMap();

    // Useful to IS EMPTY / IS NOT EMPTY parsing
    private int nbOpNot = 0;

    /**
     * constructor
     * @param ejbql root of the lexical tree
     * @param dd Deployment Descriptor
     */
    public EjbqlVariableVisitor(ASTEJBQL ejbql, DeploymentCmp2Desc dd) throws Exception {
        TraceEjb.query.log(BasicLevel.DEBUG, "");

        QueryTree qt = null;
        this.dd = dd;
        manager = dd.getJormManager();
        visit(ejbql);
        //traceIds("After the ejbql visit");

        // replace map values with JormExtent or Medor Field
        for (Iterator i = ids.keySet().iterator(); i.hasNext();) {
            getQueryTree((String) i.next());
        }
        //traceIds("After iterating with getQueryTree");

        // build the Fields map from the Ids map
        for (Iterator i = ids.keySet().iterator(); i.hasNext();) {
            String id = (String) i.next();
            IdValue idv = (IdValue) ids.get(id);
            // add the ident in the fields list
            Field fField = idv.qt.getTupleStructure().getField(id + "." + Field.PNAMENAME);
            fields.put(id + "." + Field.PNAMENAME, fField);
            // for each path, add it in the fields list
            for (int pi = 0; pi < idv.getDeclaredPathLength(); pi++) {
                String fName = idv.getMergedPath(pi);
                if (!fName.endsWith(".collectionIsEmpty")
                    && !fName.endsWith(".collectionIsNotEmpty")) {
                    fField = idv.qt.getTupleStructure().getField(fName);
                    fields.put(fName, fField);
                }
            }
        }
        // For debug
        /*
        StringBuffer lfn = new StringBuffer("fields names = ");
        for (Iterator i = fields.keySet().iterator(); i.hasNext();) {
            lfn = lfn.append((String) i.next() + ",");
        }
        TraceEjb.query.log(BasicLevel.DEBUG, lfn);
        */
    }

    private QueryTree getQueryTree(String id)
        throws ParseException, MedorException {

        if (TraceEjb.isDebugQuery()) {
            TraceEjb.query.log(BasicLevel.DEBUG, "id=" + id);
        }

        IdValue iv = (IdValue) ids.get(id);
        if (iv == null) {
            throw new ParseException("Abstract schema name \""
                                     + id + "\" has not been declared");
        }
        if (iv.qt != null) {
            if (TraceEjb.isDebugQuery()) {
                TraceEjb.query.log(BasicLevel.DEBUG, "id=" + id);
                QueryTreePrinter.printQueryTree(iv.qt, TraceEjb.query, BasicLevel.DEBUG);
            }
            return iv.qt;
        }
        if (iv.name == null) {
            throw new ParseException("Identifer '" + id
                                     + "' has not been declared");
        }

        if (iv.name.length == 1) {
            // iv.name[0] is the Abstract schema name
            String cn = dd.asn2BeanDesc(iv.name[0]).getJormClassName();
            Class theClass = manager.getClass(cn);
            if (theClass == null) {
                throw new ParseException("Abstract schema name \"" + iv.name[0]
                                         + "\" has not been declared in the jorm meta information");
            }
            iv.qt = new ClassExtent(theClass, id, Field.PNAMENAME, false);
        } else {
            // collection member declaration
            iv.qt = getQueryTree(iv.name[0]);
            iv.qt = getNavNode(iv.qt, iv.name, id, NAVOP_IN);
        }
        for (int i = 0; i < iv.getDeclaredPathLength(); i++) {
            if (!iv.qt.getTupleStructure().contains(iv.getMergedPath(i))) {
                String[] p = iv.getSplitedPath(i);
                String alias = mergePath(p, 0, p.length - 1);
                byte op = NAVOP_NAV;
                if (endsWith(p, NavigatorNodeFactory.GENCLASS_ELEMENT_NAME)) {
                    // in case of collection_valued_path_expression in member
                    // suppress the ".element" which had been added in the path
                    alias = mergePath(p, 0, p.length);
                    p = basePath(p);
                    op = NAVOP_MEMBER_OF;
                } else if (endsWith(p, "collectionIsEmpty")) {
                    // in case of collection_valued_path_expression in is empty
                    alias = mergePath(p, 0, p.length);
                    p = basePath(p);
                    op = NAVOP_IS_EMPTY;
                } else if (endsWith(p, "collectionIsNotEmpty")) {
                    // in case of collection_valued_path_expression in is not empty
                    alias = mergePath(p, 0, p.length);
                    p = basePath(p);
                    op = NAVOP_IS_NOT_EMPTY;
                }
                iv.qt = getNavNode(iv.qt, p, alias, op);
            }
        }
        if (iv.name.length > 1) {
            setQueryTree(iv.name[0], iv.qt);
        }
        if (TraceEjb.isDebugQuery()) {
            TraceEjb.query.log(BasicLevel.DEBUG, "id=" + id);
            QueryTreePrinter.printQueryTree(iv.qt, TraceEjb.query, BasicLevel.DEBUG);
        }
        return iv.qt;
    }

    private QueryTree getNavNode(QueryTree qt, String[] pathName, String alias, byte op)
        throws MedorException {

        if (TraceEjb.isDebugQuery()) {
            TraceEjb.query.log(BasicLevel.DEBUG,
                               "pathName=" + mergePath(pathName)
                               + ", alias=" + alias
                               + ", op=" + op);
        }

        QueryTree rqt = null;

        Field f = qt.getTupleStructure().getField(pathName[0] + "." + Field.PNAMENAME);
        NavigatorOperator nav = new NavigatorOperator(new BasicFieldOperand(f), pathName[1]);
        for (int i = 2; i < pathName.length; i++) {
            nav = new NavigatorOperator(nav, pathName[i]);
        }
        switch (op) {
        case NAVOP_IN:
            rqt = NavigatorNodeFactory.navigate(nav,
                                                alias + "." + Field.PNAMENAME,
                                                NavigatorNodeFactory.NAVIGATION_INTO_LAST
                                                ).getQueryTree();
            break;
        case NAVOP_NAV:
            rqt = NavigatorNodeFactory.navigate(nav, alias, NavigatorNodeFactory.NAVIGATION
                                                 ).getQueryTree();
            break;
        case NAVOP_MEMBER_OF:
            rqt = NavigatorNodeFactory.navigate(nav, alias, NavigatorNodeFactory.NAVIGATION_INTO_LAST
                                                 ).getQueryTree();
            break;
        case NAVOP_IS_EMPTY:
            rqt = NavigatorNodeFactory.navigate(nav, alias, NavigatorNodeFactory.IS_EMPTY
                                                 ).getQueryTree();
            break;
        case NAVOP_IS_NOT_EMPTY:
            rqt = NavigatorNodeFactory.navigate(nav, alias, NavigatorNodeFactory.IS_NOT_EMPTY
                                                 ).getQueryTree();
            break;
        }
        return rqt;
    }

    private void setQueryTree(String id, QueryTree qt) {
        if (TraceEjb.isDebugQuery()) {
            TraceEjb.query.log(BasicLevel.DEBUG, "id=" + id);
        }
        IdValue iv = (IdValue) ids.get(id);
        iv.qt = qt;
        if (iv.name.length > 1) {
            setQueryTree(iv.name[0], iv.qt);
        }
    }

    /**
     * get the abstract schema name or collection values path of the given identifier.
     */
    public String [] getName(String id) {
        IdValue iv = (IdValue) ids.get(id);
        return iv.name;
    }

    /**
     * get the Map that was built from visiting the lexical query tree
     * This map allows to get the org.objectweb.medor.api.Field from its name (ident or path).
     */
    public Map getFields() {
        return fields;
    }

    /**
     * trace the ids structure
     */
    public void traceIds(String msg) {
        if (TraceEjb.isDebugQuery()) {
            TraceEjb.query.log(BasicLevel.DEBUG, msg + ", Begin of IDS structure:");
            // trace the ids structure
            for (Iterator i = ids.keySet().iterator(); i.hasNext();) {
                String id = (String) i.next();
                IdValue idv = (IdValue) ids.get(id);
                TraceEjb.query.log(BasicLevel.DEBUG,
                                   "ids[" + id + "]=" + idv);
                if (idv.qt != null) {
                    QueryTreePrinter.printQueryTree(idv.qt, TraceEjb.query, BasicLevel.DEBUG);
                }
            }
            TraceEjb.query.log(BasicLevel.DEBUG, msg + ", End of IDS structure:");
        }
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTFromClause node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTCollectionMemberDeclaration node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTRangeVariableDeclaration node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTCollectionValuedPathExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * Push the Node to the stack
     */
    public Object visit(ASTAbstractSchemaName node, Object data) {
        ((Stack) data).push(node.value);
        return null;
    }

    /**
     * store the pair identifier,Node from the Stack in HashMap
     */
    public Object visit(ASTIdentifier node, Object data) {
        String id = (String) node.value;
        String name = (String) ((Stack) data).pop();
        IdValue iv = (IdValue) ids.get(id);
        if (iv == null) {
            iv = new IdValue(name);
            ids.put(id, iv);
        } else {
            iv.name = splitPath(name);
        }
        if (TraceEjb.isDebugQuery()) {
            TraceEjb.query.log(BasicLevel.DEBUG, "visit-ASTIdentifier: "
                               + "id=" + id + ",name=" + name + ",iv=" + iv);
        }
        return null;
    }

    /**
     * Push the Node to the stack
     */
    public Object visit(ASTPath node, Object data) {
        ((Stack) data).push(node.value);
        return null;
    }

    private void processVariable(String path) {
        // Common to select, where and orderby clauses
        String id = splitPath(path)[0];
        IdValue iv = (IdValue) ids.get(id);
        if (iv == null) {
            iv = new IdValue();
            ids.put(id, iv);
        }
        iv.addPath(path);
        if (TraceEjb.isDebugQuery()) {
            TraceEjb.query.log(BasicLevel.DEBUG, "path=" + path + ",id=" + id + ",iv=" + iv);
        }
    }

    // -------------- select ---------------//

    public Object visit(ASTSelectClause node, Object data) {
        visit((SimpleNode) node, data);
        Stack st = (Stack) data;
        if (!st.empty()) {
            processVariable((String) st.pop());
        }
        return null;
    }

    public Object visit(ASTIdentificationVariable node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTSingleValuedPathExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    // ----------- Where -----------------//
    public Object visit(ASTWhereClause node, Object data) {
        visit((SimpleNode) node, data);
        Stack st = (Stack) data;
        while (!st.empty()) {
            processVariable((String) st.pop());
        }
        return null;
    }

    public Object visit(ASTSingleValuedNavigation node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTConditionalExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTConditionalTerm node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTConditionalFactor node, Object data) {
        if (node.not) {
            nbOpNot++;
        }
        visit((SimpleNode) node, data);
        if (node.not) {
            nbOpNot--;
        }
        return null;
    }

    public Object visit(ASTBetweenExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTInExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTLikeExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTNullComparisonExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTEmptyCollectionComparisonExpression node, Object data) {
        visit((SimpleNode) node, data);
        Stack st = (Stack) data;
        String path = (String) st.pop();
        String suffix;
        if (node.not) {
            if ((nbOpNot % 2) == 0) {
                suffix = "collectionIsNotEmpty";
            } else {
                suffix = "collectionIsEmpty";
            }
        } else {
            if ((nbOpNot % 2) == 0) {
                suffix = "collectionIsEmpty";
            } else {
                suffix = "collectionIsNotEmpty";
            }
        }
        st.push(path + "." + suffix);
        return null;
    }

    public Object visit(ASTCollectionMemberExpression node, Object data) {
        visit((SimpleNode) node, data);
        Stack st = (Stack) data;
        String path = (String) st.pop();
        st.push(path + "." + NavigatorNodeFactory.GENCLASS_ELEMENT_NAME);
        return null;
    }

    public Object visit(ASTComparisonExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTArithmeticExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTArithmeticTerm node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTArithmeticFactor node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTStringExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTDatetimeExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTBooleanExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTEntityBeanExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTFunctionsReturningStrings node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTFunctionsReturningNumerics node, Object data) {
        return visit((SimpleNode) node, data);
    }

    // ----------- Order by -----------------//
    public Object visit(ASTOrderByClause node, Object data) {
        visit((SimpleNode) node, data);
        Stack st = (Stack) data;
        while (!st.empty()) {
            processVariable((String) st.pop());
        }
        return null;
    }

    public Object visit(ASTOrderByItem node, Object data) {
        return visit((SimpleNode) node, data);
    }

}
