/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: ConfigGenerator.java,v 1.7 2003/10/17 15:55:59 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen;

import java.util.Iterator;

import org.objectweb.jonas_ws.deployment.api.ServiceRef;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.wsgen.jarfile.EjbJarManager;
import org.objectweb.jonas_ws.wsgen.jarfile.WarManager;


/**
 * This interface provides methods used to generate configuration files for a
 * Web services Engine.
 * @author Xavier Delplanque & Guillaume Sauthier
 */
public interface ConfigGenerator {

    // configurator option names.
    public final static String VERBOSE     = "verbose";
    public final static String DEBUG       = "debug";
    public final static String NO_CONFIG   = "no.config";
    public final static String KEEP_GEN    = "keep.gen";
    public final static String PARSING_VAL = "parsing.validation";
    public final static String JAVAC_NAME  = "javac.name";
    public final static String JAVAC_OPTS  = "javac.opts";

    public final static String SERVLET_CLASS   = "servlet.class";
    public final static String SERVLET_NAME    = "servlet.name";
    public final static String DEFAULT_MAPPING = "default.mapping";
    
    // options default values.
    public final static Boolean VERBOSE_DEFAULT   = new Boolean(false);
    public final static Boolean DEBUG_DEFAULT     = new Boolean(false);
    public final static Boolean NO_CONFIG_DEFAULT = new Boolean(false);
    public final static Boolean KEEP_GEN_DEFAULT  = new Boolean(false);
    public final static String JAVAC_NAME_DEFAULT = "javac";

    /**
     * Generate endpoint configuration files for the set of ServiceDesc
     * included in WSDeploymentDesc contained in a WebApp. Common method 
     * for all Web Services implementation (Glue/Axis).
     *
     * @param wm the managed war file.
     * @param dd The DeploymentDesc to process (image of a webservices.xml)
     *
     * @throws WsGenException if generation fails
     */
    public void genWebAppEndpointConfig(WarManager wm,
                                        WSDeploymentDesc dd)
        throws WsGenException;

    /**
     * Generate client configuration files for the list of ServiceRef of a War.
     * Common method for all Web Services implementation (Glue/Axis).
     *
     * @param wm the managed war file.
     * @param servRefs The list of ServiceRef to process.
     *
     * @throws WsGenException if generation fails
     */
    public void genWebAppClientConfig(WarManager wm,
                                      ServiceRef[] servRefs)
        throws WsGenException;

    /**
     * Generate endpoint configuration files for the set of ServiceDesc
     * included in WSDeploymentDesc contained in an EjbJar. Common method 
     * for all Web Services implementation (Glue/Axis).
     *
     * @param wm the war file dispatching request on EJBs endpoint.
     * @param dd The DeploymentDesc to process (image of a webservices.xml)
     *
     * @throws WsGenException if generation fails
     */
    public void genEjbJarEndpointConfig(WarManager wm,
                                        WSDeploymentDesc dd)
        throws WsGenException;

    /**
     * Generate client configuration files for each ServiceRef of bean descs.
     * Common method for all Web Services implementation (Glue/Axis).
     *
     * @param jm the jar file containing clients.
     * @param bdIt an iterator on a bean desc list
     *
     * @throws WsGenException if generation fails
     */
    public void genEjbJarClientConfig(EjbJarManager jm,
                                      Iterator bdIt)
        throws WsGenException;

    /**
     * Remove temp directories.
     */
    public void clean();

}
