/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: EarWsGen.java,v 1.2 2003/10/17 15:55:59 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Vector;

import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDescException;
import org.objectweb.jonas_ws.deployment.lib.WSDeploymentDescManager;
import org.objectweb.jonas_ws.wsgen.jarfile.EarManager;
import org.objectweb.jonas_ws.wsgen.jarfile.EjbJarManager;
import org.objectweb.jonas_ws.wsgen.jarfile.WarManager;

import org.objectweb.jonas_ejb.deployment.api.DeploymentDesc;
import org.objectweb.jonas_ejb.deployment.api.DeploymentDescException;
import org.objectweb.jonas_ejb.deployment.lib.EjbDeploymentDescManager;

import org.objectweb.jonas_web.deployment.api.WebContainerDeploymentDesc;
import org.objectweb.jonas_web.deployment.api.WebContainerDeploymentDescException;
import org.objectweb.jonas_web.deployment.lib.WebDeploymentDescManager;

import org.objectweb.jonas_ear.deployment.api.EarDeploymentDesc;
import org.objectweb.jonas_ear.deployment.api.EarDeploymentDescException;
import org.objectweb.jonas_ear.deployment.xml.Ejb;
import org.objectweb.jonas_ear.deployment.xml.Web;

/**
 * Hold specific process for Ear files.
 *
 * @author Guillaume Sauthier
 */
public class EarWsGen extends WsGen {

    /**
     * Create an EarWsGen
     */
    public EarWsGen(String outputDir,
                    ConfigGenerator cGen,
                    boolean isVerbose,
                    boolean isDebug,
                    String earName)
        throws WsGenException  {

        super(outputDir,
              cGen,
              isVerbose,
              isDebug,
              earName);
    }

    /**
     * Modify the given Ear to add it Webservices files
     */
    public void configure() throws WsGenException {

        URL earURL = getExplodedJar(fileInputName);
        //Create classLoader
        //parent classloader is the current classloader
        ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
        URLClassLoader loaderCls = new URLClassLoader(new URL[] {earURL},
                                                      currentLoader);

        EarDeploymentDesc earDD = null;
        debug("Getting the deployment descriptor of the file" + fileInputName);
        try {
            earDD = EarDeploymentDesc.getInstance(fileInputName, loaderCls);
        } catch (EarDeploymentDescException e) {
            String err = "Error in the Deployment descriptor '"
                + fileInputName + "': " + e;
            error(err);
            fatalError(err);
        }

        //We get the tags from the Deployment descriptor
        Web[] webTags = earDD.getWebTags();
        Ejb[] ejbTags = earDD.getEjbTags();

        //Check if all modules are inside the EAR file
        //no relatives mode like ../../file1.jar
        File fEar = new File(fileInputName);
        File tmpFile = null;

        // list of files
        File[] ejbFiles = new File[ejbTags.length];
        String[] ejbEntryNames = new String[ejbTags.length];
        String[] ejbPaths = new String[ejbTags.length];

        File[] warFiles = new File[webTags.length];
        String[] warEntryNames = new String[webTags.length];
        String[] warPaths = new String[webTags.length];

        try {
            for (int i = 0; i < ejbTags.length; i++) {
                ejbEntryNames[i] = ejbTags[i].getValue();
                ejbFiles[i] = new File(fEar, ejbEntryNames[i]);
                tmpFile = ejbFiles[i].getCanonicalFile();
                ejbPaths[i] = tmpFile.getCanonicalPath();

                if (!tmpFile.getPath().startsWith(fEar.getPath())) {
                    String err =  "Error : The ejb-jar file " 
                        + ejbTags[i].getValue()  + " is not inside the ear file "
                        + fEar;
                    error(err);
                    fatalError(err);
                }
            }

            for (int i = 0; i < webTags.length; i++) {
                warEntryNames[i] = webTags[i].getWebUri().getValue();
                warFiles[i] = new File(fEar, warEntryNames[i]);
                tmpFile = warFiles[i].getCanonicalFile();
                warPaths[i] = tmpFile.getCanonicalPath();

                if (!tmpFile.getPath().startsWith(fEar.getPath())) {
                    String err =  "Error : The war file " 
                        + webTags[i].getWebUri().getValue() 
                        + " is not inside the ear file " + fEar;
                    error(err);
                    fatalError(err);
                }
            }

        } catch (IOException ioe) {
            String err =  "Error while trying to get the canonical file of " + tmpFile;
            error(err + " : " + ioe.getMessage());
            fatalError(err);
        }

        // We have the list of files to process now
        // give them to WsGen instance
        String filename = null;
        try {

            EarManager earm = new EarManager(fileInputName, outputdir);

            // add Webservices files in Ejbs
            for (int i = 0; i < ejbPaths.length; i++) {

                filename = ejbPaths[i];
                configEjbJar(earm, filename);

                // todo : verifier que l ancien jar est bien ecrase
                // overwrite the archive contained in the ear.

                earm.addFile(ejbEntryNames[i], ejbFiles[i]);
            }
            // add Webservices files in WebApps
            for (int i = 0 ; i < warFiles.length; i++) {

                filename = warPaths[i];
                configWebApp(filename);

                // todo : verifier que l ancien jar est bien ecrase
                // overwrite the archive contained in the ear.

                earm.addFile(warEntryNames[i], warFiles[i]);
            }
            earm.close();
        } catch (Exception e) {
            String err = "Error in generation for " + filename;
            fatalError(err);
        }

        // remove the exploded ear
        File tempEAR = new File(earURL.getFile());
        try {
            if (!tempEAR.delete()) {
                String err = "Cannot delete the ear directory '" + tempEAR.getName() + "'";
                fatalError(err);
            }
        } catch (NullPointerException npe) {
            warning("Null File name given '" + tempEAR.getName() + "'");
        } catch (SecurityException se) {
            warning("Security error when deleting '" + tempEAR.getName() + "'");
        }
    }


    /**
     * Modify the given EjbJar (contained in an Ear) to add it Webservices files
     *
     * @param fileInputName the name of the Ejbjar file
     */
    private void configEjbJar(EarManager em, String ejbjarFile) {
        debug("EjbJar Case : " + ejbjarFile);

        URL ejbURL = null;
        try {
            ejbURL = new File(ejbjarFile).toURL();
        } catch (MalformedURLException mue) {
            error(mue.getMessage());
            fatalError(mue);
        }

        //Create classLoader
        //parent classloader is the current classloader
        ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
        URLClassLoader fileCL = new URLClassLoader(new URL[] {ejbURL},
                                                   currentLoader);

        try {

            // Client Files generation.
            // get the web app deployment descriptor.
            DeploymentDesc dd = null;
            try {
                dd = EjbDeploymentDescManager.getDeploymentDesc(ejbjarFile, fileCL);
            } catch (DeploymentDescException dde) {
                throw new WsGenException(dde);
            }

            EjbJarManager jm = new EjbJarManager(ejbjarFile, tmpDir.getPath());

            cg.genEjbJarClientConfig(jm, dd.getBeanDescIterator());

            jm.save();

            // Endpoint files generation.
            // get the webservices deployment descriptor.
            WSDeploymentDesc wsdd = null;
            try {
                wsdd = WSDeploymentDescManager.getDeploymentDesc(ejbjarFile, fileCL);
            } catch (WSDeploymentDescException wsdde) {
                String err = "Error while reading Deployment Desc from " + ejbjarFile;
                throw new WsGenException(wsdde);
            }

            if (wsdd != null) {
                // the ejb may have endpoints.

                // create a war file and add it the WS servlet.
                String warFileName = ejbjarFile.substring(0, ejbjarFile.length() - 3) + "war";
                WarManager wm = new WarManager(warFileName);

                // add webservices files in ejbjar
                cg.genEjbJarEndpointConfig(wm, wsdd);

                wm.save();

                // the war in the ear.
                em.addWar(warFileName);

                // to be removed :

                // remove the war file.
                try {
                    File war = new File(warFileName);
                    if (!war.delete()) {
                        warning("Cannot delete the war '" + warFileName + "'");
                    }
                } catch (NullPointerException npe) {
                    warning("Null File name given '" + warFileName + "'");
                } catch (SecurityException se) {
                    warning("Security error when deleting '" + warFileName + "'");
                }

            }
        } catch (WsGenException wsge) {
            String err = "Error in generation for " + ejbjarFile;
            fatalError(err);
        }
    }

    private void configWebApp(String webappFile) throws WsGenException {
        debug("Webapp Case : " + webappFile);


        URL warURL = getExplodedJar(webappFile);

        Vector urls = new Vector();

        File lib = new File(warURL.getFile() + File.separator + "WEB-INF" + File.separator + "lib");
        File classes = new File(warURL.getFile() + File.separator + "WEB-INF" + File.separator + "classes");
        try {
            if (lib.exists()) {
                File[] childs = lib.listFiles();
                for (int i = 0 ; i < childs.length; i++) {
                    urls.add(childs[i].toURL());
                }
            }
            if (classes.exists()) {
                urls.add(classes.toURL());
            }
        } catch (java.net.MalformedURLException mue) {
            fatalError("Error when building war '" + webappFile + "' class loader");
        }
        urls.add(warURL);

        URL[] aurls = new URL[urls.size()];
        aurls = (URL[]) urls.toArray(aurls);

        //Create classLoader
        //parent classloader is the current classloader
        ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
        URLClassLoader fileCL = new URLClassLoader(aurls,
                                                   currentLoader);

        // Endpoint files generation.
        // get the webservices deployment descriptor.
        WSDeploymentDesc wsdd = null;
        try {
            wsdd = WSDeploymentDescManager.getDeploymentDesc(webappFile, fileCL);
        } catch (WSDeploymentDescException wsdde) {
            String err = "Error while reading Deployment Desc from " + webappFile;
            throw new WsGenException(err, wsdde);
        }


        WarManager wm = new WarManager(webappFile, tmpDir.getPath());

        if (wsdd != null) {
            // the web app may have endpoints.
   
            // add webservices files in war
            cg.genWebAppEndpointConfig(wm, wsdd);

        }

        // Client Files generation.
        // get the web app deployment descriptor. 
        WebContainerDeploymentDesc wcdd = null;
        try {
            wcdd = WebDeploymentDescManager.getDeploymentDesc(webappFile, fileCL);
        } catch (WebContainerDeploymentDescException wcdde) {
            throw new WsGenException(wcdde);
        }

        if (wcdd.getServiceRefTags().length != 0) {
            // the web app may have clients
            // add clients files in war
            cg.genWebAppClientConfig(wm, wcdd.getServiceRefTags());
        }

        wm.save();

        /*
        // remove the exploded ear
        File tempWar = new File(warURL.getFile());
        try {
        if (!tempWar.delete()) {
        String err = "Cannot delete the war directory '" + tempWar.getName() + "'";
        warning(err);
        }
        } catch (NullPointerException npe) {
        warning("Null File name given '" + tempWar.getName() + "'");
        } catch (SecurityException se) {
        warning("Security error when deleting '" + tempWar.getName() + "'");
        }
        */

    }

}
