/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: EjbJarWsGen.java,v 1.2 2003/10/17 15:55:59 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDescException;
import org.objectweb.jonas_ws.deployment.lib.WSDeploymentDescManager;
import org.objectweb.jonas_ws.wsgen.jarfile.EarManager;
import org.objectweb.jonas_ws.wsgen.jarfile.EjbJarManager;
import org.objectweb.jonas_ws.wsgen.jarfile.WarManager;

import org.objectweb.jonas_ejb.deployment.api.DeploymentDesc;
import org.objectweb.jonas_ejb.deployment.api.DeploymentDescException;
import org.objectweb.jonas_ejb.deployment.lib.EjbDeploymentDescManager;

/**
 * Hold specific process for EjbJar files.
 *
 * @author Guillaume Sauthier
 */
public class EjbJarWsGen extends WsGen {

    /**
     * Create an EjbJarWsGen
     */
    public EjbJarWsGen(String outputDir,
                       ConfigGenerator cGen,
                       boolean isVerbose,
                       boolean isDebug,
                       String jarName)
        throws WsGenException {

        super(outputDir,
              cGen,
              isVerbose,
              isDebug,
              jarName);
    }

    /**
     * Modify the given EjbJar to add it Webservices files
     */
    public void configure() throws WsGenException {

        debug("EjbJar Case : " + fileInputName);

        EjbJarManager jm = new EjbJarManager(fileInputName, outputdir);

        URL ejbURL = null;
        try {
            ejbURL = new File(fileInputName).toURL();
        } catch (MalformedURLException mue) {
            error(mue.getMessage());
            fatalError(mue);
        }

        //Create classLoader
        //parent classloader is the current classloader
        ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
        URLClassLoader fileCL = new URLClassLoader(new URL[] {ejbURL},
                                                   currentLoader);

        // Client Files generation.
        // get the ejb jar deployment descriptor.
        DeploymentDesc dd = null;
        try {
            dd = EjbDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
        } catch (DeploymentDescException dde) {
            throw new WsGenException(dde);
        }

        cg.genEjbJarClientConfig(jm, dd.getBeanDescIterator());

        // save the ejbjar with client files
        jm.save();

        // Endpoint files generation.
        // get the webservices deployment descriptor.
        WSDeploymentDesc wsdd = null;

        try {
            wsdd = WSDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
        } catch (WSDeploymentDescException wsdde) {
            wsdde.printStackTrace(System.out);
            String err = "Error while reading Deployment Desc from " + fileInputName;
            throw new WsGenException(err, wsdde);
        }
        if (wsdd != null) {
            // the ejb may have endpoints.

            // create a war file and add it the WS servlet.
            String warFileName = fileInputName.substring(0, fileInputName.length() - 3) + "war";
            WarManager wm = new WarManager(warFileName, outputdir);

            // add webservices files in war
            cg.genEjbJarEndpointConfig(wm, wsdd);

            wm.save();

            // create an ear and add it the jar and war.
            // the ear name is ejbjar one with '.ear' extension.
            String earName = fileInputName.substring(0, fileInputName.length() - 3) + "ear";

            EarManager em = new EarManager(earName, outputdir);
            em.addWar(wm.getOutFilename());
            em.addEjbjar(jm.getOutFilename());
            em.save();

            // remove the war and ejbjar files
            try {
                File war = new File(wm.getOutFilename());
                File ejbjar = new File(jm.getOutFilename());
                if (!war.delete()) {
                    warning("Cannot delete the war '" + wm.getOutFilename() + "'");
                }
                if (!ejbjar.delete()) {
                    warning("Cannot delete the ejbjar '" + jm.getOutFilename() + "'");
                }
            } catch (NullPointerException npe) {
                warning("Null File name given '" + wm.getOutFilename() 
                        + "', '" + jm.getOutFilename() + "'");
            } catch (SecurityException se) {
                warning("Security error when deleting '" + wm.getOutFilename() 
                        + "', '" + jm.getOutFilename() + "'");
            }
        }
    }
 
   
}
