/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: GenerationContext.java,v 1.1 2003/10/16 14:55:52 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ws.wsgen;

import java.util.Iterator;

/**
 * The interface <code>GenerationContext</code> contains properties
 * used along the Generation process.
 *
 * @author Guillaume Sauthier
 */
public interface GenerationContext {

    /** filename property name */
    public static final String FILENAME  = "filename";

    /** classpath property name */
    public static final String CLASSPATH = "classpath";

    /** generation dir property name */
    public static final String GEN_DIR   = "gen.dir";

    /** $JAVA_HOME/bin property name */
    public static final String JAVA_HOME = "java.home.bin";

    /**
     * Set a property for generation.
     * @param name property name
     * @param value property value
     */
    public void setProperty(String name, Object value);


    /**
     * Get an property, return the property value or default
     * if the property isn't set.
     * @param name property name
     * @param defolt property value
     *
     * @return the property value or default
     * if the property isn't set.
     */
    public Object getProperty(String name, Object defolt);

    /**
     * Get an property, return the property value or null
     * if the property isn't set.
     * @param name property name
     *
     * @return the property value or null
     * if the property isn't set.
     */
    public Object getProperty(String name);


    /**
     * Removes a property (name-value pair) from the <code>GenerationContext</code>
     * @param  name Name of the property to be removed
     */
    public void removeProperty(String name);

    /**
     * Returns true if the <code>GenerationContext</code> contains a property
     * with the specified name.
     * @param   name Name of the property whose presense is to be tested
     *
     * @return  Returns true if the <code>GenerationContext</code> contains the
     *  property; otherwise false
     */
    public boolean containsProperty(String name);

    /**
     * Returns an Iterator view of the names of the properties
     * in this <code>GenerationContext</code>
     *
     * @return Iterator for the property names
     */
    public Iterator getPropertyNames();


    /**
     * Helper method returning the processed filename.
     * @return the processed filename.
     */
    public String getFilename();

    /**
     * Helper method returning the classpath.
     * @return the classpath.
     */
    public String getClasspath();

    /**
     * Helper method returning the $JAVA_HOME/bin value.
     * @return the $JAVA_HOME/bin value.
     */
    public String getJavaHomeBin();
}
