/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: WebAppWsGen.java,v 1.2 2003/10/17 15:55:59 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Vector;

import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDescException;
import org.objectweb.jonas_ws.deployment.lib.WSDeploymentDescManager;
import org.objectweb.jonas_ws.wsgen.jarfile.WarManager;

import org.objectweb.jonas_web.deployment.api.WebContainerDeploymentDesc;
import org.objectweb.jonas_web.deployment.api.WebContainerDeploymentDescException;
import org.objectweb.jonas_web.deployment.lib.WebDeploymentDescManager;

/**
 *
 * @author Guillaume Sauthier
 */
public class WebAppWsGen extends WsGen {

    /**
     * Create an WebAppWsGen
     */
    public WebAppWsGen(String outputDir,
                       ConfigGenerator cGen,
                       boolean isVerbose,
                       boolean isDebug,
                       String warName)
        throws WsGenException {

        super(outputDir,
              cGen,
              isVerbose,
              isDebug,
              warName);
    }
 
    /**
     * Modify the given Web App to add it Webservices files
     */
    public void configure() throws WsGenException {
        debug("Webapp Case : " + fileInputName);


        URL warURL = getExplodedJar(fileInputName);

        Vector urls = new Vector();

        File lib = new File(warURL.getFile() + File.separator + "WEB-INF" + File.separator + "lib");
        File classes = new File(warURL.getFile() + File.separator + "WEB-INF" + File.separator + "classes");
        try {
            if (lib.exists()) {
                File[] childs = lib.listFiles();
                for (int i = 0 ; i < childs.length; i++) {
                    urls.add(childs[i].toURL());
                }
            }
            if (classes.exists()) {
                urls.add(classes.toURL());
            }
        } catch (java.net.MalformedURLException mue) {
            fatalError("Error when building war '" + fileInputName + "' class loader");
        }
        urls.add(warURL);

        URL[] aurls = new URL[urls.size()];
        aurls = (URL[]) urls.toArray(aurls);

        //Create classLoader
        //parent classloader is the current classloader
        ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
        URLClassLoader fileCL = new URLClassLoader(aurls,
                                                   currentLoader);

        // Endpoint files generation.
        // get the webservices deployment descriptor.
        WSDeploymentDesc wsdd = null;
        try {
            wsdd = WSDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
        } catch (WSDeploymentDescException wsdde) {
            String err = "Error while reading Deployment Desc from " + fileInputName;
            throw new WsGenException(err, wsdde);
        }

        WarManager wm = new WarManager(fileInputName, outputdir);

        if (wsdd != null) {
            // the web app may have endpoints.
            // add webservices files in war
            cg.genWebAppEndpointConfig(wm, wsdd);

        }

        // Client Files generation.
        // get the web app deployment descriptor. 
        WebContainerDeploymentDesc wcdd = null;
        try {
            wcdd = WebDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
        } catch (WebContainerDeploymentDescException wcdde) {
            throw new WsGenException(wcdde);
        }

        if (wcdd.getServiceRefTags().length != 0) {
            // the web app may have clients
            // add clients files in war
            cg.genWebAppClientConfig(wm, wcdd.getServiceRefTags());
        }

        wm.save();

        // To be removed when ClassLoaders will be not necessary ...

        // remove the exploded war
        File tempWar = new File(warURL.getFile());
        try {
            if (!tempWar.delete()) {
                String err = "Cannot delete the war unpack directory '" + tempWar.getName() + "'";
                warning(err);
            }
        } catch (NullPointerException npe) {
            warning("Null File name given '" + tempWar.getName() + "'");
        } catch (SecurityException se) {
            warning("Security error when deleting '" + tempWar.getName() + "'");
        }
    }

}
