/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Guillaume Sauthier
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id: WsGenException.java,v 1.3 2003/08/26 12:18:49 delplanx Exp $
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ws.wsgen;

/**
 * This class represents the exception that can be throwned by the WsGen tool.
 */
public class WsGenException extends Exception {

    protected Exception inner = null;

    /**
     * Constructs a WsGenException with no specified detail message.
     */
    public WsGenException() {
        super();
    }

    /**
     * Constructs an WsGenException with the specified detail message.
     */
    public WsGenException(String msg) {
        super(msg);
    }

    public WsGenException(Exception inner) {
	super();
	this.inner = inner;
    }

    public WsGenException(String msg, Exception inner) {
        super(msg);
        this.inner = inner;

    }

    public String toString() {
        String s = WsGenException.class.getName() + ": " + super.getMessage();
        if (inner == null) {
            return (s);
        } else {
            return (s + " (" + inner.toString() + ")");
        }
    }

    public String getMessage() {
        String s = super.getMessage();
        if (inner == null) {
            return (s);
        } else {
            return (s + " (" + inner.getMessage() + ")");
        }
    }

    public void printStackTrace() {
        System.out.println(getMessage());
        if (inner != null) {
            inner.printStackTrace();
        }
    }
    public void printStackTrace(java.io.PrintStream s) {
        s.println(getMessage());
        if (inner != null) {
            inner.printStackTrace(s);
        }
    }
    public void printStackTrace(java.io.PrintWriter s) {
        s.println(getMessage());
        if (inner != null) {
            inner.printStackTrace(s);
        }
    }
}
