/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque
 * --------------------------------------------------------------------------
 * $Id: VContextFactory.java,v 1.6 2003/10/16 14:55:51 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.axis;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.objectweb.jonas_ws.deployment.api.HandlerRef;
import org.objectweb.jonas_ws.deployment.api.PortComponentDesc;
import org.objectweb.jonas_ws.deployment.api.ServiceDesc;
import org.objectweb.jonas_ws.deployment.api.ServiceRef;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.wsgen.WsGenException;

import org.apache.velocity.VelocityContext;

/**
 * This class allows to create the Velocity Context used to build the generated
 * sources with the Velocity templates.
 *
 * @author Xavier Delplanque (Bull)
 */
public class VContextFactory {

    /**
     * Creates the Velocity Context used to build the generated
     * files with the Velocity templates (used for EjbJar).
     *
     * @param dd The bean Deployment Descriptor
     *
     * @return a VelocityContext customized with the DD
     */
    public static VelocityContext createEndpointCtx(WSDeploymentDesc dd) {

        VelocityContext vc = new VelocityContext();

        // set portComponents
        Vector pcds = new Vector();
        List sds = dd.getServiceDescs();

        System.out.println("sds.size() : " + sds.size());
        // build a list containing all webservice port component
        for (Iterator itSd = sds.iterator(); itSd.hasNext();) {
            ServiceDesc sd = (ServiceDesc) itSd.next();
            pcds.addAll(sd.getPortComponents());

        }

        Vector portComponents = new Vector();
        for (Iterator it = pcds.iterator(); it.hasNext();) {
            System.out.println("PortComponentDesc");
            PortComponentDesc pcd = (PortComponentDesc) it.next();
            portComponents.add(new VcPortComponent(pcd));
        }
        vc.put("portComponents", portComponents);

        // set provider
        PortComponentDesc pcd0 = (PortComponentDesc) pcds.get(0);
        if (pcd0.hasBeanImpl()) {
            vc.put("provider", "EJB");
        } else if (pcd0.hasJaxRpcImpl()) {
            vc.put("provider", "RPC");
        }
        return vc;
    }

    /**
     * Creates the Velocity Context used to build the generated
     * files with the Velocity templates (used for WebApp).
     *
     * @param pcd The Port Component describing endpoint to expose.
     *
     * @return a VelocityContext customized with the PCD
     */
    public static VelocityContext createEndpointCtx(PortComponentDesc pcd) {

        VelocityContext vc = new VelocityContext();

        Vector portComponents = new Vector();
        portComponents.add(new VcPortComponent(pcd));
  
        vc.put("portComponents", portComponents);

        // set provider
        if (pcd.hasBeanImpl()) {
            vc.put("provider", "EJB");
        } else if (pcd.hasJaxRpcImpl()) {
            vc.put("provider", "RPC");
        }

        return vc;
    }

    /**
     * Creates the Velocity Context used to build the generated
     * files with the Velocity templates.
     *
     * @param sr The ServiceRef Deployment Descriptor
     *
     * @return a VelocityContext customized with the SR
     */
    static VelocityContext createClientCtx(ServiceRef sr) {

        VelocityContext vc = new VelocityContext();

        Hashtable pcds = new Hashtable();

        // for each service-ref handler
        List hrs = sr.getHandlerRefs();

        Vector commonh = new Vector();
        for (Iterator itHr = hrs.iterator(); itHr.hasNext();) {
            HandlerRef hr = (HandlerRef) itHr.next();
            List pcns = hr.getPortNames();

            if (pcns.size() != 0) {
                // a port name list is defined : add the handler to each specified port
                for (Iterator itPcn = pcns.iterator(); itPcn.hasNext();) {
                    String pcn = (String) itPcn.next();
                    if (!pcds.containsKey(pcn)) {
                        pcds.put(pcn, new Vector());
                    }
                    ((Vector) pcds.get(pcn)).add(hr);
                }
            } else {
                // no port name is specified : the handler is added to each port
                commonh.add(hr);
            }
        }

        // now, we can create the port component list

        Vector portComponents = new Vector();
        for (Enumeration enPc = pcds.keys(); enPc.hasMoreElements();) {
            String pcn = (String) enPc.nextElement();
            // add common handlers to the list
            Vector pchrs = (Vector) pcds.get(pcn);
            pchrs.addAll(commonh);
            portComponents.add(new VcPortComponent(pcn, pchrs));
        }

        vc.put("portComponents", portComponents);
        return vc;
    }
}
