/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque
 * --------------------------------------------------------------------------
 * $Id: VcBean.java,v 1.2 2003/10/16 14:55:51 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.axis;

import org.objectweb.jonas_ejb.deployment.api.BeanDesc;

/**
 * Member of a VelocityContext. Contains information about a bean.
 *
 * @author Xavier Delplanque
 */
public class VcBean {

    /** the bean jndi name */
    private String jndiName;

    /** true if the bean is local */
    private boolean local;
 
    /** the local home interface name */
    private String localHomeInterface;

    /** the local interface name */
    private String localInterface;
 
    /** true if the bean is remote */
    private boolean remote;

    /** the bean home interface name */
    private String homeInterface;

    /** the bean remote interface name */
    private String remoteInterface;

    public VcBean(BeanDesc bd) {

        // set jndiName
        jndiName = bd.getJndiName();

        // set local
        local = bd.getLocalClass() != null;

        if (local) {
            // set localHomeInterface
            localHomeInterface = bd.getLocalHomeClass().getName();

            // set localInterface
            localInterface = bd.getLocalClass().getName();
        }

        // set remote
        remote = bd.getRemoteClass() != null;
        if (remote) {
            // set homeInterface
            homeInterface = bd.getHomeClass().getName();

            // set remoteInterface
            remoteInterface = bd.getRemoteClass().getName();
        }
    }
 
    /**
     * The bean jndi name
     * @return The bean jndi name
     */
    public String getJndiName () {
        return jndiName;
    }

    /**
     * true if the bean is local
     * @return true if the bean is local
     */
    public boolean isLocal () {
        return local;
    }
 
    /**
     * the local home interface name
     * @return the local home interface name 
     * or null if the bean is not local
     */
    public String getLocalHomeInterface () {
        return localHomeInterface;
    }

    /**
     * the local interface name
     * @return the local interface name 
     * or null if the bean is not local
     */
    public String getLocalInterface () {
        return localInterface;
    }
 
    /**
     * true if the bean is remote
     * @return true if the bean is remote
     */
    public boolean isRemote () {
        return remote;
    }

    /**
     * the bean home interface name
     * @return the bean home interface name
     *  or null if the bean is not remote
     */
    public String getHomeInterface () {
        return homeInterface;
    }

    /**
     * the bean remote interface name
     * @return the bean remote interface name
     *  or null if the bean is not remote
     */
    public String getRemoteInterface () {
        return remoteInterface;
    } 
}
