/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque
 * --------------------------------------------------------------------------
 * $Id: VcHandler.java,v 1.3 2003/10/16 14:55:51 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.axis;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.objectweb.jonas_ws.deployment.api.HandlerDesc;
import org.objectweb.jonas_ws.deployment.api.HandlerRef;

/**
 * Member of a VelocityContext. Contains information about a Handler (Ref/Desc).
 *
 * @author Xavier Delplanque
 */
public class VcHandler {
    
    /** handler name (unique) */
    private String name;

    /** handler class name */
    private String className;

    /** handler init parameters */
    private Vector initParameters;

    /** handler SOAP headers */
    private Vector headers;

    /** handler SOAP roles */
    private Vector soapRoles;
    
    /**
     * Construct a VcHandler from a HandlerDesc.
     */
    public VcHandler(HandlerDesc hd) {

        // set name
        name = hd.getName();

        //set className
        className = hd.getHandlerClass().getName();

        // set initParameters
        Hashtable ips = hd.getInitParams();
        initParameters = new Vector();
        for (Enumeration itKey = ips.keys(); itKey.hasMoreElements();) {
            String key = (String) itKey.nextElement();
            String val = (String) ips.get(key);
            initParameters.add(new VcInitParam(key, val));
        }

        // set headers
        List hs = hd.getSOAPHeaders();
        headers = new Vector();
        for (Iterator itH = hs.iterator(); itH.hasNext();) {
            QName qn = (QName) itH.next();
            headers.add(new VcHeader(qn));
        }

        // set soapRoles
        soapRoles = new Vector();
        List srs = hd.getSOAPRoles();
        for (Iterator itSr = srs.iterator(); itSr.hasNext();) {
            String sr = (String) itSr.next();
            soapRoles.add(sr);
        }
    }

    /**
     * Create a VcHandler from a HandlerRef
     */
    public VcHandler(HandlerRef hr) {

        // set name
        name = hr.getName();

        //set className
        className = hr.getHandlerClass().getName();

        // set initParameters
        Hashtable ips = hr.getInitParams();
        initParameters = new Vector();
        for (Enumeration itKey = ips.keys(); itKey.hasMoreElements();) {
            String key = (String) itKey.nextElement();
            String val = (String) ips.get(key);
            initParameters.add(new VcInitParam(key, val));
        }

        // set headers
        List hs = hr.getSOAPHeaders();
        headers = new Vector();
        for (Iterator itH = hs.iterator(); itH.hasNext();) {
            QName qn = (QName) itH.next();
            headers.add(new VcHeader(qn));
        }

        // set soapRoles
        soapRoles = new Vector();
        List srs = hr.getSOAPRoles();
        for (Iterator itSr = srs.iterator(); itSr.hasNext();) {
            String sr = (String) itSr.next();
            soapRoles.add(sr);
        }
    }

    public String getName () {
        return name;
    }

    public String getClassName () {
        return className;
    }

    public Vector getInitParameters () {
        return initParameters;
    }

    public Vector getHeaders () {
        return headers;
    }

    public Vector getSoapRoles () {
        return soapRoles;
    }

}
