/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque
 * --------------------------------------------------------------------------
 * $Id: VcPortComponent.java,v 1.3 2003/10/16 14:55:51 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.axis;

import org.objectweb.jonas_ws.deployment.api.PortComponentDesc;
import org.objectweb.jonas_ws.deployment.api.HandlerDesc;
import org.objectweb.jonas_ws.deployment.api.HandlerRef;

import java.lang.reflect.Method;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * Member of a VelocityContext. Contains information about a PortComponent(Desc/Ref).
 *
 * @author Xavier Delplanque
 */
public class VcPortComponent {

    /** port component name */
    private String name;

    /** the sib if the endpoint is an ejb */
    private VcBean bean;
 
    /** a string containing sei method names separated by ',' */
    private static final String SEP = ",";

    /** a list of method names */
    private String methods;
 
    /** port component handlers */
    private Vector handlers = new Vector();

    /** the JAXRPC class name if the sib is a servlet */
    private String jaxRpcClassName;

    /**
     * Construct a VcPortComponent from a PortComponentDesc.
     */
    public VcPortComponent(PortComponentDesc pcd) {

        // set name
        name = pcd.getName();

        // set bean or jaxRpcClassName
        if (pcd.hasBeanImpl()) {
            // set bean
            bean = new VcBean(pcd.getBeanDesc());
        } else if (pcd.hasJaxRpcImpl()) {
            // set jaxRpcClassName
            jaxRpcClassName = pcd.getServiceImplementationBean().getName();
        }

        // set methods
        Class sei = pcd.getServiceEndpointInterface();
        Method[] ms = sei.getMethods();
        methods = "";
        for (int i = 0; i < ms.length; i++) {
            methods += ms[i].getName();
            if (i != ms.length) {
                methods += SEP;
            }
        }

        // set handlers
        List hs = pcd.getHandlers();
        for (Iterator itH = hs.iterator(); itH.hasNext();) {
            HandlerDesc hd = (HandlerDesc) itH.next();
            handlers.add(new VcHandler(hd));
        }

    }

    /**
     * Construct a VcPortComponent from a PortComponentDesc.
     */
    public VcPortComponent(String name, List hrs) {

        // set name
        this.name = name;

        // set handlers
        for (Iterator itH = hrs.iterator(); itH.hasNext();) {
            HandlerRef hr = (HandlerRef) itH.next();
            handlers.add(new VcHandler(hr));
        }
    }
 
    public String getName () {
        return name;
    }
 
    public VcBean getBean () {
        return bean;
    }
 
    public String getMethods () {
        return methods;
    }
 
    public Vector getHandlers () {
        return handlers;
    }
 
    public String getJaxRpcClassName () {
        return jaxRpcClassName;
    }
}
