/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: WsJarManager.java,v 1.2 2003/10/17 15:56:00 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.jarfile;

//import java

import org.objectweb.jonas_ws.wsgen.WsGenException;

/**
 * Use xml classes to get and write
 * service-ref specific information inside a jar file.
 * @author Guillaume Sauthier
 */
public abstract class WsJarManager extends JarManager {

    /**
     * Build a new instance of a EarManager.
     *
     * @param jarFilename the name of the ear file
     * that we want to manage.
     *
     * @throws WsGenException when instanciation fails.
     */
    public WsJarManager(String jarFilename) throws WsGenException {
        super(jarFilename);
    }

    /**
     * Build a new instance of a WsWarManager.
     *
     * @param jarFilename the name of the jar file we want to manage.
     * @param dir output directory
     *
     * @throws WsGenException when instanciation fails.
     */
    public WsJarManager(String jarFilename, String dir) throws WsGenException {
        super(jarFilename, dir);
    }


    /**
     * Add a jonas-init-param for a specified service-ref
     *
     * @param name the bean name where put init-param (unused for servlet clients)
     * @param serviceRefName the service-ref-name (key)
     * @param initParamName the param name
     * @param initParamValue the param value
     *
     * @throws WsGenException
     */
    public abstract void addJServiceRefInitParam(String name,
                                                 String serviceRefName,
                                                 String initParamName,
                                                 String initParamValue)
        throws WsGenException;

}
