/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: ApplicationXml.java,v 1.3 2003/10/17 15:56:00 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.xml;

import java.io.InputStream;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Wrapping class for managing an application.xml file.
 *
 * @author Guillaume Sauthier
 */
public class ApplicationXml extends Xml {

    private static final String APP_TAG = "application";
    private static final String MODULE_TAG = "module";

    private static final String WEB_TAG = "web";
    private static final String URI_TAG = "web-uri";
    private static final String CTX_TAG = "context-root";

    private static final String EJB_TAG = "ejb";
 
    /**
     * Build ApplicationXml Object wrapping xml file of the InputStream.
     * @param is InputStream of an application.xml file
     * @throws XmlException When creation fails.
     */
    public ApplicationXml (InputStream is) throws XmlException {
        super(is);
    }

    /**
     * Clean application.xml from modules.
     * @throws XmlException when cleaning fails.
     */
    public void removeModules()
        throws XmlException {
        try {
            Document core = getDocument();
            Element root = core.getDocumentElement();
            NodeList nl = root.getElementsByTagName(MODULE_TAG);
            for (int i = 0; i < nl.getLength(); i++) {
                root.removeChild(nl.item(i));
            }
        } catch (Exception e) {
            throw new XmlException("Error when remove module nodes", e);
        }
    }

    /**
     * Add module web in application.xml
     *
     * @param webName webapp filename
     * @param ctx context-root value
     */
    public void addWeb(String webName, String ctx) {

        Document core = getDocument();
        Element root = core.getDocumentElement();

        // build nodes
        /*
          <module>
           <web>
            <web-uri>#webName#</web-uri>
            <context-root>#ctx#</context-root>
           </web>
          </module>
        */
        Node uriTxt = core.createTextNode(webName);
        Node ctxTxt = core.createTextNode(ctx);
        Element uriE = core.createElement(URI_TAG);
        Element ctxE = core.createElement(CTX_TAG);
        Element webE = core.createElement(WEB_TAG);
        Element newNode = core.createElement(MODULE_TAG);

        uriE.appendChild(uriTxt);
        ctxE.appendChild(ctxTxt);
        webE.appendChild(uriE);
        webE.appendChild(ctxE);
        newNode.appendChild(webE);
        root.appendChild(newNode);
    }


    /**
     * Add module ejb in application.xml
     *
     * @param ejbName ejb-jar filename
     */
    public void addEjb(String ejbName) {

        Document core = getDocument();
        Element root = core.getDocumentElement();

        // build nodes
        /*
          <module>
           <ejb>#ejbName#</ejb>
          </module>
        */
        Node ejbNameTxt = core.createTextNode(ejbName);
        Element ejbNameE = core.createElement(EJB_TAG);
        Element newNode = core.createElement(MODULE_TAG);

        ejbNameE.appendChild(ejbNameTxt);
        newNode.appendChild(ejbNameE);
        root.appendChild(newNode);
    }

}
