/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: JonasWebXml.java,v 1.2 2003/10/17 15:56:00 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.xml;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;

import java.io.InputStream;

/**
 * Wrapper class for managing jonas-web.xml file.
 * @author Guillaume Sauthier
 */
public class JonasWebXml extends JonasSpecificXml {

    /**
     * Build JonasWebXml Object wrapping xml file of the InputStream.
     * @param is InputStream of a jonas-web.xml file
     * @throws XmlException When creation fails.
     */
    public JonasWebXml(InputStream is) throws XmlException {
        super(is);
    }

    /**
     * Set the implementing service class.
     * @param serviceRefName service-ref-name value
     * @param serviceClassName the service class implementing service-interface
     */
    public void setServiceClass(String serviceRefName, String serviceClassName) {

        // find the jonas-service-ref
        Element jsr = findJonasServiceRef(serviceRefName);

        // set the service class for this element
        setServiceClass(jsr, serviceClassName);
    }

    /**
     * Add param name/value for the specified jonas-service-ref
     * @param serviceRefName name of the jonas-service-ref (key)
     * @param paramName name of the param
     * @param paramValue value of the param
     */
    public void addJonasInitParam(String serviceRefName,
                                  String paramName,
                                  String paramValue) {
        
        // find the jonas-service-ref
        Element jsr = findJonasServiceRef(serviceRefName);
        
        // add the param in this element
        addJonasInitParam(jsr, paramName, paramValue);
    }

    /**
     * Return the specified jonas-service-ref element.
     * @param serviceRefName name of the jonas-service-ref (key)
     * @return the jonas-service-ref element with specified name
     */
    private Element findJonasServiceRef(String serviceRefName) {
        Document core = getDocument();
        Element root = core.getDocumentElement();
        
        // find the jonas-service-ref element
        return getJonasServiceRef(root, serviceRefName);
    }

    /**
     * return true is file is empty
     * @return true if the xml is Empty
     */
    public boolean isEmpty() {
        Document core = getDocument();
        Element root = core.getDocumentElement();

        // return true if there is no sur elements after jonas-web-app
        return (root.getChildNodes().getLength() == 0);
    }


}
