/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: F_EtypeOsqldateEC2.java,v 1.1 2003/06/25 16:25:55 camillej Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.rmi.RemoteException;
import java.util.Collection;
import java.sql.Date;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.etype.osqldate.Osqldate;
import org.objectweb.jonas.jtests.beans.etype.osqldate.OsqldateHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This is a test suite about java.sql.Date field of entity bean CMP version 2.
 * Beans used: etype/osqldate
 * @author Jerome Camilleri
 */

public class F_EtypeOsqldateEC2 extends A_EtypeOsqldate {

    private static String BEAN_HOME = "EtypeOsqldateEC2Home";
    protected static OsqldateHome home = null;
    private static final long ONE_HOUR = 60L*60L*1000L;
    private static final long ONE_DAY = 24L*ONE_HOUR;

    public F_EtypeOsqldateEC2(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
    }

    public OsqldateHome getHome() {
	if (home == null) {
	    try {
		home = (OsqldateHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME),
								   OsqldateHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home: "+BEAN_HOME);
	    }
	    // init here tables if CMP2
	    try {
		home.create("pk1", new Date(ONE_DAY));
		home.create("pk2", new Date(2*ONE_DAY));
		home.create("pk3", new Date(3*ONE_DAY));
		home.create("pk4", new Date(4*ONE_DAY));
		home.create("pk5", new Date(5*ONE_DAY));
		home.create("pk5bis", new Date(5*ONE_DAY));
		home.create("pktoremove", new Date(12*ONE_DAY));
		home.create("pknull", (Date) null);
		home.create("pkchangenull", new Date(12*ONE_DAY));
	    } catch (Exception i) {
	    }
	}
	return home;
    }

    public static Test suite() {
	return new TestSuite(F_EtypeOsqldateEC2.class);
    }

    /**
     * findByPrimaryKey() test
     */
    public void testCaution() throws Exception {
        fail("Column for java.sql.Date is not of date type but bythearray (serializable).");
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_EtypeOsqldateEC2(testtorun));
	}
    }
}
