/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: A_Timer.java,v 1.1 2003/09/17 08:56:40 durieuxp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.timer;


import java.rmi.RemoteException;
import java.util.Enumeration;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.CreateException;
import javax.rmi.PortableRemoteObject;
import javax.transaction.RollbackException;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.SystemException;
import javax.transaction.NotSupportedException;
import org.objectweb.jonas.jtests.beans.transacted.Simple;
import org.objectweb.jonas.jtests.beans.transacted.SimpleEHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This is test of the TimerService.
 * tests here are common to entity and session beans.
 * beans used : transacted
 * @author Philippe Durieux (jonas team)
 */
public abstract class A_Timer extends JTestCase {

    /**
     * constructor
     * @param name name of the test suite.
     */
    public A_Timer(String name) {
        super(name);
    }

    /**
     * Sets up the fixture, here load the beans if not loaded yet.
     * This method is called before a test is executed.
     */
    protected void setUp() {
        super.setUp();
        useBeans("transacted", true);
    }

    /**
     * Get an instance of the bean.
     * This method depends on the home used to get it.
     * For entity bean, the arg is used to get a particular instance.
     * For session beans, we get any session bean from the pool.
     */
    public abstract Simple getSimple(int i) throws Exception;

    // --------------------------------------------------------------------
    // test cases
    // --------------------------------------------------------------------

    /**
     * Test simple timer
     */
    public void testTimer1() throws Exception {
        int duration = 5;
        Simple s = getSimple(10);
        int oldval = s.getTimerCount();
        int id = s.setTimer(duration, 0);
        assertEquals("timer expired too quickly", oldval, s.getTimerCount());
        sleep(1000 * (duration + 1));
        assertEquals("timer did not expired", oldval + 1, s.getTimerCount());
        s.remove();
    }

    /**
     * test getTimeRemaining on a Timer
     */
    public void testTimeRemaining() throws Exception {
        int duration1 = 5000;
        int duration2 = 2000;
        Simple s = getSimple(11);
        int id1 = s.setTimer(duration1, 0);
        int id2 = s.setTimer(duration2, 0);
        long t1 = s.getTimeRemaining(id1) / 1000;
        long t2 = s.getTimeRemaining(id2) / 1000;
        sleep(1000);
        assertTrue(t1 < duration1);
        assertTrue(t2 < duration2);
        assertTrue(t1 > duration1 - 2);
        assertTrue(t2 > duration2 - 2);
        s.remove();
    }

    /**
     * Test du cancel Timer
     */
    public void testCancel1() throws Exception {
        int duration = 2;
        Simple s = getSimple(12);
        int oldval = s.getTimerCount();
        int id = s.setTimer(duration, 0);
        assertEquals("timer expired too quickly", oldval, s.getTimerCount());
        s.cancelTimer(id);
        sleep(1000 * (duration + 1));
        assertEquals("timer did expired", oldval, s.getTimerCount());
        s.remove();
    }

}
