/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Florent BENOIT
 * --------------------------------------------------------------------------
 * $Id: F_WsEarSample.java,v 1.1 2003/10/09 13:03:39 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.examples.clients.webservices;


import com.meterware.httpunit.*;
import com.meterware.servletunit.*;
import java.io.File;
import java.io.StringWriter;
import java.io.PrintWriter;
import junit.framework.*;
import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.examples.util.JExampleTestCase;

import javax.wsdl.Definition;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.extensions.soap.SOAPAddress;

import javax.xml.namespace.QName;

/**
 * Define a class to test the webServices examples
 * Test WebServices deployment OK and test Call on deployed WebService
 * @author Guillaume Sauthier
 */
public class F_WsEarSample extends A_WebServicesEndpoint {

    /**
     * URL of the wsEarSample page
     */
    private static final String URL_WSEARSAMPLE = "/wsEarSample/";

    /**
     * Main method
     * @param args the arguments
     */
    public static void main(String[] args) {

        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String sArg = args[argn];
            if (sArg.equals("-n")) {
                testtorun = args[++argn];
            }
        }

        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_WsEarSample(testtorun));
        }
    }

    /**
     * Get a new TestSuite for this class
     * @return a new TestSuite for this class
     */
    public static TestSuite suite() {
        return new TestSuite(F_WsEarSample.class);
    }

    /**
     * Setup need for these tests
     * earsample is required
     * @throws Exception if it fails
     */
    protected void setUp() throws Exception {
        super.setUp();
        useEar("wsEarSample");
    }

    /**
     * Constructor with a specified name
     * @param s name
     */
    public F_WsEarSample(String s) {
        super(s, URL_WSEARSAMPLE);
    }

    /**
     * Check if home page is accessible
     * @throws Exception if an error occurs
     */
    public void testDeploymentCompleted() throws Exception {

        // 4 links on homepage
        checkDeploymentCompleted(4);
    }

    /**
     * Check if Axis is Running
     * @throws Exception if an error occurs
     */
    public void testAxisServicesAccessible() throws Exception {
        
        // links to /services/* in first place (0)
        checkAxisServicesAccessible(0, "ssbEndpoint");
    }

    /**
     * Check if WSDL is generated
     * @throws Exception if an error occurs
     */
    public void testAxisWSDL() throws Exception {
        
        // for wsEarSample axis geenrated WSDL link is in second place (1)
        checkAxisWSDL(1);
    }

    /**
     * Check if deployed WebService is running
     * @throws Exception if an error occurs
     */
    public void testDeployedWebService() throws Exception {
        
        checkDeployedWebService("wsBean.html",
                                url + "services/ssbEndpoint",
                                "JOnAS",
                                "JOnAS SSB WebService Test Page");
    }

    /**
     * Check if EJB can access external google WebService
     * @throws Exception if an error occurs
     */
    public void testWebServiceEjbAccess() throws Exception {

        WebResponse wr = executeGoogleWebService("googleByEJB.jsp",
                                                 "ObjectWeb JOnAS");
        assertTrue("a list is returned", wr.getLinks().length > 0);
    }

    /**
     * Check if WSDL has been published
     * @throws Exception if an error occurs
     */
    public void testWSDLPublication() throws Exception {

        checkWSDLPublication("ssbEndpoint.wsdl",
                             "http://ws.beans.wssample.objectweb.org",
                             "WSBeanEndpointLocalService",
                             "WSBeanEndpointLocal",
                             url + "services/ssbEndpoint");
    }


}
