/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: F_ABEjbDII.java,v 1.2 2003/06/06 12:32:18 joaninh Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.ws.clients.jaxrpc;

import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Vector;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;

import javax.ejb.CreateException;

import org.objectweb.ws.ab_client_ejb.gen.Address;
import org.objectweb.ws.ab_client_ejb.beans.ab_diiclt.AddressBookClientDIIHome;
import org.objectweb.ws.ab_client_ejb.beans.ab_diiclt.AddressBookClientDII;

public class F_ABEjbDII extends A_ABEjbClient {

    private static String BEAN_HOME = "AddressBookClientDIIHome";
    private static AddressBookClientDIIHome bHome = null;
    private static AddressBookClientDII client = null;
    
    public F_ABEjbDII(String name) {
        super(name);
    }
    
    /**
     * init environment:
     * - load beans
     * - create/init database for entities.
     */
    protected void setUp() {
        super.setUp();
        if (bHome == null) {
            useBeans("ab_diiclt");
            try {
                bHome = (AddressBookClientDIIHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), AddressBookClientDIIHome.class);
                client = bHome.create();
                client.setEndpointURL(new java.net.URL("http://localhost:"+System.getProperty("http.port")+"/addressbook/services/AddressBookService"));
            } catch (NamingException e) {
                fail("Cannot get AddressBookClientDIIHome:"+e);
            }
            catch (CreateException ce) {
                fail("Cannot create AddressBookClientDII:"+ce);
            }
            catch (RemoteException re) {
                fail("remote AddressBookClientDII fail:"+re);
            }
            catch (java.net.MalformedURLException mue) {
                fail("not a valid URL : "+ mue);
            }
        }
    }

    protected void tearDown() throws Exception {
        unloadBeans("ab_diiclt");
        bHome = null;
        client = null;
        super.tearDown();
    }

    /**
     * 
     */
    public void testGetAddress() throws Exception {
        try {
            Address a1 = client.getAddress("Xavier Delplanque");
            checkGetAddressReturn(a1);
        }
        catch(RemoteException re) {
            fail("getAddress failed with : "+ re);
        }
    }

    public void testCountAddresses() throws Exception {
        try {
            int ac = client.countAddresses();
            checkCountAddressesReturn(ac);
        }
        catch(RemoteException re) {
            fail("countAddresses failed with : "+ re);
        }
    }

    public void testGetOwner() throws Exception {
        try {
            String owner = client.getOwner();
            checkGetOwnerReturn(owner);
        }
        catch(RemoteException re) {
            fail("getOwner failed with : "+ re);
        }
    }

    public void testAddEntry() throws Exception {
        try {
            Address a2 = new Address();
            a2.setName("JOnAS");
            a2.setCity("Echirolles");
            a2.setZip(38100);
            a2.setPhone("0476291526");
            String ret = client.addEntry(a2);
            checkAddEntryReturn("JOnAS", ret);
        }
        catch(RemoteException re) {
            fail("addEntry failed with : "+ re);
        }
    }
    
    public void testGetAllAddresses() throws Exception {
        try {
            Address[] adds = client.getAllAddresses();
            checkGetAllAddressesReturn(adds);
        }
        catch(RemoteException re) {
            fail("getAllAddresses failed with : "+ re);
        }
    }
    
 
    public static Test suite() {
        return new TestSuite(F_ABEjbDII.class);
        
    }
    
    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_ABEjbDII(testtorun));
        }
    }
}
