/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: F_OrderingClassDii.java,v 1.1 2003/05/27 12:16:50 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.ws.clients.ssb;

import org.objectweb.ws.ordering_client.OrderDiiClient;

import java.rmi.RemoteException;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.ws.ordering_client.gen.Order;

public class F_OrderingClassDii extends A_OrderingClassTest {

    private static OrderDiiClient client = null;

    public F_OrderingClassDii(String name) {
        super(name);
    }
    
    /**
     * init environment:
     * - load beans
     * - create/init database for entities.
     */
    protected void setUp() {
        super.setUp();
        client = new OrderDiiClient("http://localhost:"+System.getProperty("http.port")+"/order/services/OrderService");
    }

    protected void tearDown() throws Exception {
        client = null;
        super.tearDown();
    }

    /**
     * 
     */
    public void testGetCustomerOrder() throws Exception {
        try {
            Order res = client.getCustomerOrder("Xavier Delplanque");
            checkGetCustomerOrderReturn(res);
        }
        catch(RemoteException re) {
            fail("getCustomerOrder failed with : "+ re);
        }
    }

    /**
     * 
     */
    public void testProcessOrder() throws Exception {
        try {
            Order order = new Order();
            order.setCustomerName("Glen Daniels");
            order.setShippingAddress("275 Grove Street, Newton, MA");
            String [] items = new String[] { "mp3jukebox", "1600mahBattery" };
            int [] quantities = new int [] { 1, 4 };
            order.setItemCodes(items);
            order.setQuantities(quantities);

            String res = client.processOrder(order);
            checkProcessOrderReturn(res);
        }
        catch(RemoteException re) {
            fail("processOrder failed with : "+ re);
        }
    }

    /**
     * 
     */
    public static Test suite() {
        return new TestSuite(F_OrderingClassDii.class);   
    }
    
    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_OrderingClassDii(testtorun));
        }
    }
}
