/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: F_OrderingEjbDii.java,v 1.2 2003/06/06 12:32:18 joaninh Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.ws.clients.ssb;

import java.rmi.RemoteException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;

import javax.ejb.CreateException;

import org.objectweb.ws.ordering_client_ejb.gen.Order;
import org.objectweb.ws.ordering_client_ejb.beans.order_dii_c.OrderDiiClient;
import org.objectweb.ws.ordering_client_ejb.beans.order_dii_c.OrderDiiClientHome;

public class F_OrderingEjbDii extends A_OrderingEjbTest {

    private static String BEAN_HOME = "OrderDiiClientHome";
    private static OrderDiiClientHome bHome = null;
    private static OrderDiiClient client = null;
    
    public F_OrderingEjbDii(String name) {
        super(name);
    }
    
    /**
     * init environment:
     * - load beans
     * - create/init database for entities.
     */
    protected void setUp() {
        super.setUp();
        if (bHome == null) {
            useBeans("order_dii_c");
            try {
                bHome = (OrderDiiClientHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), OrderDiiClientHome.class);
                client = bHome.create();
                java.net.URL endpointURL = new java.net.URL("http://localhost:"+System.getProperty("http.port")+"/order/services/OrderService");
                client.setEndpointUrl(endpointURL);
            } 
            catch (NamingException e) {
                fail("Cannot get OrderDiiClientHome:"+e);
            }
            catch (CreateException ce) {
                fail("Cannot create OrderDiiClient:"+ce);
            }
            catch (RemoteException re) {
                fail("remote OrderDiiClient fail:"+re);
            }
            catch(java.net.MalformedURLException e) {
                fail("MalformedURLException:"+e);
            }
        }
    }

    protected void tearDown() throws Exception {
        unloadBeans("order_dii_c");
        super.tearDown();
    }

    /**
     * 
     */
    public void testGetCustomerOrder() throws Exception {
        try {
            Order res = client.getCustomerOrder("Xavier Delplanque");
            checkGetCustomerOrderReturn(res);
        }
        catch(RemoteException re) {
            fail("getCustomerOrder failed with : "+ re);
        }
    }

    /**
     * test YYY  fails
     */
    public void testProcessOrder() throws Exception {
        try {
            Order order = new Order();
            order.setCustomerName("Glen Daniels");
            order.setShippingAddress("275 Grove Street, Newton, MA");
            String [] items = new String[] { "mp3jukebox", "1600mahBattery" };
            int [] quantities = new int [] { 1, 4 };
            order.setItemCodes(items);
            order.setQuantities(quantities);

            String res = client.processOrder(order);
            checkProcessOrderReturn(res);
        }
        catch(RemoteException re) {
            fail("processOrder failed with : "+ re);
        }
    }

    public static Test suite() {
        return new TestSuite(F_OrderingEjbDii.class);   
    }
    
    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_OrderingEjbDii(testtorun));
        }
    }
}
