// OrderClientClient.java
// mini Client for accessing bean OrderClient

package org.objectweb.ws.ordering_client;

import java.rmi.RemoteException;
import java.util.Collection;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException; 
import javax.rmi.PortableRemoteObject;

import org.objectweb.ws.ordering_client.gen.Order;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.AxisFault;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;

import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;
/**
 *
 */
public class OrderDiiClient{

    private java.net.URL endpointURL = null;
    
    public OrderDiiClient(String endpointUrl){
        try {
            endpointURL = new java.net.URL(endpointUrl);
        } catch(java.net.MalformedURLException e) {
            e.printStackTrace();
            System.exit(3);
        }    
    }

    public static void main(String[] arg) {
        OrderDiiClient client = new OrderDiiClient("http://localhost:"+arg[0]+"/order/services/OrderService");
        try {            
            // Test the process of an Order
            System.out.println("______ process order test ______");        
            Order order = new Order();
            order.setCustomerName("Glen Daniels");
            order.setShippingAddress("275 Grove Street, Newton, MA");
            String [] items = new String[] { "mp3jukebox", "1600mahBattery" };
            int [] quantities = new int [] { 1, 4 };
            order.setItemCodes(items);
            order.setQuantities(quantities);
            
            System.out.println(client.processOrder(order));
            
            // Test to get the last customer order
            System.out.println("______ get customer order test ______");
            Order orderR = client.getCustomerOrder("Xavier Delplanque");
            
            String sep = System.getProperty("line.separator");
            String response = "Hi, " + orderR.getCustomerName() + "!" + sep;
            response += sep + "You're last order is the following:" + sep;
            
            String [] orderItems = orderR.getItemCodes();
            int [] orderQuantities = orderR.getQuantities();
            
            for (int i = 0; i < orderItems.length; i++) {
                response += sep + orderQuantities[i] + " of item : " + orderItems[i];
            }
            
            System.out.println(response);
            
            // ...
            
        } catch(Exception e) {
            e.printStackTrace();
            System.exit(2);
        }
        // ...
    }
    public String processOrder (Order order) throws java.rmi.RemoteException{
        Service  service = new Service();
        Call     call    = null;
        QName    qn      = new QName( "http://order.beans.ordering_app.ws.objectweb.org", "Order" );
        String result;

        try{
            call = (Call) service.createCall();
            call.registerTypeMapping(Order.class, qn,
                                 new BeanSerializerFactory(Order.class, qn),
                                 new BeanDeserializerFactory(Order.class, qn)); 
        }catch(javax.xml.rpc.ServiceException e ){

        }
        try { 
            call.setTargetEndpointAddress( endpointURL );
            call.setOperationName( new QName("OrderService", "processOrder") );

            result = (String) call.invoke( new Object[] { order } );
        } catch (AxisFault fault) {
            result = "Error : " + fault.toString();
        }
        return result;

    }
    public Order getCustomerOrder (String customer) throws java.rmi.RemoteException{
        Service  service = new Service();
        Call     call    = null;
        QName    qn      = new QName( "http://order.beans.ordering_app.ws.objectweb.org", "Order" );

        Order result = null;

        try{
            call = (Call) service.createCall();
            call.registerTypeMapping(Order.class, qn,
                                 new BeanSerializerFactory(Order.class, qn),
                                 new BeanDeserializerFactory(Order.class, qn)); 
        }catch(javax.xml.rpc.ServiceException e ){
            
        }
        try {
            call.setTargetEndpointAddress( endpointURL );
            call.setOperationName( new QName("OrderService", "getCustomerOrder") );

            result = (Order) call.invoke( new Object[] { customer } );
        } catch (AxisFault fault) {
            //result = "Error : " + fault.toString();
        }
        return result;
    }
}
