/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JTestCase.java,v 1.1 2003/06/06 12:32:17 joaninh Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.ws.util;

import java.rmi.RemoteException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.UserTransaction;
import junit.framework.TestCase;
import org.objectweb.jonas.adm.AdmInterface;

import org.objectweb.jonas.ear.EarServiceException;
import org.objectweb.jonas.web.JWebContainerServiceException;

/**
 * JTestCase extends TestCase to provide a set of standard routines
 * used in jonas tests.
 */
public abstract class JTestCase extends TestCase {

    protected static String jonasName = "EJB"; // change this XXX
    protected static String testtorun = null;
    protected static Context ictx = null;
    public static UserTransaction utx = null;
    private static AdmInterface admI = null;

    protected StringBuffer msgerror = null;

    public JTestCase(String name) {
        super(name);
    }

    private Context getInitialContext() throws NamingException {

        String registryPort = System.getProperty("jonas.registryport");
        return new InitialContext();
    }

    /**
     * common setUp routine, used for every test.
     */
    protected void setUp() {
        try {
            // get InitialContext
            if (ictx == null) {
                ictx = getInitialContext();
            }
            // get UserTransaction
            if (utx == null) {
                utx = (UserTransaction) ictx.lookup("javax.transaction.UserTransaction");
            }
        } catch (NamingException e) {
            System.err.println("Cannot setup test: " + e);
            e.printStackTrace();
        }
        debug("Junit Running " + getName());
    }

    protected void tearDown() throws Exception {
    }

    /**
     * load a bean jar file in the jonas server
     * @param filename jar file, without ".jar" extension
     */
    public void useBeans(String filename) {
        debug("useBeans " + filename);
        try {
            // Load bean in JOnAS if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            if (!admI.isLoaded(filename + ".jar")) {
                admI.addBeans(filename + ".jar");
            }
        } catch (Exception e) {
            System.err.println("Cannot load bean: " + e);
        }
    }

    /**
     * unload a bean
     * @param filename jar file, without ".jar" extension
     */
    public void unloadBeans(String filename) {
        debug("unloadBeans " + filename);
        try {
            // Load bean in EJBServer if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            admI.removeBeans(filename + ".jar");
        } catch (Exception e) {
            System.err.println("Cannot unload bean: " + e);
        }
    }


    /**
     * load a webapp war file in the jonas server
     * @param filename war file, without ".war" extension
     */
    public void useWebapp(String filename) {
        debug("useWebapp " + filename);
        try {
            // Load webapp in JOnAS if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            try {
                admI.addWar(filename + ".war");
            } catch (JWebContainerServiceException e) {
                // Perhaps already loaded
            }
        } catch (Exception e) {
            System.err.println("Cannot load webapp: " + e);
        }
    }

    /**
     * unload a War file
     * @param filename the War file name (without ".war")
     */
    public void unloadWebapp(String filename) {
        debug("unloadWebapp " + filename);
        try {
            // unLoad webapp from JOnAS
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            try {
                admI.removeWar(filename + ".war");
            } catch (JWebContainerServiceException e) {
                System.err.println("Cannot unload webapp: " + e);
            }
        } catch (Exception e) {
            System.err.println("Cannot unload webapp: " + e);
        }
    }

    /**
     * load a application ear file in the jonas server
     * @param filename ear file, without ".ear" extension
     */
    public void useApplication(String filename) {
        debug("useApplication " + filename);
        try {
            // Load application in JOnAS if not already loaded.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            try {
                admI.addEar(filename + ".ear");
            } catch (EarServiceException e) {
                // Perhaps already loaded
            }
        } catch (Exception e) {
            System.err.println("Cannot load application: " + e);
        }
    }

    /**
     * unload a Ear file
     * @param filename the Ear file name (without ".ear")
     */
    public void unloadApplication(String filename) {
        debug("unloadApplication " + filename);
        try {
            // unLoad application in JOnAS.
            if (ictx == null) {
                ictx = getInitialContext();
            }
            if (admI == null) {
                admI = (AdmInterface) ictx.lookup(jonasName + "_Adm");
            }
            try {
                admI.removeEar(filename + ".ear");
            } catch (EarServiceException e) {
                System.err.println("Cannot unload application: " + e);
            }
        } catch (Exception e) {
            System.err.println("Cannot unload application: " + e);
        }
    }


    /**
     * for debugging : Uncomment the line
     * @param msg message
     */
    public static void debug(String msg) {
        // System.out.println(msg);
    }

}
