package bank;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * @author S.Chassande-Barrioz
 */
public class Client {
    protected String lastName;
    protected String firstName;
    protected String address;
    protected Collection accounts;
    protected Agency agency;

    public Client() {
    }

    public Client(String lastName,
                  String firstName,
                  String address,
                  Agency agency) {
        this.lastName = lastName;
        this.firstName = firstName;
        this.address = address;
        this.agency = agency;
        this.accounts = new ArrayList();
    }

    private Account getAccount(String number) {
        Iterator it = accounts.iterator();
        while(it.hasNext()) {
            Account a = (Account) it.next();
            if (a.number.equals(number)) {
                return a;
            }
        }
        return null;
    }

    public synchronized Account createAccount(String number) {
        Account a = getAccount(number);
        if (a == null) {
            a = new Account(number, this, 0);
            a.client = this;
            accounts.add(a);
        }
        return a;
    }

    public synchronized void removeAccount(Account a) {
        if (accounts.remove(a)) {
            a.client = null;
        }
    }

    public synchronized void removeAllAccounts() {
        Iterator it = accounts.iterator();
        while(it.hasNext()) {
            Account a = (Account) it.next();
            a.client = null;
        }
        accounts.clear();
    }

}
