/*
 * @(#) SecurityInterceptor.java	1.0 02/07/15
 *
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * 
 * JONAS
 *
 * This library is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * 
 *
 */
package org.objectweb.jonas.security.interceptors.iiop;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.omg.CORBA.LocalObject;
import org.objectweb.security.context.SecurityContext;


public abstract class SecurityInterceptor extends LocalObject {

    /**
     * security context id
     */
    public static int SEC_CTX_ID = 101;

    /**
     * Custom UTF8 marshalling SecurityContext
     * The resulting bute array is composed of the following elements:
     * principal-name, roles-number, role1, ...., runas-number, runas1, ....
     * @return byte [] the marshalled context
     * @param ctx SecurityContext
     */
    public byte [] marshallSecurityContext(SecurityContext ctx) {
        byte [] result = null;
        try {
            if (ctx != null) {
                // build a new byte from ctx
                ArrayList cResult = new ArrayList();
                // add the pname
                addBytes(cResult, ctx.getCallerPrincipal().getName());
                // add the roles
                String [] roles = ctx.getCallerPrincipalRoles();
                int nbRoles = (roles == null) ? 0 : roles.length;
                addBytes(cResult, new Integer(nbRoles).toString());
                for (int i = 0; i < nbRoles; i++) {
                    addBytes(cResult, roles[i]);
                }
                // add runas
                addBytes(cResult, new Integer(ctx.getRunAsStack().size()).toString());
                Iterator iRunAs = ctx.getRunAsStack().iterator();
                while (iRunAs.hasNext()) {
                    addBytes(cResult, (String) iRunAs.next());
                }
                // result casting
                result = new byte[cResult.size()];
                for (int i = 0; i < cResult.size(); i++) {
                    result[i] = ((Byte) cResult.get(i)).byteValue();
                }
            }

        } catch (java.io.UnsupportedEncodingException uee) {
            uee.printStackTrace();
        }
        return result;
    }

    /**
     * Custom UTF8 marshalling SecurityContext 
     * @param byteCtx the marshalled context
     * @return SecurityContext
     */
    public SecurityContext unmarshallSecurityContext(byte [] byteCtx) {
        SecurityContext result = null;
        if ((byteCtx != null) && (byteCtx.length > 0)) {
            try {
                Iterator istrs = getBytes2Strings(byteCtx).iterator();
                String principalName = (String) istrs.next();
                int nbRoles = new Integer((String) istrs.next()).intValue();
                ArrayList roles = null;
                if (nbRoles > 0) {
                    roles = new ArrayList();
                }
                while (nbRoles > 0) {
                    roles.add((String) istrs.next());
                    nbRoles--;
                }
                int nbRunas = new Integer((String) istrs.next()).intValue();
                ArrayList runas = null;
                if (nbRunas > 0) {
                    runas = new ArrayList();
                }
                while (nbRunas > 0) {
                    runas.add((String) istrs.next());
                    nbRunas--;
                }
                result = new SecurityContext(principalName, roles, runas);
            } catch (java.io.UnsupportedEncodingException uee) {
                uee.printStackTrace();
            }
        }
        return result;
    }

    /**
     * Add to the collection of a byte[], the given string.
     * (its lengths, following by its value)
     * @param c collection of byte[]
     * @param toAdd string to transform to a byte[] and to add to the collection
     * @throws java.io.UnsupportedEncodingException
     */
    private void addBytes(Collection c, String toAdd) throws java.io.UnsupportedEncodingException {
        byte [] b = toAdd.getBytes("UTF8");
        c.add(new Byte((byte) b.length));
        for (int i = 0; i < b.length; i++) {
            c.add(new Byte(b[i]));
        }
    }

    /**
     * Transform a bytes array to a Strin array.
     * (The bytes array contains for each string, its length, following by its value).
     * @param bytes byte array to transform
     * @return the array list of string
     * @throws java.io.UnsupportedEncodingException
     */
    private ArrayList getBytes2Strings(byte[] bytes) throws java.io.UnsupportedEncodingException {
        ArrayList strs = new ArrayList();
        int index = 0;
        while (bytes.length > index) {
            int rSize = bytes[index];
            index++;
            byte[] rName = new byte [rSize];
            for (int j = 0; j < rSize; j++) {
                rName[j] = bytes[index];
                index++;
            }
            strs.add(new String(rName, "UTF8"));
        }
        return strs;
    }

    public String contextString(SecurityContext scx) {
        return (scx.toString());
    }
    
}

